@@ -1,5 +1,5 @@
 // This file is a part of the OpenSurgSim project.
-// Copyright 2013, SimQuest Solutions Inc.
+// Copyright 2013-2015, SimQuest Solutions Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -17,51 +17,24 @@
 #include <string>
 
 #include "Examples/Stapling/StaplerBehavior.h"
-#include "SurgSim/Blocks/KeyboardTogglesComponentBehavior.h"
-#include "SurgSim/Blocks/TransferPhysicsToGraphicsMeshBehavior.h"
-#include "SurgSim/Blocks/VisualizeContactsBehavior.h"
-#include "SurgSim/Collision/ShapeCollisionRepresentation.h"
+#include "SurgSim/Blocks/Blocks.h"
+#include "SurgSim/Collision/Collision.h"
+#include "SurgSim/DataStructures/DataStructures.h"
 #include "SurgSim/Devices/IdentityPoseDevice/IdentityPoseDevice.h"
 #include "SurgSim/Devices/Keyboard/KeyCode.h"
 #include "SurgSim/Devices/MultiAxis/MultiAxisDevice.h"
-#include "SurgSim/Framework/ApplicationData.h"
-#include "SurgSim/Framework/BasicSceneElement.h"
-#include "SurgSim/Framework/BehaviorManager.h"
-#include "SurgSim/Framework/Runtime.h"
-#include "SurgSim/Framework/Scene.h"
-#include "SurgSim/Graphics/Camera.h"
-#include "SurgSim/Graphics/Mesh.h"
-#include "SurgSim/Graphics/MeshRepresentation.h"
-#include "SurgSim/Graphics/OsgLight.h"
-#include "SurgSim/Graphics/OsgManager.h"
-#include "SurgSim/Graphics/OsgMaterial.h"
-#include "SurgSim/Graphics/OsgMeshRepresentation.h"
-#include "SurgSim/Graphics/OsgSceneryRepresentation.h"
-#include "SurgSim/Graphics/OsgView.h"
-#include "SurgSim/Graphics/OsgViewElement.h"
-#include "SurgSim/Graphics/OsgShader.h"
-#include "SurgSim/Graphics/OsgTexture2d.h"
-#include "SurgSim/Graphics/OsgUniform.h"
-#include "SurgSim/Input/InputComponent.h"
-#include "SurgSim/Input/InputManager.h"
-#include "SurgSim/Math/MeshShape.h"
-#include "SurgSim/Math/RigidTransform.h"
-#include "SurgSim/Physics/DeformableCollisionRepresentation.h"
-#include "SurgSim/Physics/Fem3DRepresentation.h"
-#include "SurgSim/Physics/FixedRepresentation.h"
-#include "SurgSim/Physics/RigidCollisionRepresentation.h"
-#include "SurgSim/Physics/RigidRepresentation.h"
-#include "SurgSim/Physics/PhysicsManager.h"
-#include "SurgSim/Physics/VirtualToolCoupler.h"
-#include "SurgSim/DataStructures/PlyReader.h"
-#include "SurgSim/Graphics/MeshPlyReaderDelegate.h"
+#include "SurgSim/Framework/Framework.h"
+#include "SurgSim/Graphics/Graphics.h"
+#include "SurgSim/Input/Input.h"
+#include "SurgSim/Math/Math.h"
+#include "SurgSim/Physics/Physics.h"
 
 using SurgSim::Blocks::KeyboardTogglesComponentBehavior;
 using SurgSim::Blocks::TransferPhysicsToGraphicsMeshBehavior;
 using SurgSim::Blocks::VisualizeContactsBehavior;
 using SurgSim::Collision::ShapeCollisionRepresentation;
-using SurgSim::Device::IdentityPoseDevice;
-using SurgSim::Device::MultiAxisDevice;
+using SurgSim::Devices::IdentityPoseDevice;
+using SurgSim::Devices::MultiAxisDevice;
 using SurgSim::Framework::BasicSceneElement;
 using SurgSim::Framework::BehaviorManager;
 using SurgSim::Framework::Runtime;
@@ -96,15 +69,17 @@ static std::shared_ptr<SurgSim::Framework::SceneElement> createFemSceneElement(
 	const std::string& name,
 	const std::string& filename,
 	SurgSim::Math::IntegrationScheme integrationScheme,
+	SurgSim::Math::LinearSolver linearSolver,
 	std::shared_ptr<SurgSim::Graphics::OsgMaterial> material)
 {
 	// Create a SceneElement that bundles the pieces associated with the finite element model
 	std::shared_ptr<SceneElement> sceneElement = std::make_shared<BasicSceneElement>(name);
 
-	// Set the file name which contains the tetrahedral mesh. File will be loaded by 'doInitialize()' call.
+	// Load the tetrahedral mesh.
 	std::shared_ptr<Fem3DRepresentation> physicsRepresentation = std::make_shared<Fem3DRepresentation>("Physics");
-	physicsRepresentation->setFilename(filename);
+	physicsRepresentation->loadFem(filename);
 	physicsRepresentation->setIntegrationScheme(integrationScheme);
+	physicsRepresentation->setLinearSolver(linearSolver);
 	sceneElement->addComponent(physicsRepresentation);
 
 	// Load the surface triangle mesh of the finite element model
@@ -113,7 +88,7 @@ static std::shared_ptr<SurgSim::Framework::SceneElement> createFemSceneElement(
 
 	// Create a triangle mesh for visualizing the surface of the finite element model
 	auto graphicalFem = std::make_shared<OsgMeshRepresentation>("Graphics");
-	graphicalFem->setFilename(filename);
+	graphicalFem->loadMesh(filename);
 	sceneElement->addComponent(graphicalFem);
 
 	// Create material to transport the Textures
@@ -135,7 +110,7 @@ static std::shared_ptr<SurgSim::Framework::SceneElement> createFemSceneElement(
 	// WireFrame of the finite element model
 	std::shared_ptr<SurgSim::Graphics::MeshRepresentation> wireFrameFem
 		= std::make_shared<SurgSim::Graphics::OsgMeshRepresentation>("Wire Frame");
-	wireFrameFem->setFilename(filename);
+	wireFrameFem->setShape(meshShape);
 	wireFrameFem->setDrawAsWireFrame(true);
 	wireFrameFem->setLocalActive(false);
 	sceneElement->addComponent(wireFrameFem);
@@ -156,7 +131,7 @@ static std::shared_ptr<SurgSim::Framework::SceneElement> createFemSceneElement(
 std::shared_ptr<SceneryRepresentation> createSceneryObject(const std::string& name, const std::string& fileName)
 {
 	std::shared_ptr<SceneryRepresentation> sceneryRepresentation = std::make_shared<OsgSceneryRepresentation>(name);
-	sceneryRepresentation->setFileName(fileName);
+	sceneryRepresentation->loadModel(fileName);
 	return sceneryRepresentation;
 }
 
@@ -170,7 +145,7 @@ std::shared_ptr<SceneElement> createStaplerSceneElement(const std::string& stapl
 
 	std::shared_ptr<MeshRepresentation> meshShapeVisualization =
 		std::make_shared<OsgMeshRepresentation>("Collision Mesh");
-	meshShapeVisualization->setFilename(filename);
+	meshShapeVisualization->setShape(meshShapeForCollision);
 	meshShapeVisualization->setDrawAsWireFrame(true);
 	meshShapeVisualization->setLocalActive(false);
 
@@ -202,7 +177,7 @@ std::shared_ptr<SceneElement> createStaplerSceneElement(const std::string& stapl
 
 	std::shared_ptr<VisualizeContactsBehavior> visualizeContactsBehavior =
 		std::make_shared<VisualizeContactsBehavior>("Contacts");
-	visualizeContactsBehavior->setCollisionRepresentation(collisionRepresentation);
+	visualizeContactsBehavior->setSource(collisionRepresentation);
 	// Note: Since usually the penetration depth of a collision is so small (at the magnitude of mm),
 	// if we use the depth as the length of vector, the vector field will be too small to be seen on the screen.
 	// Thus, we enlarge the vector field by 200 times.
@@ -241,6 +216,7 @@ std::shared_ptr<SceneElement> createStaplerSceneElement(const std::string& stapl
 			std::make_shared<ShapeCollisionRepresentation>("VirtualToothCollision" + std::to_string(i));
 		virtualToothCollision->setShape(*it);
 		virtualToothCollision->setLocalPose(RigidTransform3d::Identity());
+		virtualToothCollision->ignore(collisionRepresentation);
 
 		virtualTeeth[i] = virtualToothCollision;
 		sceneElement->addComponent(virtualToothCollision);
@@ -272,7 +248,7 @@ std::shared_ptr<SceneElement> createArmSceneElement(
 	// Visualization of arm collision mesh
 	std::shared_ptr<MeshRepresentation> meshShapeVisualization =
 		std::make_shared<OsgMeshRepresentation>("Collision Mesh");
-	meshShapeVisualization->setFilename(filename);
+	meshShapeVisualization->setShape(meshShape);
 	meshShapeVisualization->setDrawAsWireFrame(true);
 	meshShapeVisualization->setLocalActive(false);
 
@@ -281,6 +257,7 @@ std::shared_ptr<SceneElement> createArmSceneElement(
 
 	std::shared_ptr<RigidCollisionRepresentation> collisionRepresentation =
 		std::make_shared<RigidCollisionRepresentation>("Collision");
+	collisionRepresentation->ignore("wound/Collision");
 	physicsRepresentation->setCollisionRepresentation(collisionRepresentation);
 
 	std::shared_ptr<SceneElement> armSceneElement = std::make_shared<BasicSceneElement>(armName);
@@ -294,19 +271,6 @@ std::shared_ptr<SceneElement> createArmSceneElement(
 	return armSceneElement;
 }
 
-template <typename Type>
-std::shared_ptr<Type> getComponentChecked(std::shared_ptr<SurgSim::Framework::SceneElement> sceneElement,
-		const std::string& name)
-{
-	std::shared_ptr<SurgSim::Framework::Component> component = sceneElement->getComponent(name);
-	SURGSIM_ASSERT(component != nullptr) << "Failed to get Component named '" << name << "'.";
-
-	std::shared_ptr<Type> result = std::dynamic_pointer_cast<Type>(component);
-	SURGSIM_ASSERT(result != nullptr) << "Failed to convert Component to requested type.";
-
-	return result;
-}
-
 std::shared_ptr<OsgViewElement> createViewElement()
 {
 	auto result = std::make_shared<OsgViewElement>("StaplingDemoView");
@@ -326,13 +290,13 @@ std::shared_ptr<OsgViewElement> createViewElement()
 
 std::shared_ptr<SurgSim::Graphics::OsgMaterial> createShinyMaterial(
 	const SurgSim::Framework::ApplicationData& data,
-	std::shared_ptr<SurgSim::Graphics::OsgShader> shader,
+	std::shared_ptr<SurgSim::Graphics::OsgProgram> program,
 	std::string defaultTextureName = "Textures/checkered.png")
 {
 	// Default Material with shader
 	// using scopes to keep from having to introduce new variables with different types
 	auto material = std::make_shared<SurgSim::Graphics::OsgMaterial>("shiny");
-	material->setShader(shader);
+	material->setProgram(program);
 
 	{
 		auto uniform = std::make_shared<SurgSim::Graphics::OsgUniform<Vector4f>>("diffuseColor");
@@ -413,7 +377,7 @@ int main(int argc, char* argv[])
 	inputManager->addDevice(view->getKeyboardDevice());
 
 	// Shader should be shared between all materials using the same shader
-	auto shader = SurgSim::Graphics::loadShader(*runtime->getApplicationData(), "Shaders/ds_mapping_material");
+	auto shader = SurgSim::Graphics::loadProgram(*runtime->getApplicationData(), "Shaders/ds_mapping_material");
 	SURGSIM_ASSERT(shader != nullptr) << "Shader could not be loaded.";
 
 	RigidTransform3d armPose = makeRigidTransform(Quaterniond::Identity(), Vector3d(0.0, -0.2, 0.0));
@@ -435,7 +399,8 @@ int main(int argc, char* argv[])
 	std::shared_ptr<SceneElement> wound =
 		createFemSceneElement("wound",
 							  woundFilename,
-							  SurgSim::Math::INTEGRATIONSCHEME_LINEAR_IMPLICIT_EULER,
+							  SurgSim::Math::INTEGRATIONSCHEME_LINEAR_EULER_IMPLICIT,
+							  SurgSim::Math::LINEARSOLVER_LU,
 							  material);
 	wound->setPose(armPose);
 
@@ -445,17 +410,18 @@ int main(int argc, char* argv[])
 		std::make_shared<KeyboardTogglesComponentBehavior>("KeyboardBehavior");
 	keyboardBehavior->setInputComponent(keyboardComponent);
 
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_A, stapler->getComponent("Handle"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_A, stapler->getComponent("Indicator"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_A, stapler->getComponent("Markings"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_A, stapler->getComponent("Trigger"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_B, stapler->getComponent("Contacts"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_C, stapler->getComponent("Collision Mesh"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_D, arm->getComponent("Forearm"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_D, arm->getComponent("Upperarm"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_E, arm->getComponent("Collision Mesh"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_F, wound->getComponent("Graphics"));
-	keyboardBehavior->registerKey(SurgSim::Device::KeyCode::KEY_G, wound->getComponent("Wire Frame"));
+	using SurgSim::Devices::KeyCode;
+	keyboardBehavior->registerKey(KeyCode::KEY_A, stapler->getComponent("Handle"));
+	keyboardBehavior->registerKey(KeyCode::KEY_A, stapler->getComponent("Indicator"));
+	keyboardBehavior->registerKey(KeyCode::KEY_A, stapler->getComponent("Markings"));
+	keyboardBehavior->registerKey(KeyCode::KEY_A, stapler->getComponent("Trigger"));
+	keyboardBehavior->registerKey(KeyCode::KEY_B, stapler->getComponent("Contacts"));
+	keyboardBehavior->registerKey(KeyCode::KEY_C, stapler->getComponent("Collision Mesh"));
+	keyboardBehavior->registerKey(KeyCode::KEY_D, arm->getComponent("Forearm"));
+	keyboardBehavior->registerKey(KeyCode::KEY_D, arm->getComponent("Upperarm"));
+	keyboardBehavior->registerKey(KeyCode::KEY_E, arm->getComponent("Collision Mesh"));
+	keyboardBehavior->registerKey(KeyCode::KEY_F, wound->getComponent("Graphics"));
+	keyboardBehavior->registerKey(KeyCode::KEY_G, wound->getComponent("Wire Frame"));
 
 	std::shared_ptr<SceneElement> keyboard = std::make_shared<BasicSceneElement>("SceneElement");
 	keyboard->addComponent(keyboardComponent);
@@ -468,19 +434,6 @@ int main(int argc, char* argv[])
 	scene->addSceneElement(wound);
 	scene->addSceneElement(keyboard);
 
-	// Exclude collision between certain Collision::Representations
-	physicsManager->addExcludedCollisionPair(
-		getComponentChecked<SurgSim::Collision::Representation>(stapler, "Collision"),
-		getComponentChecked<SurgSim::Collision::Representation>(stapler, "VirtualToothCollision0"));
-
-	physicsManager->addExcludedCollisionPair(
-		getComponentChecked<SurgSim::Collision::Representation>(stapler, "Collision"),
-		getComponentChecked<SurgSim::Collision::Representation>(stapler, "VirtualToothCollision1"));
-
-	physicsManager->addExcludedCollisionPair(
-		getComponentChecked<SurgSim::Collision::Representation>(wound, "Collision"),
-		getComponentChecked<SurgSim::Collision::Representation>(arm, "Collision"));
-
 	runtime->execute();
 	return 0;
 }
