@@ -740,6 +740,7 @@ PlyFile *ply_read(FILE *fp, int *nelems, char ***elem_names)
   if (!words || !equal_strings (words[0], "ply"))
   {
 	  if (words) free(words);
+	  if (plyfile) free(plyfile);
 	  return (NULL);
   }
 
@@ -749,7 +750,11 @@ PlyFile *ply_read(FILE *fp, int *nelems, char ***elem_names)
 
     if (equal_strings (words[0], "format")) {
       if (nwords != 3)
+	  {
+	    if (words) free(words);
+	    if (plyfile) free(plyfile);
         return (NULL);
+	  }
       if (equal_strings (words[1], "ascii"))
         plyfile->file_type = PLY_ASCII;
       else if (equal_strings (words[1], "binary_big_endian"))
@@ -758,7 +763,8 @@ PlyFile *ply_read(FILE *fp, int *nelems, char ***elem_names)
         plyfile->file_type = PLY_BINARY_LE;
       else
 	  {
-		  free(words);
+	      if (words) free(words);
+	      if (plyfile) free(plyfile);
 		  return (NULL);
 	  }
 
@@ -2226,49 +2232,81 @@ void get_binary_item(
 
   switch (type) {
     case PLY_CHAR:
-      fread (ptr, 1, 1, fp);
+      if (fread (ptr, 1, 1, fp) != 1)
+      {
+        fprintf (stderr, "get_binary_item: read error\n");
+        exit (-1);
+      }
       *int_val = *((char *) ptr);
       *uint_val = *int_val;
       *double_val = *int_val;
       break;
     case PLY_UCHAR:
-      fread (ptr, 1, 1, fp);
+      if (fread (ptr, 1, 1, fp) != 1)
+      {
+        fprintf (stderr, "get_binary_item: read error\n");
+        exit (-1);
+      }
       *uint_val = *((unsigned char *) ptr);
       *int_val = *uint_val;
       *double_val = *uint_val;
       break;
     case PLY_SHORT:
-      fread (ptr, 2, 1, fp);
+      if (fread (ptr, 2, 1, fp) != 1)
+      {
+        fprintf (stderr, "get_binary_item: read error\n");
+        exit (-1);
+      }
       *int_val = *((short int *) ptr);
       *uint_val = *int_val;
       *double_val = *int_val;
       break;
     case PLY_USHORT:
-      fread (ptr, 2, 1, fp);
+      if (fread (ptr, 2, 1, fp) != 1)
+      {
+        fprintf (stderr, "get_binary_item: read error\n");
+        exit (-1);
+      }
       *uint_val = *((unsigned short int *) ptr);
       *int_val = *uint_val;
       *double_val = *uint_val;
       break;
     case PLY_INT:
-      fread (ptr, 4, 1, fp);
+      if (fread (ptr, 4, 1, fp) != 1)
+      {
+        fprintf (stderr, "get_binary_item: read error\n");
+        exit (-1);
+      }
       *int_val = *((int *) ptr);
       *uint_val = *int_val;
       *double_val = *int_val;
       break;
     case PLY_UINT:
-      fread (ptr, 4, 1, fp);
+      if (fread (ptr, 4, 1, fp) != 1)
+      {
+        fprintf (stderr, "get_binary_item: read error\n");
+        exit (-1);
+      }
       *uint_val = *((unsigned int *) ptr);
       *int_val = *uint_val;
       *double_val = *uint_val;
       break;
     case PLY_FLOAT:
-      fread (ptr, 4, 1, fp);
+      if (fread (ptr, 4, 1, fp) != 1)
+      {
+        fprintf (stderr, "get_binary_item: read error\n");
+        exit (-1);
+      }
       *double_val = *((float *) ptr);
 	  *int_val = (int)(*double_val);
 	  *uint_val = (unsigned int)(*double_val);
       break;
     case PLY_DOUBLE:
-      fread (ptr, 8, 1, fp);
+      if (fread (ptr, 8, 1, fp) != 1)
+      {
+        fprintf (stderr, "get_binary_item: read error\n");
+        exit (-1);
+      }
       *double_val = *((double *) ptr);
 	  *int_val = (int)(*double_val);
 	  *uint_val = (unsigned int)(*double_val);
