@@ -26,10 +26,12 @@
 
 namespace SurgSim
 {
-namespace Device
+namespace Devices
 {
 class LabJackScaffold;
 
+SURGSIM_STATIC_REGISTRATION(LabJackDevice);
+
 namespace LabJack
 {
 ///@{
@@ -276,18 +278,14 @@ public:
 	/// \param uniqueName A unique name for the device that will be used by the application.
 	explicit LabJackDevice(const std::string& uniqueName);
 
+	SURGSIM_CLASSNAME(SurgSim::Devices::LabJackDevice);
+
 	/// Destructor.
 	virtual ~LabJackDevice();
 
-	/// Fully initialize the device.
-	/// When the manager object creates the device, the internal state of the device usually isn't fully
-	/// initialized yet.  This method performs any needed initialization.
-	/// \return True on success.
-	/// \exception Asserts if already initialized, or if unable to get a scaffold.
-	virtual bool initialize() override;
+	bool initialize() override;
 
-	/// Check whether this device is initialized.
-	bool isInitialized() const;
+	bool isInitialized() const override;
 
 	/// Set the model, e.g., U6.
 	/// \param model The model.
@@ -314,6 +312,21 @@ public:
 	/// \return The address of the LabJack, e.g., "1" or "192.168.7.23".
 	const std::string& getAddress() const;
 
+	/// Reset LabJack during destruct.
+	/// \param reset true if the hardware should reset & re-enumerate when this object destructs.
+	/// \note The LabJack hardware has the feature that it continues operation and remembers all settings even after
+	///		the associated LabJackDevice object destructs and communication ends.
+	///		This function causes the hardware to reset when the LabJackDevice object destructs, thereby returning to its
+	///		default (boot-up) settings and forcing USB re-enumeration.
+	/// \note After a reset, it will take a few seconds before the hardware can communicate.
+	/// \warning If the LabJackDevice object does not cleanly destruct (e.g., the executable halts due to an exception),
+	///		then the hardware will not reset.
+	void setResetOnDestruct(bool reset);
+
+	/// Get whether or not the hardware should reset when the LabJackDevice object destructs.
+	/// \return true if should reset on destruct.
+	bool getResetOnDestruct() const;
+
 	/// Enable digital input line.
 	/// \param channel The channel number.
 	/// \exception Asserts if already initialized.
@@ -462,13 +475,10 @@ public:
 	int getAnalogInputSettling() const;
 
 private:
-	/// Finalize (de-initialize) the device.
-	/// \return True if device was successfully un-registered.
-	/// \exception Asserts if not initialized.
-	virtual bool finalize() override;
-
 	friend class LabJackScaffold;
 
+	bool finalize() override;
+
 	/// The single scaffold object that handles communications with all instances of LabJackDevice.
 	std::shared_ptr<LabJackScaffold> m_scaffold;
 
@@ -481,6 +491,9 @@ private:
 	/// The address, or a zero-length string to indicate the first-found device of this type on this connection.
 	std::string m_address;
 
+	/// Whether or not the hardware should reset when this object destructs.
+	bool m_reset;
+
 	/// The line numbers for the digital inputs.
 	std::unordered_set<int> m_digitalInputChannels;
 
@@ -517,7 +530,7 @@ private:
 	int m_analogInputSettling;
 };
 
-};  // namespace Device
+};  // namespace Devices
 };  // namespace SurgSim
 
 #endif  // SURGSIM_DEVICES_LABJACK_LABJACKDEVICE_H
