@@ -17,8 +17,8 @@
 
 #include <algorithm>
 #include <array>
-#include <boost/thread/mutex.hpp>
-#include <boost/thread/locks.hpp>
+#include <boost/chrono.hpp>
+#include <boost/thread.hpp>
 #include <errno.h>
 #include <labjackusb.h> // the low-level LabJack library (aka exodriver)
 #include <list>
@@ -40,7 +40,7 @@
 
 namespace SurgSim
 {
-namespace Device
+namespace Devices
 {
 
 namespace
@@ -187,40 +187,19 @@ public:
 	/// \param model The model of LabJack device to open (see strings in LabJackUD.h).
 	/// \param connection How to connect to the device (e.g., USB) (see strings in LabJackUD.h).
 	/// \param address Either the ID or serial number (if USB), or the IP address.
-	Handle(SurgSim::Device::LabJack::Model model, SurgSim::Device::LabJack::Connection connection,
+	Handle(SurgSim::Devices::LabJack::Model model, SurgSim::Devices::LabJack::Connection connection,
 		const std::string& address) :
 		m_deviceHandle(LABJACK_INVALID_HANDLE),
 		m_address(address),
 		m_model(model),
 		m_connection(connection),
-		m_scaffold(LabJackScaffold::getOrCreateSharedInstance())
+		m_logger(Framework::Logger::getLogger("Devices/LabJack"))
 	{
-		create();
-	}
-
-	/// Destructor.
-	~Handle()
-	{
-		SURGSIM_ASSERT(!isValid()) << "Expected destroy() to be called before Handle object destruction.";
-	}
-
-	/// \return Whether or not the wrapped handle is valid.
-	bool isValid() const
-	{
-		return (m_deviceHandle != LABJACK_INVALID_HANDLE);
-	}
-
-	/// Helper function called by the constructor to open the LabJack device for communications.
-	void create()
-	{
-		SURGSIM_ASSERT(!isValid()) <<
-			"Expected LabJackScaffold::Handle::create() to be called on an uninitialized object.";
-
 		bool result = true;
 
 		if (m_model == LabJack::MODEL_UE9)
 		{
-			SURGSIM_LOG_SEVERE(m_scaffold->getLogger()) << "Failed to open a device. " <<
+			SURGSIM_LOG_SEVERE(m_logger) << "Failed to open a device. " <<
 				"The UE9 model LabJack is not supported for the low-level driver used on Linux & Mac. " <<
 				"The commands for the UE9 have a different structure, which is not currently implemented." <<
 				std::endl <<
@@ -231,7 +210,7 @@ public:
 
 		if (m_connection != LabJack::CONNECTION_USB)
 		{
-			SURGSIM_LOG_SEVERE(m_scaffold->getLogger()) << "Failed to open a device. " <<
+			SURGSIM_LOG_SEVERE(m_logger) << "Failed to open a device. " <<
 				"The LabJackDevice connection must be set to USB for the low-level driver used on Linux & Mac." <<
 				std::endl <<
 				"  Model: '" << m_model << "'.  Connection: '" << m_connection << "'.  Address: '" <<
@@ -248,7 +227,7 @@ public:
 			}
 			catch (int e)
 			{
-				SURGSIM_LOG_SEVERE(m_scaffold->getLogger()) << "Failed to open a device. " <<
+				SURGSIM_LOG_SEVERE(m_logger) << "Failed to open a device. " <<
 					"The LabJackDevice address should be a string representation of an unsigned integer " <<
 					"corresponding to the device number (or the empty string to get the first device), " <<
 					"but the conversion from string to integer failed." << std::endl <<
@@ -261,11 +240,19 @@ public:
 		if (result)
 		{
 			const unsigned int dwReserved = 0; // Not used, set to 0.
-			m_deviceHandle = LJUSB_OpenDevice(deviceNumber, dwReserved, m_model);
+			int tries = 3;
+			m_deviceHandle = LABJACK_INVALID_HANDLE;
+			while ((m_deviceHandle == LABJACK_INVALID_HANDLE) && (--tries >= 0))
+			{
+				m_deviceHandle = LJUSB_OpenDevice(deviceNumber, dwReserved, m_model);
+				if ((m_deviceHandle == LABJACK_INVALID_HANDLE) && (tries >= 0))
+				{
+					boost::this_thread::sleep_for(boost::chrono::milliseconds(1000));
+				}
+			}
 			if (m_deviceHandle == LABJACK_INVALID_HANDLE)
 			{
-				SURGSIM_LOG_SEVERE(m_scaffold->getLogger()) << "Failed to open a device." <<
-					std::endl <<
+				SURGSIM_LOG_SEVERE(m_logger) << "Failed to open a device." << std::endl <<
 					"  Model: '" << m_model << "'.  Connection: '" << m_connection << "'.  Address: '" <<
 					m_address << "'." << std::endl <<
 					"  labjackusb error code: " << errno << "." << std::endl;
@@ -274,16 +261,85 @@ public:
 		}
 	}
 
-	bool destroy()
+	/// Destructor.
+	~Handle()
+	{
+		destroy();
+	}
+
+	/// \return Whether or not the wrapped handle is valid.
+	bool isValid() const
+	{
+		return (m_deviceHandle != LABJACK_INVALID_HANDLE);
+	}
+
+	/// Close communication with the hardware.
+	/// \param reset true to cause a hardware reset & USB re-enumeration.  Otherwise the hardware's settings will be
+	///		unchanged (i.e., it will continue timing, counting, and outputting).
+	void destroy(bool reset = false)
 	{
-		bool result = false;
 		if (isValid())
 		{
+			if (reset)
+			{
+				std::array<BYTE, LabJack::MAXIMUM_BUFFER> sendBytes;
+				sendBytes[1] = 0x99;  //Command byte, Reset
+				sendBytes[2] = 2;  // Bit 1: Hard Reset.  Bit 0: Soft Reset.
+				sendBytes[3] = 0x00;
+
+				int sendBytesSize = 4;
+				int readBytesSize = 4;
+
+				sendBytes[0] = LabJack::normalChecksum8(sendBytes, sendBytesSize);
+
+				const int sent = LJUSB_Write(m_deviceHandle, &(sendBytes[0]), sendBytesSize);
+
+				bool sendResult = true;
+				if (sent < sendBytesSize)
+				{
+					SURGSIM_LOG_SEVERE(m_logger) <<
+						"Failed to write reset command to a device." <<
+						"  Model: '" << m_model << "'.  Connection: '" << m_connection << "'.  Address: '" <<
+						m_address << "'." << std::endl << sendBytesSize << " bytes should have been sent, but only " <<
+						sent << " were actually sent." << std::endl <<
+						"  labjackusb error code: " << errno << "." << std::endl;
+					sendResult = false;
+				}
+
+				if (sendResult)
+				{
+					std::array<BYTE, LabJack::MAXIMUM_BUFFER> readBytes;
+					const int read = LJUSB_Read(m_deviceHandle, &(readBytes[0]), readBytesSize);
+					if (read < readBytesSize)
+					{
+						SURGSIM_LOG_SEVERE(m_logger) << "Failed to read response of reset command." <<
+							"  Model: '" << m_model << "'.  Connection: '" << m_connection << "'.  Address: '" <<
+							m_address << "'." << std::endl <<
+							readBytesSize << " bytes were expected, but only " << read << " were received." <<
+							std::endl << "  labjackusb error code: " << errno << "." << std::endl;
+					}
+					else if (LabJack::normalChecksum8(readBytes, readBytesSize) != readBytes[0])
+					{
+						SURGSIM_LOG_SEVERE(m_logger) << "Failed to read response of reset command." <<
+							"  Model: '" << m_model << "'.  Connection: '" << m_connection << "'.  Address: '" <<
+							m_address << "'." << std::endl <<
+							"The checksums are bad." << std::endl << "  labjackusb error code: " << errno << "." <<
+							std::endl;
+					}
+					else if (readBytes[3] != 0)
+					{
+						SURGSIM_LOG_SEVERE(m_logger) << "Failed to read response of reset command." <<
+							"  Model: '" << m_model << "'.  Connection: '" << m_connection << "'.  Address: '" <<
+							m_address << "'." << std::endl <<
+							"The device library returned an error code: " << static_cast<int>(readBytes[3]) << "." <<
+							std::endl << "  labjackusb error code: " << errno << "." << std::endl;
+					}
+				}
+			}
+
 			LJUSB_CloseDevice(m_deviceHandle);
 			m_deviceHandle = LABJACK_INVALID_HANDLE;
-			result = true;
 		}
-		return result;
 	}
 
 	/// \return The LabJack SDK's handle wrapped by this Handle.
@@ -302,11 +358,11 @@ private:
 	/// The address used to open the device.  Can be the empty string if the first-found device was opened.
 	std::string m_address;
 	/// The model of the device.
-	SurgSim::Device::LabJack::Model m_model;
+	SurgSim::Devices::LabJack::Model m_model;
 	/// The connection to the device.
-	SurgSim::Device::LabJack::Connection m_connection;
-	/// The scaffold.
-	std::shared_ptr<LabJackScaffold> m_scaffold;
+	SurgSim::Devices::LabJack::Connection m_connection;
+	/// The logger.
+	std::shared_ptr<Framework::Logger> m_logger;
 };
 
 /// The per-device data.
@@ -324,13 +380,20 @@ public:
 		timerOutputChannels(getTimerOutputChannels(device->getTimers())),
 		analogInputs(device->getAnalogInputs()),
 		analogOutputChannels(device->getAnalogOutputs()),
-		cachedOutputIndices(false)
+		cachedOutputIndices(false),
+		configured(false)
 	{
 	}
 
 
 	~DeviceData()
 	{
+		if (thread != nullptr)
+		{
+			thread->stop();
+			thread = nullptr;
+		}
+		deviceHandle->destroy(deviceObject->getResetOnDestruct());
 	}
 
 	/// The corresponding device object.
@@ -365,8 +428,10 @@ public:
 	std::unordered_map<int, int> analogInputIndices;
 	/// True if the output indices have been cached.
 	bool cachedOutputIndices;
-	// Calibration constants.  The meaning of each entry is specific to the model (i.e., LabJack::Model).
+	/// Calibration constants.  The meaning of each entry is specific to the model (i.e., LabJack::Model).
 	double calibration[40];
+	/// True if the device has been successfully configured.
+	bool configured;
 
 private:
 	/// Given all the timers, return just the ones that provide inputs.
@@ -397,18 +462,18 @@ private:
 		std::unordered_set<int> timersWithOutputs;
 		for (auto timer = timers.cbegin(); timer != timers.cend(); ++timer)
 		{
-			if ((timer->second.mode != LabJack::TIMERMODE_PWM_16BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_PWM_8BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_RISING_EDGES_32BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_FALLING_EDGES_32BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_DUTY_CYCLE) &&
-				(timer->second.mode != LabJack::TIMERMODE_FIRMWARE_COUNTER) &&
-				(timer->second.mode != LabJack::TIMERMODE_FIRMWARE_COUNTER_DEBOUNCED) &&
-				(timer->second.mode != LabJack::TIMERMODE_FREQUENCY_OUTPUT) &&
-				(timer->second.mode != LabJack::TIMERMODE_QUADRATURE) &&
-				(timer->second.mode != LabJack::TIMERMODE_RISING_EDGES_16BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_FALLING_EDGES_16BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_LINE_TO_LINE))
+			if ((timer->second.mode == LabJack::TIMERMODE_PWM_16BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_PWM_8BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_RISING_EDGES_32BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_FALLING_EDGES_32BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_DUTY_CYCLE) ||
+				(timer->second.mode == LabJack::TIMERMODE_FIRMWARE_COUNTER) ||
+				(timer->second.mode == LabJack::TIMERMODE_FIRMWARE_COUNTER_DEBOUNCED) ||
+				(timer->second.mode == LabJack::TIMERMODE_FREQUENCY_OUTPUT) ||
+				(timer->second.mode == LabJack::TIMERMODE_QUADRATURE) ||
+				(timer->second.mode == LabJack::TIMERMODE_RISING_EDGES_16BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_FALLING_EDGES_16BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_LINE_TO_LINE))
 			{
 				timersWithOutputs.insert(timer->first);
 			}
@@ -444,9 +509,9 @@ private:
 };
 
 LabJackScaffold::LabJackScaffold() :
+	m_logger(Framework::Logger::getLogger("Devices/LabJack")),
 	m_state(new StateData)
 {
-	m_logger = SurgSim::Framework::Logger::getLogger("LabJack device");
 	SURGSIM_LOG_DEBUG(m_logger) << "Shared scaffold created.  labjackusb driver version: " <<
 		LJUSB_GetLibraryVersion() << ".";
 }
@@ -454,17 +519,9 @@ LabJackScaffold::LabJackScaffold() :
 LabJackScaffold::~LabJackScaffold()
 {
 	boost::lock_guard<boost::mutex> lock(m_state->mutex);
-
 	if (!m_state->activeDeviceList.empty())
 	{
 		SURGSIM_LOG_SEVERE(m_logger) << "Destroying scaffold while devices are active!?!";
-		for (auto it = m_state->activeDeviceList.begin();  it != m_state->activeDeviceList.end();  ++it)
-		{
-			if ((*it)->thread)
-			{
-				destroyPerDeviceThread(it->get());
-			}
-		}
 		m_state->activeDeviceList.clear();
 	}
 	SURGSIM_LOG_DEBUG(m_logger) << "Shared scaffold destroyed.";
@@ -499,8 +556,8 @@ bool LabJackScaffold::registerDevice(LabJackDevice* device)
 
 	if (result && (address.length() > 0))
 	{
-		const SurgSim::Device::LabJack::Model model = device->getModel();
-		const SurgSim::Device::LabJack::Connection connection = device->getConnection();
+		const SurgSim::Devices::LabJack::Model model = device->getModel();
+		const SurgSim::Devices::LabJack::Connection connection = device->getConnection();
 
 		auto const sameInitialization = std::find_if(m_state->activeDeviceList.cbegin(),
 			m_state->activeDeviceList.cend(),
@@ -562,7 +619,6 @@ bool LabJackScaffold::registerDevice(LabJackDevice* device)
 					SURGSIM_LOG_INFO(m_logger) << "Tried to register a device named '" << device->getName() <<
 						"', but a device with the same handle (" << handle->get() << ") is already present!  " <<
 						"This can happen if multiple LabJack devices are used without setting their addresses.";
-					handle->destroy(); // The handle was initialized and will be destructed.
 					result = false;
 				}
 				else
@@ -615,14 +671,18 @@ bool LabJackScaffold::registerDevice(LabJackDevice* device)
 			}
 
 			std::unique_ptr<LabJackThread> thread(new LabJackThread(this, info.get()));
-			thread->setRate(device->getMaximumUpdateRate());
-			thread->start();
-
-			info.get()->thread = std::move(thread);
-			m_state->activeDeviceList.emplace_back(std::move(info));
+			result = info->configured;
+			if (result)
+			{
+				thread->setRate(device->getMaximumUpdateRate());
+				thread->start();
+				info.get()->thread = std::move(thread);
+				m_state->activeDeviceList.emplace_back(std::move(info));
+			}
 		}
 	}
 
+	SURGSIM_LOG_IF(result, m_logger, INFO) << "Device " << device->getName() << " registered.";
 	return result;
 }
 
@@ -635,22 +695,15 @@ bool LabJackScaffold::unregisterDevice(const LabJackDevice* const device)
 			[device](const std::unique_ptr<DeviceData>& info) { return info->deviceObject == device; });
 		if (matching != m_state->activeDeviceList.end())
 		{
-			if ((*matching)->thread)
-			{
-				destroyPerDeviceThread(matching->get());
-				matching->get()->deviceHandle->destroy();
-			}
 			m_state->activeDeviceList.erase(matching);
 			// the iterator is now invalid but that's OK
 			found = true;
+			SURGSIM_LOG_INFO(m_logger) << "Device " << device->getName() << " unregistered.";
 		}
 	}
 
-	if (!found)
-	{
-		SURGSIM_LOG_CRITICAL(m_logger) << "Attempted to release a non-registered device named '" <<
-			device->getName() << ".";
-	}
+	SURGSIM_LOG_IF(!found, m_logger, CRITICAL) << "Attempted to release a non-registered device named '" <<
+		device->getName() << ".";
 	return found;
 }
 
@@ -687,14 +740,12 @@ bool LabJackScaffold::runInputFrame(LabJackScaffold::DeviceData* info)
 		info->cachedOutputIndices = true;
 	}
 
-	if (!info->deviceObject->hasOutputProducer() || info->cachedOutputIndices)
+	if (!updateDevice(info))
 	{
-		if (!updateDevice(info))
-		{
-			return false;
-		}
-		info->deviceObject->pushInput();
+		return false;
 	}
+	info->deviceObject->pushInput();
+
 	return true;
 }
 
@@ -1007,17 +1058,6 @@ bool LabJackScaffold::updateDevice(LabJackScaffold::DeviceData* info)
 	return true; // Returning false ends the thread.
 }
 
-bool LabJackScaffold::destroyPerDeviceThread(DeviceData* data)
-{
-	SURGSIM_ASSERT(data->thread) << "LabJack: destroying a per-device thread, but none exists for this DeviceData";
-
-	std::unique_ptr<LabJackThread> thread = std::move(data->thread);
-	thread->stop();
-	thread.reset();
-
-	return true;
-}
-
 SurgSim::DataStructures::DataGroup LabJackScaffold::buildDeviceInputData()
 {
 	SurgSim::DataStructures::DataGroupBuilder builder;
@@ -1049,9 +1089,10 @@ std::shared_ptr<LabJackScaffold> LabJackScaffold::getOrCreateSharedInstance()
 	return sharedInstance.get();
 }
 
-bool LabJackScaffold::configureDevice(DeviceData* deviceData)
+void LabJackScaffold::configureDevice(DeviceData* deviceData)
 {
-	return configureClockAndTimers(deviceData) && configureDigital(deviceData) && configureAnalog(deviceData);
+	deviceData->configured =
+		configureClockAndTimers(deviceData) && configureDigital(deviceData) && configureAnalog(deviceData);
 }
 
 bool LabJackScaffold::configureClockAndTimers(DeviceData* deviceData)
@@ -1503,12 +1544,5 @@ bool LabJackScaffold::configureDigital(DeviceData* deviceData)
 	return result;
 }
 
-std::shared_ptr<SurgSim::Framework::Logger> LabJackScaffold::getLogger() const
-{
-	return m_logger;
-}
-
-
-
-};  // namespace Device
+};  // namespace Devices
 };  // namespace SurgSim
