@@ -16,8 +16,8 @@
 #include "SurgSim/Devices/LabJack/LabJackScaffold.h"
 
 #include <algorithm>
-#include <boost/thread/mutex.hpp>
-#include <boost/thread/locks.hpp>
+#include <boost/chrono.hpp>
+#include <boost/thread.hpp>
 #include <iostream>
 #include <LabJackUD.h> // the high-level LabJack library.
 #include <list>
@@ -35,7 +35,7 @@
 
 namespace SurgSim
 {
-namespace Device
+namespace Devices
 {
 
 namespace
@@ -89,15 +89,33 @@ public:
 		m_model(model),
 		m_connection(connection),
 		m_deviceHandle(LABJACK_INVALID_HANDLE),
-		m_scaffold(LabJackScaffold::getOrCreateSharedInstance())
+		m_logger(Framework::Logger::getLogger("Devices/LabJack"))
 	{
-		create();
+		int firstFound = 0;
+		if (m_address.length() == 0)
+		{
+			firstFound = 1;  // If no address is specified, grab the first device found of this model and connection.
+		}
+
+		int tries = 3;
+		LJ_ERROR error = LJE_MIN_USER_ERROR;
+		while (!isOk(error) && (--tries >= 0))
+		{
+			error = OpenLabJack(m_model, m_connection, m_address.c_str(), firstFound, &m_deviceHandle);
+			if (!isOk(error) && (tries >= 0))
+			{
+				boost::this_thread::sleep_for(boost::chrono::milliseconds(1000));
+			}
+		}
+		SURGSIM_LOG_IF(!isOk(error), m_logger, SEVERE) <<
+			"Failed to initialize a device. Model: " << m_model << ". Connection: " << m_connection << ". Address: '" <<
+			m_address << "'." << std::endl << formatErrorMessage(error);
 	}
 
 	/// Destructor.
 	~Handle()
 	{
-		SURGSIM_ASSERT(!isValid()) << "Expected destroy() to be called before Handle object destruction.";
+		destroy();
 	}
 
 	/// \return Whether or not the wrapped handle is valid.
@@ -106,41 +124,23 @@ public:
 		return (m_deviceHandle != LABJACK_INVALID_HANDLE);
 	}
 
-	/// Helper function called by the constructor to open the LabJack device for communications.
-	void create()
+	/// Close communication with the hardware.
+	/// \param reset true to cause a hardware reset & USB re-enumeration.  Otherwise the hardware's settings will be
+	///		unchanged (i.e., it will continue timing, counting, and outputting).
+	void destroy(bool reset = false)
 	{
-		SURGSIM_ASSERT(!isValid()) <<
-			"Expected LabJackScaffold::Handle::create() to be called on an uninitialized object.";
-
-		int firstFound = 0;
-		if (m_address.length() == 0)
-		{
-			firstFound = 1;  // If no address is specified, grab the first device found of this model and connection.
-		}
-
-		const LJ_ERROR error = OpenLabJack(m_model, m_connection, m_address.c_str(), firstFound, &m_deviceHandle);
-		SURGSIM_LOG_IF(!isOk(error), m_scaffold->getLogger(), SEVERE) <<
-			"Failed to initialize a device. Model: " << m_model << ". Connection: " << m_connection << ". Address: '" <<
-			m_address << "'." << std::endl << formatErrorMessage(error);
-	}
-
-	bool destroy()
-	{
-		bool result = false;
 		if (isValid())
 		{
-			// Reset the pin configuration.
-			const LJ_ERROR error = ePut(m_deviceHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
-			result = isOk(error);
-			SURGSIM_LOG_IF(!result, m_scaffold->getLogger(), SEVERE) <<
-				"Failed to reset a device's pin configuration. Model: " << m_model << ". Connection: " <<
-				m_connection << ". Address: '" << m_address << "'." << std::endl << formatErrorMessage(error);
-			if (result)
+			if (reset)
 			{
-				m_deviceHandle = LABJACK_INVALID_HANDLE;
+				const LJ_ERROR error = ResetLabJack(m_deviceHandle);
+				SURGSIM_LOG_IF(!isOk(error), m_logger, SEVERE) <<
+					"Failed to reset the LabJack device. Model: " << m_model << ". Connection: " <<
+					m_connection << ". Address: '" << m_address << "'." << std::endl << formatErrorMessage(error);
 			}
+
+			m_deviceHandle = LABJACK_INVALID_HANDLE;
 		}
-		return result;
 	}
 
 	/// \return The LabJackUD's handle wrapped by this Handle.
@@ -162,8 +162,8 @@ private:
 	LabJack::Model m_model;
 	/// The connection to the device.
 	LabJack::Connection m_connection;
-	/// The scaffold.
-	std::shared_ptr<LabJackScaffold> m_scaffold;
+	/// The logger.
+	std::shared_ptr<Framework::Logger> m_logger;
 };
 
 /// The per-device data.
@@ -173,21 +173,28 @@ public:
 	/// Initialize the data, creating a thread.
 	DeviceData(LabJackDevice* device, std::unique_ptr<Handle>&& handle) :
 		deviceObject(device),
-		deviceHandle(std::move(handle)),
 		thread(),
+		deviceHandle(std::move(handle)),
 		digitalInputChannels(device->getDigitalInputs()),
 		digitalOutputChannels(device->getDigitalOutputs()),
 		timerInputChannels(getTimerInputChannels(device->getTimers())),
 		timerOutputChannels(getTimerOutputChannels(device->getTimers())),
 		analogInputs(device->getAnalogInputs()),
 		analogOutputChannels(device->getAnalogOutputs()),
-		cachedOutputIndices(false)
+		cachedOutputIndices(false),
+		configured(false)
 	{
 	}
 
 
 	~DeviceData()
 	{
+		if (thread != nullptr)
+		{
+			thread->stop();
+			thread = nullptr;
+		}
+		deviceHandle->destroy(deviceObject->getResetOnDestruct());
 	}
 
 	/// The corresponding device object.
@@ -222,6 +229,8 @@ public:
 	std::unordered_map<int, int> analogInputIndices;
 	/// True if the output indices have been cached.
 	bool cachedOutputIndices;
+	/// True if the device has been successfully configured.
+	bool configured;
 
 private:
 	/// Given all the timers, return just the ones that provide inputs.
@@ -252,18 +261,18 @@ private:
 		std::unordered_set<int> timersWithOutputs;
 		for (auto timer = timers.cbegin(); timer != timers.cend(); ++timer)
 		{
-			if ((timer->second.mode != LabJack::TIMERMODE_PWM_16BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_PWM_8BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_RISING_EDGES_32BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_FALLING_EDGES_32BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_DUTY_CYCLE) &&
-				(timer->second.mode != LabJack::TIMERMODE_FIRMWARE_COUNTER) &&
-				(timer->second.mode != LabJack::TIMERMODE_FIRMWARE_COUNTER_DEBOUNCED) &&
-				(timer->second.mode != LabJack::TIMERMODE_FREQUENCY_OUTPUT) &&
-				(timer->second.mode != LabJack::TIMERMODE_QUADRATURE) &&
-				(timer->second.mode != LabJack::TIMERMODE_RISING_EDGES_16BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_FALLING_EDGES_16BIT) &&
-				(timer->second.mode != LabJack::TIMERMODE_LINE_TO_LINE))
+			if ((timer->second.mode == LabJack::TIMERMODE_PWM_16BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_PWM_8BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_RISING_EDGES_32BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_FALLING_EDGES_32BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_DUTY_CYCLE) ||
+				(timer->second.mode == LabJack::TIMERMODE_FIRMWARE_COUNTER) ||
+				(timer->second.mode == LabJack::TIMERMODE_FIRMWARE_COUNTER_DEBOUNCED) ||
+				(timer->second.mode == LabJack::TIMERMODE_FREQUENCY_OUTPUT) ||
+				(timer->second.mode == LabJack::TIMERMODE_QUADRATURE) ||
+				(timer->second.mode == LabJack::TIMERMODE_RISING_EDGES_16BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_FALLING_EDGES_16BIT) ||
+				(timer->second.mode == LabJack::TIMERMODE_LINE_TO_LINE))
 			{
 				timersWithOutputs.insert(timer->first);
 			}
@@ -299,27 +308,18 @@ private:
 };
 
 LabJackScaffold::LabJackScaffold() :
+	m_logger(Framework::Logger::getLogger("Devices/LabJack")),
 	m_state(new StateData)
 {
-	m_logger = SurgSim::Framework::Logger::getLogger("LabJack device");
-	SURGSIM_LOG_DEBUG(m_logger) << "Shared scaffold created.  LabJackUD driver version: " <<
-		GetDriverVersion() << ".";
+	SURGSIM_LOG_DEBUG(m_logger) << "Shared scaffold created.  LabJackUD driver version: " << GetDriverVersion();
 }
 
 LabJackScaffold::~LabJackScaffold()
 {
 	boost::lock_guard<boost::mutex> lock(m_state->mutex);
-
 	if (!m_state->activeDeviceList.empty())
 	{
 		SURGSIM_LOG_SEVERE(m_logger) << "Destroying scaffold while devices are active!?!";
-		for (auto it = m_state->activeDeviceList.begin();  it != m_state->activeDeviceList.end();  ++it)
-		{
-			if ((*it)->thread)
-			{
-				destroyPerDeviceThread(it->get());
-			}
-		}
 		m_state->activeDeviceList.clear();
 	}
 	SURGSIM_LOG_DEBUG(m_logger) << "Shared scaffold destroyed.";
@@ -424,7 +424,6 @@ bool LabJackScaffold::registerDevice(LabJackDevice* device)
 						SURGSIM_LOG_INFO(m_logger) << "Tried to register a device named '" << device->getName() <<
 							"', but a device with the same handle (" << handle->get() << ") is already present!  " <<
 							"This can happen if multiple LabJack devices are used without setting their addresses.";
-						handle->destroy(); // The handle was initialized and will be destructed.
 						result = false;
 					}
 					else
@@ -479,14 +478,18 @@ bool LabJackScaffold::registerDevice(LabJackDevice* device)
 			}
 
 			std::unique_ptr<LabJackThread> thread(new LabJackThread(this, info.get()));
-			thread->setRate(device->getMaximumUpdateRate());
-			thread->start();
-
-			info.get()->thread = std::move(thread);
-			m_state->activeDeviceList.emplace_back(std::move(info));
+			result = info->configured;
+			if (result)
+			{
+				thread->setRate(device->getMaximumUpdateRate());
+				thread->start();
+				info.get()->thread = std::move(thread);
+				m_state->activeDeviceList.emplace_back(std::move(info));
+			}
 		}
 	}
 
+	SURGSIM_LOG_IF(result, m_logger, INFO) << "Device " << device->getName() << " registered.";
 	return result;
 }
 
@@ -499,22 +502,15 @@ bool LabJackScaffold::unregisterDevice(const LabJackDevice* const device)
 			[device](const std::unique_ptr<DeviceData>& info) { return info->deviceObject == device; });
 		if (matching != m_state->activeDeviceList.end())
 		{
-			if ((*matching)->thread)
-			{
-				destroyPerDeviceThread(matching->get());
-				matching->get()->deviceHandle->destroy();
-			}
 			m_state->activeDeviceList.erase(matching);
 			// the iterator is now invalid but that's OK
 			found = true;
+			SURGSIM_LOG_INFO(m_logger) << "Device " << device->getName() << " unregistered.";
 		}
 	}
 
-	if (!found)
-	{
-		SURGSIM_LOG_CRITICAL(m_logger) << "Attempted to release a non-registered device named '" <<
-			device->getName() << ".";
-	}
+	SURGSIM_LOG_IF(!found, m_logger, CRITICAL) << "Attempted to release a non-registered device named '" <<
+		device->getName() << ".";
 	return found;
 }
 
@@ -551,14 +547,12 @@ bool LabJackScaffold::runInputFrame(LabJackScaffold::DeviceData* info)
 		info->cachedOutputIndices = true;
 	}
 
-	if (!info->deviceObject->hasOutputProducer() || info->cachedOutputIndices)
+	if (!updateDevice(info))
 	{
-		if (!updateDevice(info))
-		{
-			return false;
-		}
-		info->deviceObject->pushInput();
+		return false;
 	}
+	info->deviceObject->pushInput();
+
 	return true;
 }
 
@@ -772,17 +766,6 @@ bool LabJackScaffold::updateDevice(LabJackScaffold::DeviceData* info)
 	return true;
 }
 
-bool LabJackScaffold::destroyPerDeviceThread(DeviceData* data)
-{
-	SURGSIM_ASSERT(data->thread) << "LabJack: destroying a per-device thread, but none exists for this DeviceData";
-
-	std::unique_ptr<LabJackThread> thread = std::move(data->thread);
-	thread->stop();
-	thread.reset();
-
-	return true;
-}
-
 SurgSim::DataStructures::DataGroup LabJackScaffold::buildDeviceInputData()
 {
 	SurgSim::DataStructures::DataGroupBuilder builder;
@@ -814,7 +797,7 @@ std::shared_ptr<LabJackScaffold> LabJackScaffold::getOrCreateSharedInstance()
 	return sharedInstance.get();
 }
 
-bool LabJackScaffold::configureDevice(DeviceData* deviceData)
+void LabJackScaffold::configureDevice(DeviceData* deviceData)
 {
 	LJ_HANDLE rawHandle = deviceData->deviceHandle->get();
 
@@ -825,7 +808,8 @@ bool LabJackScaffold::configureDevice(DeviceData* deviceData)
 		"Failed to reset configuration for a device named '" << deviceData->deviceObject->getName() << "." <<
 		std::endl << formatErrorMessage(error);
 
-	return result && configureClockAndTimers(deviceData) && configureDigital(deviceData) && configureAnalog(deviceData);
+	deviceData->configured = result &&
+		configureClockAndTimers(deviceData) && configureDigital(deviceData) && configureAnalog(deviceData);
 }
 
 bool LabJackScaffold::configureClockAndTimers(DeviceData* deviceData)
@@ -1022,10 +1006,5 @@ bool LabJackScaffold::configureDigital(DeviceData* deviceData)
 	return true;
 }
 
-std::shared_ptr<SurgSim::Framework::Logger> LabJackScaffold::getLogger() const
-{
-	return m_logger;
-}
-
-};  // namespace Device
+};  // namespace Devices
 };  // namespace SurgSim
