@@ -1,5 +1,5 @@
 // This file is a part of the OpenSurgSim project.
-// Copyright 2013, SimQuest Solutions Inc.
+// Copyright 2013-2016, SimQuest Solutions Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -246,7 +246,7 @@ TEST(ComponentTests, NullPointerSerialization)
 	auto container = std::make_shared<TestComponent3>("TestComponent3");
 	YAML::Node containerNode = YAML::convert<Component>::encode(*container);
 	std::shared_ptr<TestComponent3> newContainer = std::dynamic_pointer_cast<TestComponent3>(
-		containerNode.as<std::shared_ptr<Component>>());
+				containerNode.as<std::shared_ptr<Component>>());
 	EXPECT_EQ(nullptr, newContainer->getComponentOne());
 	EXPECT_EQ(nullptr, newContainer->getComponentTwo());
 	EXPECT_EQ(container->getName(), newContainer->getName());
@@ -486,7 +486,9 @@ TEST(ComponentTests, SetActiveTest)
 	EXPECT_FALSE(component->getValue<bool>("IsActive"));
 
 	auto sceneElement = std::make_shared<SurgSim::Framework::BasicSceneElement>("SceneElement");
+	auto runtime = std::make_shared<SurgSim::Framework::Runtime>();
 	sceneElement->addComponent(component);
+	runtime->getScene()->addSceneElement(sceneElement);
 	EXPECT_TRUE(sceneElement->isActive());
 
 	// An inactive component in an active SceneElement is 'inactive'.
@@ -519,6 +521,7 @@ TEST(ComponentTests, SetActiveTest)
 			break;
 		}
 	}
+	ASSERT_NE(nullptr, decodedComponent);
 	EXPECT_EQ(nullptr, decodedComponent->getSceneElement());
 	EXPECT_TRUE(decodedComponent->isActive());
 	EXPECT_TRUE(decodedComponent->isLocalActive());
@@ -528,9 +531,26 @@ TEST(ComponentTests, SetActiveTest)
 	decodedComponent = nullptr;
 	auto decodedSceneElement = std::make_shared<SurgSim::Framework::BasicSceneElement>("Decoded");
 	decodedSceneElement->decode(node);
+	runtime->getScene()->addSceneElement(decodedSceneElement);
 	EXPECT_FALSE(decodedSceneElement->isActive());
 	decodedComponent = decodedSceneElement->getComponent("Component");
 	EXPECT_NE(nullptr, decodedComponent->getSceneElement());
 	EXPECT_FALSE(decodedComponent->isActive());
 	EXPECT_TRUE(decodedComponent->isLocalActive());
 }
+
+TEST(ComponentTests, CheckAndConvertTest)
+{
+	using SurgSim::Framework::checkAndConvert;
+
+	auto original = std::make_shared<MockComponent>("test");
+	auto other = std::make_shared<MockBehavior>("other");
+	std::shared_ptr<Component> source = original;
+	std::shared_ptr<MockComponent> result;
+
+	EXPECT_NO_THROW(result = checkAndConvert<MockComponent>(source, "MockComponent"));
+	EXPECT_EQ(result, original);
+
+
+	EXPECT_ANY_THROW(result = checkAndConvert<MockComponent>(other, "MockBehavior"));
+}
