@@ -34,7 +34,9 @@ public:
 	/// \param equation The ode equation to be solved
 	explicit OdeSolverLinearRungeKutta4(OdeEquation* equation);
 
-	virtual void solve(double dt, const OdeState& currentState, OdeState* newState) override;
+	/// The parameter computeCompliance is irrelevant for any Linear solver as it is a constant matrix.
+	/// It will be precomputed on the first call and use in the following calls, no matter what the parameter value is.
+	void solve(double dt, const OdeState& currentState, OdeState* newState, bool computeCompliance = true) override;
 
 private:
 	bool m_initialized;
