@@ -23,24 +23,39 @@ namespace SurgSim
 namespace Math
 {
 
+Shape::Shape()
+{
+	m_aabb.setEmpty();
+}
+
 Shape::~Shape()
 {
 }
 
-Shape::FactoryType& Shape::getFactory()
+bool Shape::isTransformable() const
 {
-	static FactoryType factory;
-	return factory;
+	return false;
+}
+
+std::shared_ptr<Shape> Shape::getTransformed(const RigidTransform3d& pose) const
+{
+	SURGSIM_FAILURE() << "getTransformed not implemented for " << getClassName();
+	return nullptr;
 }
 
 /// Get class name
 std::string Shape::getClassName() const
 {
 	SURGSIM_LOG_WARNING(SurgSim::Framework::Logger::getDefaultLogger()) <<
-		"getClassName() called on Math::Shape base class, this is wrong" <<
-		" in almost all cases, this means there is a class that does not have getClassName() defined.";
+			"getClassName() called on Math::Shape base class, this is wrong" <<
+			" in almost all cases, this means there is a class that does not have getClassName() defined.";
 	return "SurgSim::Math::Shape";
 }
 
+const Math::Aabbd& Shape::getBoundingBox() const
+{
+	return m_aabb;
+}
+
 } // namespace Math
-} // namespace SurgSim
\ No newline at end of file
+} // namespace SurgSim
