@@ -1,5 +1,5 @@
 // This file is a part of the OpenSurgSim project.
-// Copyright 2013, SimQuest Solutions Inc.
+// Copyright 2013-2015, SimQuest Solutions Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -144,6 +144,38 @@ protected:
 };
 
 
+TEST_F(GeometryTest, BaryCentricOfSegment)
+{
+	typedef Eigen::Matrix<SizeType, 2, 1> Vector2;
+
+	Vector2 outputPoint;
+	EXPECT_TRUE(barycentricCoordinates(plainSegment.a, plainSegment.a, plainSegment.b, &outputPoint));
+	EXPECT_TRUE(Vector2(1.0, 0.0).isApprox(outputPoint));
+
+	EXPECT_TRUE(barycentricCoordinates(plainSegment.b, plainSegment.a, plainSegment.b, &outputPoint));
+	EXPECT_TRUE(Vector2(0.0, 1.0).isApprox(outputPoint));
+
+	// Halfway points
+	EXPECT_TRUE(barycentricCoordinates(plainSegment.pointOnLine(0.5), plainSegment.a, plainSegment.b, &outputPoint));
+	EXPECT_TRUE(Vector2(0.5, 0.5).isApprox(outputPoint));
+
+	// Random point
+	EXPECT_TRUE(barycentricCoordinates(plainSegment.pointOnLine(0.327), plainSegment.a, plainSegment.b, &outputPoint));
+	EXPECT_TRUE(Vector2(1.0 - 0.327, 0.327).isApprox(outputPoint));
+
+	// Point not on line
+	VectorType orthogonal =
+		plainSegment.ab.cross(VectorType(plainSegment.ab[0] * 0.5, plainSegment.ab[1] * 0.6, plainSegment.ab[2] * 0.7));
+	VectorType point = plainSegment.pointOnLine(0.486) + orthogonal;
+	EXPECT_TRUE(barycentricCoordinates(point, plainSegment.a, plainSegment.b, &outputPoint));
+	EXPECT_TRUE(Vector2(1.0 - 0.486, 0.486).isApprox(outputPoint));
+
+	// Degenerate
+	EXPECT_FALSE(barycentricCoordinates(degenerateSegment.a, degenerateSegment.a, degenerateSegment.b, &outputPoint));
+	EXPECT_TRUE(boost::math::isnan(outputPoint[0]) && boost::math::isnan(outputPoint[1]));
+}
+
+
 TEST_F(GeometryTest, BaryCentricWithNormal)
 {
 	// Order of Points is v0,v1,v2
@@ -224,10 +256,10 @@ TEST_F(GeometryTest, BaryCentricWithoutNormal)
 	EXPECT_TRUE(eigenEqual(VectorType(0.2, 0.25, 0.55), outputPoint));
 
 	// Degenerate
-	EXPECT_FALSE(barycentricCoordinates(inputPoint, tri.v1, tri.v1, tri.v2, &outputPoint));
+	EXPECT_FALSE(barycentricCoordinates(inputPoint, tri.v0, tri.v0, tri.v2, &outputPoint));
 	EXPECT_TRUE(eigenAllNan(outputPoint));
 
-	EXPECT_FALSE(barycentricCoordinates(inputPoint, tri.v0, tri.v0, tri.v2, &outputPoint));
+	EXPECT_FALSE(barycentricCoordinates(inputPoint, tri.v0, tri.v1, tri.v1, &outputPoint));
 	EXPECT_TRUE(eigenAllNan(outputPoint));
 
 	EXPECT_FALSE(barycentricCoordinates(inputPoint, tri.v2, tri.v1, tri.v2, &outputPoint));
@@ -563,11 +595,11 @@ TEST_F(GeometryTest, DistanceSegmentSegment)
 	// <13> projections intersect, short segments
 	const VectorType aPoint = VectorType(1, 0, 0);
 	const SizeType shortLength = 0.0001;
-	const Segment shortSegment = Segment(VectorType(0, -shortLength/2, 0), VectorType(0, shortLength/2, 0));
+	const Segment shortSegment = Segment(VectorType(0, -shortLength / 2, 0), VectorType(0, shortLength / 2, 0));
 	const VectorType shortSegmentNormal = shortSegment.ab.cross(aPoint).normalized();
 	const Segment otherShortSegment = Segment(aPoint, aPoint + shortLength * shortSegmentNormal);
 	segments.push_back(SegmentData(shortSegment, otherShortSegment,
-		shortSegment.pointOnLine(0.5), otherShortSegment.a));
+								   shortSegment.pointOnLine(0.5), otherShortSegment.a));
 
 	for (size_t i = 0; i < segments.size(); ++i)
 	{
@@ -586,7 +618,7 @@ TEST_F(GeometryTest, DistanceSegmentSegment)
 	closestPoint = plainSegment.a;
 	const Vector3d segmentDirection = plainSegment.a - plainSegment.b;
 	otherSegment = Segment(closestPoint + plainNormal * 4 + 2 * segmentDirection,
-		closestPoint + plainNormal * 4 + 4 * segmentDirection);
+						   closestPoint + plainNormal * 4 + 4 * segmentDirection);
 	distance = distanceSegmentSegment(plainSegment.a, plainSegment.b, otherSegment.a, otherSegment.b, &p0, &p1);
 	segments.push_back(SegmentData(plainSegment, otherSegment, closestPoint, otherSegment.a));
 
@@ -598,7 +630,7 @@ TEST_F(GeometryTest, DistanceSegmentSegment)
 	// <1> anti-parallel, non-overlapping
 	closestPoint = plainSegment.a;
 	otherSegment = Segment(closestPoint + plainNormal * 4 + 4 * segmentDirection,
-		closestPoint + plainNormal * 4 + 2 * segmentDirection);
+						   closestPoint + plainNormal * 4 + 2 * segmentDirection);
 	distance = distanceSegmentSegment(plainSegment.a, plainSegment.b, otherSegment.a, otherSegment.b, &p0, &p1);
 	segments.push_back(SegmentData(plainSegment, otherSegment, closestPoint, otherSegment.b));
 
@@ -721,16 +753,16 @@ TEST_F(GeometryTest, DistancePointTriangle)
 	// Degenerate Edges
 	// Edge v0v1
 	distance = distancePointTriangle(inputPoint,
-			   tri.v0, (tri.v0 + tri.v0v1 * epsilon * 0.01).eval(), tri.v2,
-			   &result);
+									 tri.v0, (tri.v0 + tri.v0v1 * epsilon * 0.01).eval(), tri.v2,
+									 &result);
 	expectedDistance = distancePointSegment(inputPoint, tri.v0, tri.v2, &closestPoint);
 	EXPECT_NEAR(expectedDistance, distance, epsilon);
 	EXPECT_TRUE(eigenEqual(closestPoint, result));
 
 	// Edge v0v2
 	distance = distancePointTriangle(inputPoint,
-			   (tri.v2 - tri.v0v2 * epsilon * 0.01).eval(), tri.v1 , tri.v2,
-			   &result);
+									 (tri.v2 - tri.v0v2 * epsilon * 0.01).eval(), tri.v1 , tri.v2,
+									 &result);
 	expectedDistance = distancePointSegment(inputPoint, tri.v1, tri.v2, &closestPoint);
 	EXPECT_NEAR(expectedDistance, distance, epsilon);
 	EXPECT_TRUE(eigenEqual(closestPoint, result));
@@ -781,6 +813,49 @@ TEST_F(GeometryTest, PointInsideTriangleWithoutNormal)
 	EXPECT_FALSE(isPointInsideTriangle(inputPoint, tri.v0, tri.v1, tri.v2));
 }
 
+TEST_F(GeometryTest, PointOnTriangleEdgeWithNormal)
+{
+	EXPECT_TRUE(isPointOnTriangleEdge(tri.v0, tri.v0, tri.v1, tri.v2, tri.n));
+	EXPECT_TRUE(isPointOnTriangleEdge(tri.v1, tri.v0, tri.v1, tri.v2, tri.n));
+	EXPECT_TRUE(isPointOnTriangleEdge(tri.v2, tri.v0, tri.v1, tri.v2, tri.n));
+
+	VectorType inputPoint = tri.v0 + tri.v0v1 * 0.2;
+	EXPECT_TRUE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2, tri.n));
+	inputPoint = tri.v0 + tri.v0v2 * 0.2;
+	EXPECT_TRUE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2, tri.n));
+	inputPoint = tri.v1 + tri.v1v2 * 0.2;
+	EXPECT_TRUE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2, tri.n));
+
+	inputPoint = tri.v0 + tri.v0v1 * 1.5;
+	EXPECT_FALSE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2, tri.n));
+	EXPECT_FALSE(isPointOnTriangleEdge(inputPoint, tri.v1, tri.v1, tri.v2, tri.n));
+
+	inputPoint = tri.v0 + tri.v0v2 * 2 + tri.v0v1 * 2;
+	EXPECT_FALSE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2, tri.n));
+
+}
+
+TEST_F(GeometryTest, PointOnTriangleEdgeWithoutNormal)
+{
+	EXPECT_TRUE(isPointOnTriangleEdge(tri.v0, tri.v0, tri.v1, tri.v2));
+	EXPECT_TRUE(isPointOnTriangleEdge(tri.v1, tri.v0, tri.v1, tri.v2));
+	EXPECT_TRUE(isPointOnTriangleEdge(tri.v2, tri.v0, tri.v1, tri.v2));
+
+	VectorType inputPoint = tri.v0 + tri.v0v1 * 0.2;
+	EXPECT_TRUE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2));
+	inputPoint = tri.v0 + tri.v0v2 * 0.2;
+	EXPECT_TRUE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2));
+	inputPoint = tri.v1 + tri.v1v2 * 0.2;
+	EXPECT_TRUE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2));
+
+	inputPoint = tri.v0 + tri.v0v1 * 1.5;
+	EXPECT_FALSE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2));
+	EXPECT_FALSE(isPointOnTriangleEdge(inputPoint, tri.v1, tri.v1, tri.v2));
+
+	inputPoint = tri.v0 + tri.v0v2 * 2 + tri.v0v1 * 2;
+	EXPECT_FALSE(isPointOnTriangleEdge(inputPoint, tri.v0, tri.v1, tri.v2));
+}
+
 TEST_F(GeometryTest, Coplanarity)
 {
 	struct CoplanarityTestCandidate
@@ -805,22 +880,24 @@ TEST_F(GeometryTest, Coplanarity)
 		{true, Vector3d(0.0, 0.0, 0.0), Vector3d(1.0, 0.0, 0.0), Vector3d(0.0, 1.0, 0.0), Vector3d(12.3, -41.3, 0.0)},
 		{false, Vector3d(0.0, 0.0, 0.0), Vector3d(1.0, 0.0, 0.0), Vector3d(0.0, 1.0, 0.0), Vector3d(12.3, -41.3, 4.0)},
 
-		{false, Vector3d(10932.645, 43.1987, -0.009874245),
-				Vector3d(53432.4, -9.87243, 654.31),
-				Vector3d(28.71, 0.005483927, 2.34515),
-				Vector3d(5897.1, -5.432, 512152.7654)}
+		{
+			false, Vector3d(10932.645, 43.1987, -0.009874245),
+			Vector3d(53432.4, -9.87243, 654.31),
+			Vector3d(28.71, 0.005483927, 2.34515),
+			Vector3d(5897.1, -5.432, 512152.7654)
+		}
 	};
 
 	for (auto candidate = std::begin(candidates); candidate != std::end(candidates); ++candidate)
 	{
 		EXPECT_EQ(candidate->expected, isCoplanar(candidate->points[0],
-												  candidate->points[1],
-												  candidate->points[2],
-												  candidate->points[3]))
-			<< "Candidate points were [" << candidate->points[0].transpose() << "], ["
-										 << candidate->points[1].transpose() << "], ["
-										 << candidate->points[2].transpose() << "], ["
-										 << candidate->points[3].transpose() << "]";
+				  candidate->points[1],
+				  candidate->points[2],
+				  candidate->points[3]))
+				<< "Candidate points were [" << candidate->points[0].transpose() << "], ["
+				<< candidate->points[1].transpose() << "], ["
+				<< candidate->points[2].transpose() << "], ["
+				<< candidate->points[3].transpose() << "]";
 	}
 }
 
@@ -1027,6 +1104,45 @@ TEST_F(GeometryTest, SegmentPlaneDistance)
 	}
 }
 
+template <class T>
+class GeometryVectorTestBase : public testing::Test
+{
+public:
+	typedef T Scalar;
+};
+
+template <class T>
+class GeometryVector3Tests : public GeometryVectorTestBase<typename T::Scalar>
+{
+public:
+	typedef T Vector3;
+};
+
+typedef ::testing::Types<SurgSim::Math::Vector3d,
+		SurgSim::Math::Vector3f> GeometryVector3Variants;
+TYPED_TEST_CASE(GeometryVector3Tests, GeometryVector3Variants);
+
+TYPED_TEST(GeometryVector3Tests, nearestPointOnLine)
+{
+	typedef typename TestFixture::Vector3 Vector3;
+	typedef typename Vector3::Scalar T;
+	const int VOpt = Vector3::Options;
+
+	Vector3 point(static_cast<T>(2.0), static_cast<T>(-4.0), static_cast<T>(3.0));
+	Vector3 segmentEnd1(static_cast<T>(-2.0), static_cast<T>(2.0), static_cast<T>(4.0));
+	Vector3 segmentEnd2(static_cast<T>(-1.0), static_cast<T>(1.0), static_cast<T>(2.0));
+	T precision = Eigen::NumTraits<T>::dummy_precision();
+
+	// Assert if segment is degenerate
+	ASSERT_ANY_THROW((SurgSim::Math::nearestPointOnLine<T, VOpt>(point, segmentEnd1, segmentEnd1)));
+
+	// Otherwise, calculate the correct value
+	ASSERT_NO_THROW((SurgSim::Math::nearestPointOnLine<T, VOpt>(point, segmentEnd1, segmentEnd2)));
+
+	auto result = SurgSim::Math::nearestPointOnLine<T, VOpt>(point, segmentEnd1, segmentEnd2);
+	EXPECT_GT(precision, (std::abs((segmentEnd2 - segmentEnd1).dot(result - point))));
+}
+
 typedef std::tuple<MockTriangle, VectorType, double, VectorType, VectorType, int> TriPlaneData;
 void checkTriPlaneDistance(const TriPlaneData& data)
 {
@@ -1041,7 +1157,7 @@ void checkTriPlaneDistance(const TriPlaneData& data)
 
 	distance = distanceTrianglePlane(tri.v0, tri.v1, tri.v2, n, d, &triangleResultPoint, &planeResultPoint);
 	EXPECT_NEAR((planeResultPoint - triangleResultPoint).norm(), std::abs(distance), epsilon);
-	EXPECT_TRUE(distance * sign > 0 || distance == static_cast<double>(sign));
+	EXPECT_TRUE((sign == 0 && std::abs(distance) <= epsilon) || (distance * sign > 0));
 	EXPECT_TRUE(expectedTrianglePoint.isApprox(triangleResultPoint));
 	EXPECT_TRUE(expectedPlanePoint.isApprox(planeResultPoint));
 }
@@ -1559,5 +1675,234 @@ TEST_F(GeometryTest, DoesIntersectBoxCapsule)
 	}
 }
 
+TEST_F(GeometryTest, TimesOfCoplanarity)
+{
+	std::array<SizeType, 3> times;
+
+	{
+		SCOPED_TRACE("No coplanarity case in [0..1]");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 0.0, 4.0));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.0, 0.1, 10.0), VectorType(0.0, 1.1, 10.0));
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(0.0, 1.1, 10.0), VectorType(0.0, 2.1, 10.0));
+		EXPECT_EQ(0, (timesOfCoplanarityInRange01<double, Vector3d::Options>(A, B, C, D, &times)));
+	}
+
+	{
+		SCOPED_TRACE("1 case of coplanarity at time 0");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 0.0, 4.0));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.0, 1.0, -2.0), VectorType(0.0, 2.0, -3.5));
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(0.0, 1.0, -10.0));
+		EXPECT_EQ(1, (timesOfCoplanarityInRange01<double, Vector3d::Options>(A, B, C, D, &times)));
+		EXPECT_DOUBLE_EQ(0.0, times[0]);
+	}
+
+	{
+		SCOPED_TRACE("1 case of coplanarity at time 0.5");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.1, 0.0, 1.0), VectorType(-0.1, 0.0, -1.0));
+		// A(0.5) = 0.0 0.0 0.0
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 3.0, -2.0));
+		// B(0.5) = 1.5 1.5 0.0
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(-0.3, 2.0, -2.0), VectorType(0.3, 1.0, 2.0));
+		// C(0.5) = 0.0 1.5 0.0
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(1.7, 1.0, -1.0), VectorType(1.3, -1.0, 1.0));
+		// D(0.5) = 1.5 0.0 0.0
+		EXPECT_EQ(1, (timesOfCoplanarityInRange01<double, Vector3d::Options>(A, B, C, D, &times)));
+		EXPECT_NEAR(0.5, times[0], Math::Geometry::ScalarEpsilon);
+	}
+
+	{
+		SCOPED_TRACE("2 cases of coplanarity at times 0.089971778657590915 and 0.5");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.4, 0.8, 1.0), VectorType(0.8, 1.0, -1.0));
+		// A(0.5) = 0.6 0.9 0.0
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 3.0, -2.0));
+		// B(0.5) = 1.5 1.5 0.0
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(-0.3, 2.0, -2.0), VectorType(0.3, 1.0, 2.0));
+		// C(0.5) = 0.0 1.5 0.0
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(1.7, 1.0, -1.0), VectorType(1.3, -1.0, 1.0));
+		// D(0.5) = 1.5 0.0 0.0
+		EXPECT_EQ(2, (timesOfCoplanarityInRange01<double, Vector3d::Options>(A, B, C, D, &times)));
+		EXPECT_NEAR(0.089971778657590915, times[0], Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.5, times[1], Math::Geometry::ScalarEpsilon);
+	}
+
+	{
+		SCOPED_TRACE("1 case of coplanarity at time 1");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 1.24), VectorType(2.0, 0.0, 3.0));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.0, 1.0, -2.0), VectorType(2.0, 0.0, 3.0));
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(0.1, 1.0, 1.1), VectorType(0.0, 1.0, -1.54));
+		EXPECT_EQ(1, (timesOfCoplanarityInRange01<double, Vector3d::Options>(A, B, C, D, &times)));
+		EXPECT_NEAR(1.0, times[0], Math::Geometry::ScalarEpsilon);
+	}
+}
+
+TEST_F(GeometryTest, CcdIntersectionsSegmentSegment)
+{
+	SizeType time, s0p1Factor, s1p1Factor;
+
+	{
+		SCOPED_TRACE("No intersection");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 0.0, 4.0));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.0, 0.1, 4.0), VectorType(0.0, 1.1, 8.0));
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(0.0, 1.1, 6.0), VectorType(0.0, 2.1, 54.0));
+		EXPECT_FALSE(calculateCcdContactSegmentSegment(A, B, C, D, &time, &s0p1Factor, &s1p1Factor));
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=0");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 0.0, 4.0));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.0, 1.0, -2.0), VectorType(0.0, 2.0, -3.5));
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(0.0, 1.0, -10.0));
+		EXPECT_TRUE(calculateCcdContactSegmentSegment(A, B, C, D, &time, &s0p1Factor, &s1p1Factor));
+		EXPECT_DOUBLE_EQ(0.0, time);
+		EXPECT_DOUBLE_EQ(0.0, s0p1Factor);
+		EXPECT_DOUBLE_EQ(1.0, s1p1Factor);
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=0.5");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.1, 0.0, 1.0), VectorType(-0.1, 0.0, -1.0));
+		// A(0.5) = 0.0 0.0 0.0
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 3.0, -2.0));
+		// B(0.5) = 1.5 1.5 0.0
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(-0.3, 2.0, -2.0), VectorType(0.3, 1.0, 2.0));
+		// C(0.5) = 0.0 1.5 0.0
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(1.7, 1.0, -1.0), VectorType(1.3, -1.0, 1.0));
+		// D(0.5) = 1.5 0.0 0.0
+		// At time 0.5, the segments AB and CD are coplanar and intersect exactly in their middle
+		EXPECT_TRUE(calculateCcdContactSegmentSegment(A, B, C, D, &time, &s0p1Factor, &s1p1Factor));
+		EXPECT_NEAR(0.5, time, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.5, s0p1Factor, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.5, s1p1Factor, Math::Geometry::ScalarEpsilon);
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=0.5");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.4, 0.0, 1.0), VectorType(0.6, 0.0, -1.0));
+		// A(0.5) = 0.5 0.0 0.0
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 3.0, -2.0));
+		// B(0.5) = 1.5 1.5 0.0
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(-0.3, 2.0, -2.0), VectorType(0.3, 1.0, 2.0));
+		// C(0.5) = 0.0 1.5 0.0
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(1.7, 1.0, -1.0), VectorType(1.3, -1.0, 1.0));
+		// D(0.5) = 1.5 0.0 0.0
+		// At time 0.5, the segments AB and CD are coplanar and intersect at P:
+		// P = A + 0.4 AB = (0.5 0.0 0.0) + 0.4 (1.0 1.5 0.0)  = (0.9 0.6 0.0)
+		// P = C + 0.6 CD = (0.0 1.5 0.0) + 0.6 (1.5 -1.5 0.0) = (0.9 0.6 0.0)
+		EXPECT_TRUE(calculateCcdContactSegmentSegment(A, B, C, D, &time, &s0p1Factor, &s1p1Factor));
+		EXPECT_NEAR(0.5, time, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.4, s0p1Factor, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.6, s1p1Factor, Math::Geometry::ScalarEpsilon);
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=0.5 (2 cubic roots in [0..1]: 0.0899 (no collision) and 0.5 (collision))");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.4, 0.8, 1.0), VectorType(0.8, 1.0, -1.0));
+		// A(0.5) = 0.6 0.9 0.0
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 3.0, -2.0));
+		// B(0.5) = 1.5 1.5 0.0
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(-0.3, 2.0, -2.0), VectorType(0.3, 1.0, 2.0));
+		// C(0.5) = 0.0 1.5 0.0
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(1.7, 1.0, -1.0), VectorType(1.3, -1.0, 1.0));
+		// D(0.5) = 1.5 0.0 0.0
+		// At time 0.5, all 4 points are coplanar and AB/CD intersect:
+		// P = A +   0*AB = (0.6 0.9 0.0)
+		// P = C + 0.4*CD = (0.0 1.5 0.0) + 0.4 (1.5 -1.5 0.0) = (0.6 0.9 0.0)
+		EXPECT_TRUE(calculateCcdContactSegmentSegment(A, B, C, D, &time, &s0p1Factor, &s1p1Factor));
+		EXPECT_NEAR(0.5, time, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.0, s0p1Factor, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.4, s1p1Factor, Math::Geometry::ScalarEpsilon);
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=1");
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(1.0, 0.0, 1.24), VectorType(2.0, 0.0, 3.0));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.0, 1.0, -2.0), VectorType(2.0, 0.0, 3.0));
+		std::pair<VectorType, VectorType> D = std::make_pair(VectorType(0.1, 1.0, 1.1), VectorType(0.0, 1.0, -1.54));
+		EXPECT_TRUE(calculateCcdContactSegmentSegment(A, B, C, D, &time, &s0p1Factor, &s1p1Factor));
+		EXPECT_DOUBLE_EQ(1.0, time);
+		EXPECT_DOUBLE_EQ(1.0, s0p1Factor);
+		EXPECT_DOUBLE_EQ(0.0, s1p1Factor);
+	}
+}
+
+TEST_F(GeometryTest, CcdIntersectionsPointTriangle)
+{
+	SizeType time, tv01Factor, tv02Factor;
+
+	{
+		SCOPED_TRACE("No intersection");
+		std::pair<VectorType, VectorType> P = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 0.0, 4.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(0.0, 0.1, 4.0), VectorType(0.0, 1.1, 8.0));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.0, 1.1, 6.0), VectorType(0.0, 2.1, 54.0));
+		EXPECT_FALSE(calculateCcdContactPointTriangle(P, A, B, C, &time, &tv01Factor, &tv02Factor));
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=0");
+		std::pair<VectorType, VectorType> P = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 0.0, 4.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(0.0, 1.0, -2.0), VectorType(0.0, 2.0, -3.5));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(0.0, 1.0, -10.0));
+		EXPECT_TRUE(calculateCcdContactPointTriangle(P, A, B, C, &time, &tv01Factor, &tv02Factor));
+		EXPECT_DOUBLE_EQ(0.0, time);
+		EXPECT_DOUBLE_EQ(0.0, tv01Factor);
+		EXPECT_DOUBLE_EQ(1.0, tv02Factor);
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=0.5");
+		std::pair<VectorType, VectorType> P = std::make_pair(VectorType(1.1, 0.0, 1.0), VectorType(0.9, 2.0, -1.0));
+		// P(0.5) = 1.0 1.0 0.0
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 3.0, -2.0));
+		// A(0.5) = 1.5 1.5 0.0
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(-0.3, 2.0, -2.0), VectorType(0.3, 1.0, 2.0));
+		// B(0.5) = 0.0 1.5 0.0
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(1.7, 1.0, -1.0), VectorType(1.3, -1.0, 1.0));
+		// C(0.5) = 1.5 0.0 0.0
+		// At time 0.5, all 4 points are coplanar and P is inside ABC with the barycentric coordinates (1/3 1/3 1/3)
+		EXPECT_TRUE(calculateCcdContactPointTriangle(P, A, B, C, &time, &tv01Factor, &tv02Factor));
+		EXPECT_NEAR(0.5, time, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(1.0 / 3.0, tv01Factor, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(1.0 / 3.0, tv01Factor, Math::Geometry::ScalarEpsilon);
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=0.5 (2 cubic roots in [0..1]: 0.0899 (no collision) and 0.5 (collision))");
+		std::pair<VectorType, VectorType> P = std::make_pair(VectorType(0.4, 0.8, 1.0), VectorType(0.8, 1.0, -1.0));
+		// P(0.5) = 0.6 0.9 0.0
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(1.0, 0.0, 2.0), VectorType(2.0, 3.0, -2.0));
+		// A(0.5) = 1.5 1.5 0.0
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(-0.3, 2.0, -2.0), VectorType(0.3, 1.0, 2.0));
+		// B(0.5) = 0.0 1.5 0.0
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(1.7, 1.0, -1.0), VectorType(1.3, -1.0, 1.0));
+		// C(0.5) = 1.5 0.0 0.0
+		// At time 0.5, all 4 points are coplanar and P is inside ABC with the barycentric coordinates (0 0.6 0.4)
+		// P = A + 0.6 AB + 0.4 AC = (1.5 1.5 0.0) + 0.6 (-1.5 0.0 0.0) + 0.4 (0.0 -1.5 0.0) = (0.6 0.9 0.0)
+		EXPECT_TRUE(calculateCcdContactPointTriangle(P, A, B, C, &time, &tv01Factor, &tv02Factor));
+		EXPECT_NEAR(0.5, time, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.6, tv01Factor, Math::Geometry::ScalarEpsilon);
+		EXPECT_NEAR(0.4, tv02Factor, Math::Geometry::ScalarEpsilon);
+	}
+
+	{
+		SCOPED_TRACE("Intersection at t=1");
+		std::pair<VectorType, VectorType> P = std::make_pair(VectorType(1.0, 0.0, 1.24), VectorType(2.0, 0.0, 3.0));
+		std::pair<VectorType, VectorType> A = std::make_pair(VectorType(0.0, 0.0, 0.0), VectorType(1.0, 0.0, 1.0));
+		std::pair<VectorType, VectorType> B = std::make_pair(VectorType(0.0, 1.0, -2.0), VectorType(2.0, 0.0, 3.0));
+		std::pair<VectorType, VectorType> C = std::make_pair(VectorType(0.1, 1.0, 1.1), VectorType(0.0, 1.0, -1.54));
+		EXPECT_TRUE(calculateCcdContactPointTriangle(P, A, B, C, &time, &tv01Factor, &tv02Factor));
+		EXPECT_DOUBLE_EQ(1.0, time);
+		EXPECT_DOUBLE_EQ(1.0, tv01Factor);
+		EXPECT_DOUBLE_EQ(0.0, tv02Factor);
+	}
+}
+
 }; // namespace Math
 }; // namespace SurgSim
