@@ -173,6 +173,7 @@ TYPED_TEST(QuaternionTests, SetToZero)
 // Test conversion to and from yaml node
 TYPED_TEST(QuaternionTests, YamlConvert)
 {
+	typedef typename TestFixture::AngleAxis AngleAxis;
 	typedef typename TestFixture::Quaternion Quaternion;
 	typedef typename TestFixture::Scalar T;
 
@@ -187,6 +188,15 @@ TYPED_TEST(QuaternionTests, YamlConvert)
 
 	ASSERT_NO_THROW({Quaternion expected = node.as<Quaternion>();});
 	EXPECT_TRUE(quaternion.isApprox(node.as<Quaternion>()));
+
+	// test decoding from AngleAxis
+	AngleAxis angleAxis;
+	angleAxis.angle() = T(0.1);
+	angleAxis.axis()[0] = T(0.1);
+	angleAxis.axis()[1] = T(1.2);
+	angleAxis.axis()[2] = T(2.3);
+	YAML::Node angleAxisNode = YAML::convert<AngleAxis>::encode(angleAxis);
+	EXPECT_TRUE(Quaternion(angleAxis).isApprox(angleAxisNode.as<Quaternion>()));
 }
 
 
