@@ -1,17 +1,17 @@
-//// This file is a part of the OpenSurgSim project.
-//// Copyright 2013, SimQuest Solutions Inc.
-////
-//// Licensed under the Apache License, Version 2.0 (the "License");
-//// you may not use this file except in compliance with the License.
-//// You may obtain a copy of the License at
-////
-////     http://www.apache.org/licenses/LICENSE-2.0
-////
-//// Unless required by applicable law or agreed to in writing, software
-//// distributed under the License is distributed on an "AS IS" BASIS,
-//// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-//// See the License for the specific language governing permissions and
-//// limitations under the License.
+// This file is a part of the OpenSurgSim project.
+// Copyright 2013, SimQuest Solutions Inc.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
 #include <gtest/gtest.h>
 
@@ -22,23 +22,19 @@
 #include "SurgSim/Math/Shape.h"
 #include "SurgSim/Math/Shapes.h"
 #include "SurgSim/Math/Vector.h"
+#include "SurgSim/DataStructures/OctreeNode.h"
 
-using SurgSim::Math::Vector3d;
-using SurgSim::Math::Matrix33d;
 
-using SurgSim::Math::BoxShape;
-using SurgSim::Math::CapsuleShape;
-using SurgSim::Math::CylinderShape;
-using SurgSim::Math::DoubleSidedPlaneShape;
-using SurgSim::Math::OctreeShape;
-using SurgSim::Math::PlaneShape;
-using SurgSim::Math::Shape;
-using SurgSim::Math::SphereShape;
-
-namespace {
-	const double epsilon = 1e-10;
+namespace
+{
+const double epsilon = 1e-10;
 }
 
+namespace SurgSim
+{
+namespace Math
+{
+
 class ShapeTest : public ::testing::Test
 {
 public:
@@ -135,17 +131,22 @@ TEST_F(ShapeTest, Sphere)
 	double coef = 2.0 / 5.0 * expectedMass * r2;
 	Matrix33d expectedInertia;
 	expectedInertia << coef, 0.0, 0.0,
-					   0.0, coef, 0.0,
-					   0.0, 0.0, coef;
+					0.0, coef, 0.0,
+					0.0, 0.0, coef;
 
 	double volume = sphere.getVolume();
 	Vector3d center = sphere.getCenter();
 	Matrix33d inertia = sphere.getSecondMomentOfVolume() * m_rho;
 
+	Math::Aabbd aabb(Vector3d(-m_radius, -m_radius, -m_radius));
+	aabb.extend(Vector3d(m_radius, m_radius, m_radius));
+
 	EXPECT_NEAR(expectedVolume, volume, epsilon);
 	EXPECT_TRUE(center.isZero());
 	EXPECT_TRUE(expectedInertia.isApprox(inertia));
 	EXPECT_TRUE(sphere.isValid());
+	EXPECT_FALSE(sphere.isTransformable());
+	EXPECT_TRUE(aabb.isApprox(sphere.getBoundingBox()));
 }
 
 TEST_F(ShapeTest, BoxSerializationTest)
@@ -222,16 +223,21 @@ TEST_F(ShapeTest, Box)
 	double z2 = m_size[2] * m_size[2];
 	Matrix33d expectedInertia;
 	expectedInertia << coef * (y2 + z2), 0.0, 0.0,
-					   0.0, coef * (x2 + z2), 0.0,
-					   0.0, 0.0, coef * (x2 + y2);
+					0.0, coef * (x2 + z2), 0.0,
+					0.0, 0.0, coef * (x2 + y2);
 
 	double volume = box.getVolume();
 	Vector3d center = box.getCenter();
 	Matrix33d inertia = box.getSecondMomentOfVolume() * m_rho;
 
+	Aabbd aabb(size / 2.0);
+	aabb.extend(-(size / 2.0));
+
 	EXPECT_NEAR(expectedVolume, volume, epsilon);
 	EXPECT_TRUE(center.isZero());
 	EXPECT_TRUE(expectedInertia.isApprox(inertia));
+	EXPECT_FALSE(box.isTransformable());
+	EXPECT_TRUE(aabb.isApprox(box.getBoundingBox()));
 }
 
 TEST_F(ShapeTest, CylinderSerializationTest)
@@ -305,17 +311,21 @@ TEST_F(ShapeTest, Cylinder)
 	double coef    = 1.0 / 12.0 * expectedMass * (3.0 * (r1sq) + l2);
 	Matrix33d expectedInertia;
 	expectedInertia << coef, 0.0, 0.0,
-					   0.0, coefDir, 0.0,
-					   0.0, 0.0, coef;
+					0.0, coefDir, 0.0,
+					0.0, 0.0, coef;
 
 	double volume = cylinder.getVolume();
 	Vector3d center = cylinder.getCenter();
 	Matrix33d inertia = cylinder.getSecondMomentOfVolume() * m_rho;
+	Aabbd aabb(Vector3d(-m_radius, -m_length / 2.0, -m_radius));
+	aabb.extend(Vector3d(m_radius, m_length / 2.0, m_radius));
 
 	EXPECT_NEAR(expectedVolume, volume, epsilon);
 	EXPECT_TRUE(center.isZero());
 	EXPECT_TRUE(expectedInertia.isApprox(inertia));
 	EXPECT_TRUE(cylinder.isValid());
+	EXPECT_FALSE(cylinder.isTransformable());
+	EXPECT_TRUE(aabb.isApprox(cylinder.getBoundingBox()));
 }
 
 TEST_F(ShapeTest, CapsuleSerializationTest)
@@ -396,17 +406,23 @@ TEST_F(ShapeTest, Capsule)
 	coef += 1.0 / 12.0 * massCylinder * (3 * r2 + l2);
 	Matrix33d expectedInertia;
 	expectedInertia << coef, 0.0, 0.0,
-					   0.0, coefDir, 0.0,
-					   0.0, 0.0, coef;
+					0.0, coefDir, 0.0,
+					0.0, 0.0, coef;
 
 	double volume = capsule.getVolume();
 	Vector3d center = capsule.getCenter();
 	Matrix33d inertia = capsule.getSecondMomentOfVolume() * m_rho;
 
+	Aabbd aabb(Vector3d(-m_radius, -m_length / 2.0 - m_radius, -m_radius));
+	aabb.extend(Vector3d(m_radius, m_length / 2.0 + m_radius, m_radius));
+
+
 	EXPECT_NEAR(expectedVolume, volume, epsilon);
 	EXPECT_TRUE(center.isZero());
 	EXPECT_TRUE(expectedInertia.isApprox(inertia));
 	EXPECT_TRUE(capsule.isValid());
+	EXPECT_FALSE(capsule.isTransformable());
+	EXPECT_TRUE(aabb.isApprox(capsule.getBoundingBox()));
 }
 
 TEST_F(ShapeTest, DoubleSidedPlaneShapeSerializationTest)
@@ -439,7 +455,7 @@ TEST_F(ShapeTest, DoubleSidedPlaneShapeSerializationTest)
 
 		std::shared_ptr<DoubleSidedPlaneShape> doubleSidedPlaneShape;
 		ASSERT_NO_THROW(doubleSidedPlaneShape =
-			std::dynamic_pointer_cast<DoubleSidedPlaneShape>(node.as<std::shared_ptr<Shape>>()));
+							std::dynamic_pointer_cast<DoubleSidedPlaneShape>(node.as<std::shared_ptr<Shape>>()));
 		EXPECT_EQ("SurgSim::Math::DoubleSidedPlaneShape", doubleSidedPlaneShape->getClassName());
 		EXPECT_TRUE(doubleSidedPlaneShape->isValid());
 	}
@@ -457,98 +473,98 @@ TEST_F(ShapeTest, DoubleSidedPlaneShape)
 	EXPECT_NEAR(0.0, doubleSidedPlaneShape.getD(), epsilon);
 	EXPECT_TRUE(doubleSidedPlaneShape.getNormal().isApprox(Vector3d(0.0, 1.0, 0.0)));
 	EXPECT_TRUE(doubleSidedPlaneShape.isValid());
+	EXPECT_FALSE(doubleSidedPlaneShape.isTransformable());
+
+	// There is no sense to trying to build the Bounding box here
+	EXPECT_TRUE(doubleSidedPlaneShape.getBoundingBox().isEmpty());
 }
 
 
-TEST_F(ShapeTest, OctreeSerializationTest)
+TEST_F(ShapeTest, OctreeShapeSerializationTest)
 {
-	const std::string fileName = "OctreeShapeData/staple.vox";
+	const std::string fileName = "Geometry/staple.ply";
 	SurgSim::Framework::Runtime runtime("config.txt");
 
-	{
-		YAML::Node node;
-		node["SurgSim::Math::OctreeShape"]["FileName"] = fileName;
+	auto shape = std::make_shared<OctreeShape>();
 
-		std::shared_ptr<Shape> shape;
-		ASSERT_NO_THROW(shape = node.as<std::shared_ptr<Shape>>());
+	ASSERT_NO_THROW(shape->loadOctree(fileName));
 
-		EXPECT_EQ("SurgSim::Math::OctreeShape", shape->getClassName());
-		EXPECT_EQ(fileName, shape->getValue<std::string>("FileName"));
-		EXPECT_TRUE(shape->isValid());
-	}
+	YAML::Node node = YAML::convert<std::shared_ptr<SurgSim::Math::Shape>>::encode(shape);
 
-	{
-		std::shared_ptr<Shape> shape;
-		ASSERT_NO_THROW(shape = Shape::getFactory().create("SurgSim::Math::OctreeShape"));
-		shape->setValue("FileName", fileName);
-		EXPECT_TRUE(shape->isValid());
+	EXPECT_TRUE(node[shape->getClassName()].IsMap());
+	EXPECT_TRUE(node[shape->getClassName()]["Octree"].IsMap());
 
-		YAML::Node node;
-		ASSERT_NO_THROW(node = shape);
-		EXPECT_TRUE(node.IsMap());
-		EXPECT_EQ(1u, node.size());
+	std::shared_ptr<SurgSim::Math::Shape> decodedShape;
+	ASSERT_NO_THROW(decodedShape = node.as<std::shared_ptr<SurgSim::Math::Shape>>());
+	ASSERT_NE(nullptr, decodedShape);
 
-		ASSERT_TRUE(node["SurgSim::Math::OctreeShape"].IsDefined());
-		auto data = node["SurgSim::Math::OctreeShape"];
-		EXPECT_EQ(1u, data.size());
+	auto convertedShape = std::dynamic_pointer_cast<OctreeShape>(decodedShape);
+	ASSERT_NE(nullptr, convertedShape);
 
-		std::shared_ptr<OctreeShape> newOctreeShape;
-		ASSERT_NO_THROW(newOctreeShape = std::dynamic_pointer_cast<OctreeShape>(node.as<std::shared_ptr<Shape>>()));
-		EXPECT_EQ("SurgSim::Math::OctreeShape", newOctreeShape->getClassName());
-		EXPECT_EQ(fileName, newOctreeShape->getFileName());
-		EXPECT_TRUE(newOctreeShape->isValid());
-	}
+	ASSERT_NE(nullptr, convertedShape->getOctree());
+	EXPECT_EQ(fileName, convertedShape->getOctree()->getFileName());
 }
 
 TEST_F(ShapeTest, OctreeShape)
 {
 	OctreeShape::NodeType::AxisAlignedBoundingBox boundingBox(Vector3d::Zero(), m_size);
 	std::shared_ptr<OctreeShape::NodeType> node = std::make_shared<OctreeShape::NodeType>(boundingBox);
+	SurgSim::Framework::Runtime runtime("config.txt");
+
 
 	{
-		ASSERT_NO_THROW({OctreeShape octree; EXPECT_FALSE(octree.isValid());});
-		ASSERT_NO_THROW({OctreeShape octree(*node); EXPECT_TRUE(octree.isValid());});
+		ASSERT_NO_THROW({OctreeShape shape; EXPECT_FALSE(shape.isValid());});
+		ASSERT_NO_THROW({OctreeShape shape(*node); EXPECT_TRUE(shape.isValid());});
 	}
 
 	{
-		OctreeShape octree;
-		EXPECT_EQ(nullptr, octree.getRootNode());
-		octree.setRootNode(node);
-		EXPECT_EQ(node, octree.getRootNode());
-		EXPECT_TRUE(octree.isValid());
+		OctreeShape shape;
+		EXPECT_NE(nullptr, shape.getOctree());
+		shape.setOctree(node);
+		EXPECT_EQ(node, shape.getOctree());
+		EXPECT_TRUE(shape.isValid());
 	}
 
 	{
-		SurgSim::Framework::Runtime runtime("config.txt");
-		const std::string fileName = "OctreeShapeData/staple.vox";
-		OctreeShape octree;
-		EXPECT_NO_THROW(octree.setRootNode(node));
-		EXPECT_NO_THROW(octree.load(fileName));
-		EXPECT_EQ(fileName, octree.getFileName());
+		SCOPED_TRACE("Normal Loading");
+		const std::string fileName = "Geometry/staple.ply";
+		OctreeShape shape;
+		EXPECT_NO_THROW(shape.setOctree(node));
+		EXPECT_NO_THROW(shape.loadOctree(fileName));
+		EXPECT_EQ(fileName, shape.getOctree()->getFileName());
 
-		EXPECT_EQ(octree.getClassName(), "SurgSim::Math::OctreeShape");
-		EXPECT_EQ(SurgSim::Math::SHAPE_TYPE_OCTREE, octree.getType());
-		EXPECT_THROW(octree.getVolume(), SurgSim::Framework::AssertionFailure);
-		EXPECT_TRUE(octree.getCenter().isApprox(Vector3d::Zero(), epsilon));
-		EXPECT_THROW(octree.getSecondMomentOfVolume(), SurgSim::Framework::AssertionFailure);
-		EXPECT_EQ(fileName, octree.getFileName());
-		EXPECT_TRUE(octree.isValid());
+		EXPECT_EQ(shape.getClassName(), "SurgSim::Math::OctreeShape");
+		EXPECT_EQ(SurgSim::Math::SHAPE_TYPE_OCTREE, shape.getType());
+		EXPECT_THROW(shape.getVolume(), SurgSim::Framework::AssertionFailure);
+		EXPECT_TRUE(shape.getCenter().isApprox(Vector3d::Zero(), epsilon));
+		EXPECT_THROW(shape.getSecondMomentOfVolume(), SurgSim::Framework::AssertionFailure);
+		EXPECT_EQ(fileName, shape.getOctree()->getFileName());
+		EXPECT_TRUE(shape.isValid());
+		EXPECT_TRUE(shape.getOctree()->getBoundingBox().isApprox(shape.getBoundingBox()));
+	}
+
+	{
+		SCOPED_TRACE("Alternative load through property");
+		const std::string fileName = "Geometry/staple.ply";
+		OctreeShape shape;
+		EXPECT_NO_THROW(shape.setValue("OctreeFileName", fileName));
+		EXPECT_EQ(fileName, shape.getOctree()->getFileName());
 	}
 
 	{
 		SCOPED_TRACE("Load nonexistent file will throw");
 		SurgSim::Framework::ApplicationData appData("config.txt");
 		const std::string fileName = "Nonexistent file";
-		OctreeShape octree;
-		EXPECT_ANY_THROW(octree.load(fileName, appData));
+		OctreeShape shape;
+		EXPECT_ANY_THROW(shape.loadOctree(fileName));
 	}
 
 	{
 		SCOPED_TRACE("Load existent file containing invalid Octree will throw");
 		SurgSim::Framework::ApplicationData appData("config.txt");
-		const std::string fileName = "OctreeShapeData/invalid-staple.vox";
-		OctreeShape octree;
-		EXPECT_ANY_THROW(octree.load(fileName, appData));
+		const std::string fileName = "Geometry/invalid-staple.ply";
+		OctreeShape shape;
+		EXPECT_ANY_THROW(shape.loadOctree(fileName));
 	}
 }
 
@@ -599,4 +615,10 @@ TEST_F(ShapeTest, PlaneShape)
 	EXPECT_NEAR(0.0, planeShape.getD(), epsilon);
 	EXPECT_TRUE(planeShape.getNormal().isApprox(Vector3d(0.0, 1.0, 0.0)));
 	EXPECT_TRUE(planeShape.isValid());
-}
\ No newline at end of file
+	EXPECT_FALSE(planeShape.isTransformable());
+	EXPECT_TRUE(planeShape.getBoundingBox().isEmpty());
+}
+
+
+}
+}
