@@ -16,6 +16,7 @@
 #ifndef SURGSIM_PHYSICS_BUILDMLCP_H
 #define SURGSIM_PHYSICS_BUILDMLCP_H
 
+#include "SurgSim/Framework/Macros.h"
 #include "SurgSim/Physics/Computation.h"
 
 namespace SurgSim
@@ -31,13 +32,15 @@ public:
 	/// \param doCopyState Specify if the output state in Computation::Update() is a copy or not of the input state
 	explicit BuildMlcp(bool doCopyState = false);
 
+	SURGSIM_CLASSNAME(SurgSim::Physics::BuildMlcp);
+
 	/// Destructor
 	virtual ~BuildMlcp();
 
 protected:
 	/// Override doUpdate from superclass
-	virtual std::shared_ptr<PhysicsManagerState>
-		doUpdate(const double& dt, const std::shared_ptr<PhysicsManagerState>& state) override;
+	std::shared_ptr<PhysicsManagerState> doUpdate(const double& dt, const std::shared_ptr<PhysicsManagerState>& state)
+		override;
 };
 
 }; // namespace Physics
