@@ -1,5 +1,5 @@
 // This file is a part of the OpenSurgSim project.
-// Copyright 2014, SimQuest Solutions Inc.
+// Copyright 2015, SimQuest Solutions Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -16,31 +16,51 @@
 #ifndef SURGSIM_PHYSICS_FEM3DPLYREADERDELEGATE_H
 #define SURGSIM_PHYSICS_FEM3DPLYREADERDELEGATE_H
 
+#include <array>
 #include <memory>
 
+#include "SurgSim/DataStructures/EmptyData.h"
+#include "SurgSim/DataStructures/PlyReader.h"
+#include "SurgSim/Physics/Fem3D.h"
 #include "SurgSim/Physics/FemPlyReaderDelegate.h"
 
 namespace SurgSim
 {
 namespace Physics
 {
-class Fem3DRepresentation;
 
-/// Implementation of PlyReaderDelegate for Fem3DRepresentation
 class Fem3DPlyReaderDelegate : public FemPlyReaderDelegate
 {
 public:
-	/// Constructor
-	/// \param fem The object that is updated when PlyReader::parseFile is called.
-	explicit Fem3DPlyReaderDelegate(std::shared_ptr<Fem3DRepresentation> fem);
+	/// Default constructor.
+	Fem3DPlyReaderDelegate();
+
+	/// Constructor.
+	/// \param mesh The mesh to be used, it will be cleared by the constructor.
+	explicit Fem3DPlyReaderDelegate(std::shared_ptr<Fem3D> mesh);
 
 protected:
-	virtual std::string getElementName() const override;
+	std::string getElementName() const override;
 
-	virtual void processFemElement(const std::string& elementName) override;
+	bool fileIsAcceptable(const PlyReader& reader) override;
+
+	void endParseFile() override;
+
+	void processVertex(const std::string& elementName) override;
+
+	void processFemElement(const std::string& elementName) override;
+
+	void processBoundaryCondition(const std::string& elementName) override;
+
+	/// End file callback
+	void endFile();
+
+private:
+	/// Fem3D mesh asset to contain the ply file information
+	std::shared_ptr<Fem3D> m_mesh;
 };
 
 } // namespace Physics
 } // namespace SurgSim
 
-#endif // SURGSIM_PHYSICS_FEM3DPLYREADERDELEGATE_H
\ No newline at end of file
+#endif // SURGSIM_PHYSICS_FEM3DPLYREADERDELEGATE_H
