@@ -19,7 +19,7 @@
 #include <memory>
 #include <vector>
 
-
+#include "SurgSim/Framework/Macros.h"
 #include "SurgSim/Physics/Computation.h"
 
 namespace SurgSim
@@ -29,7 +29,7 @@ namespace Physics
 
 class Representation;
 
-/// Apply the Freemotion calcluation to all physics representations
+/// Apply the FreeMotion calculation to all physics representations
 class FreeMotion  : public Computation
 {
 public:
@@ -38,19 +38,18 @@ public:
 	/// \param doCopyState Specify if the output state in Computation::Update() is a copy or not of the input state
 	explicit FreeMotion(bool doCopyState = false);
 
+	SURGSIM_CLASSNAME(SurgSim::Physics::FreeMotion);
+
 	/// Destructor
 	~FreeMotion();
 
 protected:
-
 	/// Override doUpdate from superclass
-	virtual std::shared_ptr<PhysicsManagerState> doUpdate(
-		const double& dt,
-		const std::shared_ptr<PhysicsManagerState>& state) override;
-
+	std::shared_ptr<PhysicsManagerState> doUpdate(const double& dt, const std::shared_ptr<PhysicsManagerState>& state)
+		override;
 };
 
 }; // Physics
 }; // SurgSim
 
-#endif
\ No newline at end of file
+#endif
