@@ -1,5 +1,5 @@
 // This file is a part of the OpenSurgSim project.
-// Copyright 2013, SimQuest Solutions Inc.
+// Copyright 2013-2016, SimQuest Solutions Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -40,10 +40,35 @@ std::shared_ptr<PhysicsManagerState> PostUpdate::doUpdate(
 	const std::shared_ptr<PhysicsManagerState>& state)
 {
 	std::shared_ptr<PhysicsManagerState> result = state;
-	std::vector<std::shared_ptr<Representation>> representations = result->getRepresentations();
-	for (auto it = representations.begin(); it != representations.end(); ++it)
 	{
-		(*it)->afterUpdate(dt);
+		const auto& representations = result->getActiveRepresentations();
+		for (auto& representation : representations)
+		{
+			representation->afterUpdate(dt);
+		}
+	}
+
+	{
+		// Clear the collisions
+		auto const& representations = result->getCollisionRepresentations();
+
+		for (auto& representation : representations)
+		{
+			representation->getCollisions().unsafeGet().clear();
+		}
+
+		// Update the representations with the contact data.
+		auto& pairs = result->getCollisionPairs();
+		for (auto& pair : pairs)
+		{
+			pair->updateRepresentations();
+		}
+
+		// Publish Results
+		for (auto& representation : representations)
+		{
+			representation->getCollisions().publish();
+		}
 	}
 
 	return result;
