@@ -18,6 +18,7 @@
 
 #include <memory>
 
+#include "SurgSim/Framework/Macros.h"
 #include "SurgSim/Physics/Computation.h"
 
 namespace SurgSim
@@ -33,13 +34,15 @@ public:
 	/// \param doCopyState Specify if the output state in Computation::Update() is a copy or not of the input state
 	explicit PreUpdate(bool doCopyState = false);
 
+	SURGSIM_CLASSNAME(SurgSim::Physics::PreUpdate);
+
 	/// Destructor
 	virtual ~PreUpdate();
 
 protected:
 	/// Override doUpdate from superclass
-	virtual std::shared_ptr<PhysicsManagerState>
-		doUpdate(const double& dt, const std::shared_ptr<PhysicsManagerState>& state) override;
+	std::shared_ptr<PhysicsManagerState> doUpdate(const double& dt, const std::shared_ptr<PhysicsManagerState>& state)
+		override;
 };
 
 }; // Physics
