@@ -16,10 +16,16 @@
 #ifndef SURGSIM_PHYSICS_PUSHRESULTS_H
 #define SURGSIM_PHYSICS_PUSHRESULTS_H
 
+#include "SurgSim/Framework/Macros.h"
 #include "SurgSim/Physics/Computation.h"
 
 namespace SurgSim
 {
+namespace Framework
+{
+class Logger;
+}
+
 namespace Physics
 {
 
@@ -33,14 +39,18 @@ public:
 	/// \param doCopyState Specify if the output state in Computation::Update() is a copy or not of the input state
 	explicit PushResults(bool doCopyState = false);
 
+	SURGSIM_CLASSNAME(SurgSim::Physics::PushResults);
+
 	/// Destructor
 	virtual ~PushResults();
 
 protected:
 
 	/// Override doUpdate from superclass
-	virtual std::shared_ptr<PhysicsManagerState>
-		doUpdate(const double& dt, const std::shared_ptr<PhysicsManagerState>& state) override;
+	std::shared_ptr<PhysicsManagerState> doUpdate(const double& dt, const std::shared_ptr<PhysicsManagerState>& state)
+	override;
+
+	std::shared_ptr<SurgSim::Framework::Logger> m_logger;
 };
 
 }; // namespace Physics
