@@ -143,38 +143,21 @@ TEST_F(BuildMlcpTests, OneRepresentationOneConstraintTest)
 	m_usedRepresentations.push_back(m_fixedWorldRepresentation);
 	// Set the representation list in the Physics Manager State
 	m_physicsManagerState->setRepresentations(m_usedRepresentations);
+	// The type of constraint.
+	auto constraintType = SurgSim::Physics::FRICTIONLESS_3DCONTACT;
 
 	// Prep the list of constraints: use only 1 constraint
 	{
-		std::shared_ptr<Localization> rigidLocalization;
-		{
-			std::shared_ptr<RigidRepresentationLocalization> rigidLocalizationTyped;
-			rigidLocalizationTyped = std::make_shared<RigidRepresentationLocalization>();
-			rigidLocalizationTyped->setRepresentation(m_usedRepresentations[0]);
-			rigidLocalizationTyped->setLocalPosition(SurgSim::Math::Vector3d::Zero());
-			rigidLocalization = rigidLocalizationTyped;
-		}
-		std::shared_ptr<RigidRepresentationContact> rigidSideContact;
-		rigidSideContact = std::make_shared<RigidRepresentationContact>();
-
-		std::shared_ptr<Localization> fixedLocalization;
-		{
-			std::shared_ptr<FixedRepresentationLocalization> fixedLocalizationTyped;
-			fixedLocalizationTyped = std::make_shared<FixedRepresentationLocalization>();
-			fixedLocalizationTyped->setRepresentation(m_fixedWorldRepresentation);
-			fixedLocalizationTyped->setLocalPosition(SurgSim::Math::Vector3d::Zero());
-			fixedLocalization = fixedLocalizationTyped;
-		}
-		std::shared_ptr<FixedRepresentationContact> fixedSideContact;
-		fixedSideContact = std::make_shared<FixedRepresentationContact>();
-
 		// Define the constraint specific data
 		std::shared_ptr<ContactConstraintData> data = std::make_shared<ContactConstraintData>();
 		data->setPlaneEquation(SurgSim::Math::Vector3d(0.0, 1.0, 0.0), 0.0);
 
 		// Set up the constraint
-		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(
-			data, rigidSideContact, rigidLocalization, fixedSideContact, fixedLocalization);
+		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(constraintType,
+			data, m_usedRepresentations[0],
+			SurgSim::DataStructures::Location(SurgSim::Math::Vector3d::Zero()),
+			m_fixedWorldRepresentation,
+			SurgSim::DataStructures::Location(SurgSim::Math::Vector3d::Zero()));
 
 		// Register the constraint in the list of used constraints for this test
 		m_usedConstraints.push_back(constraint);
@@ -218,37 +201,26 @@ TEST_F(BuildMlcpTests, TwoRepresentationsOneConstraintSize3Test)
 	m_usedRepresentations.push_back(m_fixedWorldRepresentation);
 	// Set the representation list in the Physics Manager State
 	m_physicsManagerState->setRepresentations(m_usedRepresentations);
+	// The type of constraint.
+	auto constraintType = SurgSim::Physics::FIXED_3DPOINT;
+	// Add constraint to factory.
+	ConstraintImplementation::getFactory().addImplementation(
+		typeid(RigidRepresentation),
+		std::make_shared<MockRigidConstraintFixedPoint>());
+	ConstraintImplementation::getFactory().addImplementation(
+		typeid(FixedRepresentation),
+		std::make_shared<MockFixedConstraintFixedPoint>());
 
 	// Prep the list of constraints: use only 1 constraint
 	{
-		std::shared_ptr<Localization> rigidLocalization;
-		{
-			std::shared_ptr<RigidRepresentationLocalization> rigidLocalizationTyped;
-			rigidLocalizationTyped = std::make_shared<RigidRepresentationLocalization>();
-			rigidLocalizationTyped->setRepresentation(m_usedRepresentations[0]);
-			rigidLocalizationTyped->setLocalPosition(SurgSim::Math::Vector3d::Zero());
-			rigidLocalization = rigidLocalizationTyped;
-		}
-		std::shared_ptr<MockRigidConstraintBilateral3D> rigidSideContact;
-		rigidSideContact = std::make_shared<MockRigidConstraintBilateral3D>();
-
-		std::shared_ptr<Localization> fixedLocalization;
-		{
-			std::shared_ptr<FixedRepresentationLocalization> fixedLocalizationTyped;
-			fixedLocalizationTyped = std::make_shared<FixedRepresentationLocalization>();
-			fixedLocalizationTyped->setRepresentation(m_fixedWorldRepresentation);
-			fixedLocalizationTyped->setLocalPosition(SurgSim::Math::Vector3d::Zero());
-			fixedLocalization = fixedLocalizationTyped;
-		}
-		std::shared_ptr<MockFixedConstraintBilateral3D> fixedSideContact;
-		fixedSideContact = std::make_shared<MockFixedConstraintBilateral3D>();
-
-		// Define the constraint specific data
-		std::shared_ptr<ConstraintData> data = std::make_shared<ConstraintData>();
+		auto data = std::make_shared<ConstraintData>();
 
 		// Set up the constraint
-		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(
-			data, rigidSideContact, rigidLocalization, fixedSideContact, fixedLocalization);
+		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(constraintType,
+			data, m_usedRepresentations[0],
+			SurgSim::DataStructures::Location(SurgSim::Math::Vector3d::Zero()),
+			m_fixedWorldRepresentation,
+			SurgSim::DataStructures::Location(SurgSim::Math::Vector3d::Zero()));
 
 		// Register the constraint in the list of used constraints for this test
 		m_usedConstraints.push_back(constraint);
@@ -271,7 +243,7 @@ TEST_F(BuildMlcpTests, TwoRepresentationsOneConstraintSize3Test)
 	EXPECT_EQ(3, mlcpProblem.CHt.cols());
 	EXPECT_EQ(3, mlcpProblem.H.rows());
 	EXPECT_EQ(6, mlcpProblem.H.cols());
-	EXPECT_EQ(1, mlcpProblem.mu.rows());
+	EXPECT_EQ(3, mlcpProblem.mu.rows());
 	EXPECT_EQ(1u, mlcpProblem.constraintTypes.size());
 	EXPECT_EQ(SurgSim::Math::MLCP_BILATERAL_3D_CONSTRAINT, mlcpProblem.constraintTypes[0]);
 	EXPECT_TRUE(mlcpProblem.isConsistent());
@@ -292,72 +264,36 @@ TEST_F(BuildMlcpTests, OneRepresentationTwoConstraintsTest)
 	m_usedRepresentations.push_back(m_fixedWorldRepresentation);
 	// Set the representation list in the Physics Manager State
 	m_physicsManagerState->setRepresentations(m_usedRepresentations);
+	// The type of constraint.
+	auto constraintType = SurgSim::Physics::FRICTIONLESS_3DCONTACT;
 
 	// Prep the list of constraints: use 2 constraints
 	{
-		std::shared_ptr<Localization> rigidLocalization;
-		{
-			std::shared_ptr<RigidRepresentationLocalization> rigidLocalizationTyped;
-			rigidLocalizationTyped = std::make_shared<RigidRepresentationLocalization>();
-			rigidLocalizationTyped->setRepresentation(m_usedRepresentations[0]);
-			rigidLocalizationTyped->setLocalPosition(SurgSim::Math::Vector3d::Zero());
-			rigidLocalization = rigidLocalizationTyped;
-		}
-		std::shared_ptr<RigidRepresentationContact> rigidSideContact;
-		rigidSideContact = std::make_shared<RigidRepresentationContact>();
-
-		std::shared_ptr<Localization> fixedLocalization;
-		{
-			std::shared_ptr<FixedRepresentationLocalization> fixedLocalizationTyped;
-			fixedLocalizationTyped = std::make_shared<FixedRepresentationLocalization>();
-			fixedLocalizationTyped->setRepresentation(m_fixedWorldRepresentation);
-			fixedLocalizationTyped->setLocalPosition(SurgSim::Math::Vector3d::Zero());
-			fixedLocalization = fixedLocalizationTyped;
-		}
-		std::shared_ptr<FixedRepresentationContact> fixedSideContact;
-		fixedSideContact = std::make_shared<FixedRepresentationContact>();
-
 		// Define the constraint specific data
 		std::shared_ptr<ContactConstraintData> data = std::make_shared<ContactConstraintData>();
 		data->setPlaneEquation(SurgSim::Math::Vector3d(0.0, 1.0, 0.0), 0.0);
 
 		// Set up the constraint
-		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(
-			data, rigidSideContact, rigidLocalization, fixedSideContact, fixedLocalization);
+		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(constraintType,
+			data, m_usedRepresentations[0],
+			SurgSim::DataStructures::Location(SurgSim::Math::Vector3d::Zero()),
+			m_fixedWorldRepresentation,
+			SurgSim::DataStructures::Location(SurgSim::Math::Vector3d::Zero()));
 
 		// Register the constraint in the list of used constraints for this test
 		m_usedConstraints.push_back(constraint);
 	}
 	{
-		std::shared_ptr<Localization> rigidLocalization;
-		{
-			std::shared_ptr<RigidRepresentationLocalization> rigidLocalizationTyped;
-			rigidLocalizationTyped = std::make_shared<RigidRepresentationLocalization>();
-			rigidLocalizationTyped->setRepresentation(m_usedRepresentations[0]);
-			rigidLocalizationTyped->setLocalPosition(SurgSim::Math::Vector3d::Ones());
-			rigidLocalization = rigidLocalizationTyped;
-		}
-		std::shared_ptr<RigidRepresentationContact> rigidSideContact;
-		rigidSideContact = std::make_shared<RigidRepresentationContact>();
-
-		std::shared_ptr<Localization> fixedLocalization;
-		{
-			std::shared_ptr<FixedRepresentationLocalization> fixedLocalizationTyped;
-			fixedLocalizationTyped = std::make_shared<FixedRepresentationLocalization>();
-			fixedLocalizationTyped->setRepresentation(m_fixedWorldRepresentation);
-			fixedLocalizationTyped->setLocalPosition(SurgSim::Math::Vector3d::Ones());
-			fixedLocalization = fixedLocalizationTyped;
-		}
-		std::shared_ptr<FixedRepresentationContact> fixedSideContact;
-		fixedSideContact = std::make_shared<FixedRepresentationContact>();
-
 		// Define the constraint specific data
 		std::shared_ptr<ContactConstraintData> data = std::make_shared<ContactConstraintData>();
 		data->setPlaneEquation(SurgSim::Math::Vector3d(0.0, 1.0, 0.0), 0.0);
 
 		// Set up the constraint
-		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(
-			data, rigidSideContact, rigidLocalization, fixedSideContact, fixedLocalization);
+		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(constraintType,
+			data, m_usedRepresentations[0],
+			SurgSim::DataStructures::Location(SurgSim::Math::Vector3d::Ones()),
+			m_fixedWorldRepresentation,
+			SurgSim::DataStructures::Location(SurgSim::Math::Vector3d::Ones()));
 
 		// Register the constraint in the list of used constraints for this test
 		m_usedConstraints.push_back(constraint);
@@ -409,72 +345,36 @@ TEST_F(BuildMlcpTests, TwoRepresentationsTwoConstraintsTest)
 	m_usedRepresentations.push_back(m_allRepresentations[1]);
 	// Set the representation list in the Physics Manager State
 	m_physicsManagerState->setRepresentations(m_usedRepresentations);
+	// The type of constraint.
+	auto constraintType = SurgSim::Physics::FRICTIONLESS_3DCONTACT;
 
 	// Prep the list of constraints: use 2 constraints
 	{
-		std::shared_ptr<Localization> rigid1Localization;
-		{
-			std::shared_ptr<RigidRepresentationLocalization> rigidLocalizationTyped;
-			rigidLocalizationTyped = std::make_shared<RigidRepresentationLocalization>();
-			rigidLocalizationTyped->setRepresentation(m_usedRepresentations[0]);
-			rigidLocalizationTyped->setLocalPosition(pointOrigin);
-			rigid1Localization = rigidLocalizationTyped;
-		}
-		std::shared_ptr<RigidRepresentationContact> rigidSide1Contact;
-		rigidSide1Contact = std::make_shared<RigidRepresentationContact>();
-
-		std::shared_ptr<Localization> rigid2Localization;
-		{
-			std::shared_ptr<RigidRepresentationLocalization> rigidLocalizationTyped;
-			rigidLocalizationTyped = std::make_shared<RigidRepresentationLocalization>();
-			rigidLocalizationTyped->setRepresentation(m_usedRepresentations[1]);
-			rigidLocalizationTyped->setLocalPosition(pointOrigin);
-			rigid2Localization = rigidLocalizationTyped;
-		}
-		std::shared_ptr<RigidRepresentationContact> rigidSide2Contact;
-		rigidSide2Contact = std::make_shared<RigidRepresentationContact>();
-
 		// Define the constraint specific data
 		std::shared_ptr<ContactConstraintData> data = std::make_shared<ContactConstraintData>();
 		data->setPlaneEquation(planeDirection, planeDistance);
 
 		// Set up the constraint
-		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(
-			data, rigidSide1Contact, rigid1Localization, rigidSide2Contact, rigid2Localization);
+		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(constraintType,
+			data, m_usedRepresentations[0],
+			SurgSim::DataStructures::Location(pointOrigin),
+			m_usedRepresentations[1],
+			SurgSim::DataStructures::Location(pointOrigin));
 
 		// Register the constraint in the list of used constraints for this test
 		m_usedConstraints.push_back(constraint);
 	}
 	{
-		std::shared_ptr<Localization> rigid1Localization;
-		{
-			std::shared_ptr<RigidRepresentationLocalization> rigidLocalizationTyped;
-			rigidLocalizationTyped = std::make_shared<RigidRepresentationLocalization>();
-			rigidLocalizationTyped->setRepresentation(m_usedRepresentations[0]);
-			rigidLocalizationTyped->setLocalPosition(pointOrigin);
-			rigid1Localization = rigidLocalizationTyped;
-		}
-		std::shared_ptr<RigidRepresentationContact> rigidSide1Contact;
-		rigidSide1Contact = std::make_shared<RigidRepresentationContact>();
-
-		std::shared_ptr<Localization> rigid2Localization;
-		{
-			std::shared_ptr<RigidRepresentationLocalization> rigidLocalizationTyped;
-			rigidLocalizationTyped = std::make_shared<RigidRepresentationLocalization>();
-			rigidLocalizationTyped->setRepresentation(m_usedRepresentations[1]);
-			rigidLocalizationTyped->setLocalPosition(pointOne);
-			rigid2Localization = rigidLocalizationTyped;
-		}
-		std::shared_ptr<RigidRepresentationContact> rigidSide2Contact;
-		rigidSide2Contact = std::make_shared<RigidRepresentationContact>();
-
 		// Define the constraint specific data
 		std::shared_ptr<ContactConstraintData> data = std::make_shared<ContactConstraintData>();
 		data->setPlaneEquation(planeDirection, planeDistance);
 
 		// Set up the constraint
-		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(
-			data, rigidSide1Contact, rigid1Localization, rigidSide2Contact, rigid2Localization);
+		std::shared_ptr<Constraint> constraint = std::make_shared<Constraint>(constraintType,
+			data, m_usedRepresentations[0],
+			SurgSim::DataStructures::Location(pointOrigin),
+			m_usedRepresentations[1],
+			SurgSim::DataStructures::Location(pointOne));
 
 		// Register the constraint in the list of used constraints for this test
 		m_usedConstraints.push_back(constraint);
