@@ -21,6 +21,7 @@
 #include "SurgSim/Framework/Assert.h"
 #include "SurgSim/Math/GaussLegendreQuadrature.h"
 #include "SurgSim/Math/Matrix.h"
+#include "SurgSim/Math/OdeEquation.h"
 #include "SurgSim/Math/OdeState.h"
 #include "SurgSim/Math/Quaternion.h"
 #include "SurgSim/Math/Vector.h"
@@ -28,6 +29,7 @@
 
 using SurgSim::Math::Matrix;
 using SurgSim::Math::Quaterniond;
+using SurgSim::Math::SparseMatrix;
 using SurgSim::Math::Vector;
 using SurgSim::Math::Vector3d;
 using SurgSim::Physics::Fem1DElementBeam;
@@ -40,12 +42,12 @@ const double epsilon = 1e-9;
 class MockFem1DElement : public Fem1DElementBeam
 {
 public:
-	MockFem1DElement(std::array<size_t, 2> nodeIds)
+	explicit MockFem1DElement(std::array<size_t, 2> nodeIds)
 		: Fem1DElementBeam(nodeIds)
 	{
 	}
 
-	const Eigen::Matrix<double, 12, 12>& getInitialRotation() const
+	const Eigen::Matrix<double, 12, 12>& getInitialRotation12x12() const
 	{
 		return m_R0;
 	}
@@ -120,8 +122,12 @@ public:
 	double m_rho, m_E, m_nu, m_L;
 	double m_radius;
 	Quaterniond m_orientation;
+	Vector forceVector;
+	SparseMatrix massMatrix;
+	SparseMatrix dampingMatrix;
+	SparseMatrix stiffnessMatrix;
 
-	virtual void SetUp() override
+	void SetUp() override
 	{
 		using SurgSim::Math::getSubVector;
 
@@ -138,7 +144,7 @@ public:
 
 		m_restState.setNumDof(6, m_numberNodes);
 
-		m_orientation.coeffs().setRandom();
+		m_orientation.coeffs().setLinSpaced(2.0, 3.0);
 		m_orientation.normalize();
 
 		Vector3d firstExtremity(0.1, 1.2, 2.3);
@@ -147,6 +153,25 @@ public:
 		Vector& x = m_restState.getPositions();
 		getSubVector(x, m_nodeIds[0], 6).segment<3>(0) = firstExtremity;
 		getSubVector(x, m_nodeIds[1], 6).segment<3>(0) = secondExtremity;
+
+		// Initialize the global matrices for calculations.
+		std::vector<size_t> nodeIdsVectorForm(m_nodeIds.begin(), m_nodeIds.end());
+		forceVector = Vector::Zero(6 * m_numberNodes);
+		Fem1DElementBeam beam(m_nodeIds);
+		massMatrix.resize(6 * m_numberNodes, 6 * m_numberNodes);
+		beam.assembleMatrixBlocks(SurgSim::Math::Matrix::Zero(12, 12),
+								  nodeIdsVectorForm, 6, &massMatrix, true);
+		massMatrix.makeCompressed();
+
+		dampingMatrix.resize(6 * m_numberNodes, 6 * m_numberNodes);
+		beam.assembleMatrixBlocks(SurgSim::Math::Matrix::Zero(12, 12),
+								  nodeIdsVectorForm, 6, &dampingMatrix, true);
+		dampingMatrix.makeCompressed();
+
+		stiffnessMatrix.resize(6 * m_numberNodes, 6 * m_numberNodes);
+		beam.assembleMatrixBlocks(SurgSim::Math::Matrix::Zero(12, 12),
+								  nodeIdsVectorForm, 6, &stiffnessMatrix, true);
+		stiffnessMatrix.makeCompressed();
 	}
 
 	void getExpectedMassMatrix(Eigen::Ref<SurgSim::Math::Matrix> mass)
@@ -162,52 +187,68 @@ public:
 		Eigen::Matrix<double, 12, 12> untransformedMass;
 
 		untransformedMass.col(0) <<
-			1.0 / 3.0, 0.0, 0.0, 0.0, 0.0, 0.0,
-			1.0 / 6.0, 0.0, 0.0, 0.0, 0.0, 0.0;
+								 1.0 / 3.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+									 1.0 / 6.0, 0.0, 0.0, 0.0, 0.0, 0.0;
 
 		untransformedMass.col(1) <<
-			0.0, 13.0 / 35.0 + 6.0 * Iz / (5.0 * A * L2), 0.0, 0.0, 0.0,  11.0 * L / 210.0 + Iz / (10.0 * A * L),
-			0.0,  9.0 / 70.0 - 6.0 * Iz / (5.0 * A * L2), 0.0, 0.0, 0.0, -13.0 * L / 420.0 + Iz / (10.0 * A * L);
+								 0.0, 13.0 / 35.0 + 6.0 * Iz / (5.0 * A * L2), 0.0, 0.0, 0.0,
+									  11.0 * L / 210.0 + Iz / (10.0 * A * L),
+									  0.0,  9.0 / 70.0 - 6.0 * Iz / (5.0 * A * L2), 0.0, 0.0, 0.0,
+									  -13.0 * L / 420.0 + Iz / (10.0 * A * L);
 
 		untransformedMass.col(2) <<
-			0.0, 0.0, 13.0 / 35.0 + 6.0 * Iy / (5.0 * A * L2), 0.0, -11.0 * L / 210.0 - Iy / (10.0 * A * L), 0.0,
-			0.0, 0.0,  9.0 / 70.0 - 6.0 * Iy / (5.0 * A * L2), 0.0,  13.0 * L / 420.0 - Iy / (10.0 * A * L), 0.0;
+								 0.0, 0.0, 13.0 / 35.0 + 6.0 * Iy / (5.0 * A * L2), 0.0,
+									  -11.0 * L / 210.0 - Iy / (10.0 * A * L), 0.0,
+									  0.0, 0.0,  9.0 / 70.0 - 6.0 * Iy / (5.0 * A * L2), 0.0,
+									  13.0 * L / 420.0 - Iy / (10.0 * A * L), 0.0;
 
 		untransformedMass.col(3) <<
-			0.0, 0.0, 0.0, I / (3.0 * A), 0.0, 0.0,
-			0.0, 0.0, 0.0, I / (6.0 * A), 0.0, 0.0;
+								 0.0, 0.0, 0.0, I / (3.0 * A), 0.0, 0.0,
+									  0.0, 0.0, 0.0, I / (6.0 * A), 0.0, 0.0;
 
 		untransformedMass.col(4) <<
-			0.0, 0.0, -11.0 * L / 210.0 - Iy / (10.0 * A * L), 0.0,  L2 / 105.0 + 2.0 * Iy / (15.0 * A), 0.0,
-			0.0, 0.0, -13.0 * L / 420.0 + Iy / (10.0 * A * L), 0.0, -L2 / 140.0 - Iy / (30.0 * A), 0.0;
+								 0.0, 0.0, -11.0 * L / 210.0 - Iy / (10.0 * A * L), 0.0,
+									  L2 / 105.0 + 2.0 * Iy / (15.0 * A), 0.0,
+									  0.0, 0.0, -13.0 * L / 420.0 + Iy / (10.0 * A * L), 0.0,
+									  -L2 / 140.0 - Iy / (30.0 * A), 0.0;
 
 		untransformedMass.col(5) <<
-			0.0, 11.0 * L / 210.0 + Iz / (10.0 * A * L), 0.0, 0.0, 0.0,  L2 / 105.0 + 2.0 * Iz / (15.0 * A),
-			0.0, 13.0 * L / 420.0 - Iz / (10.0 * A * L), 0.0, 0.0, 0.0, -L2 / 140.0 - Iz / (30.0 * A);
+								 0.0, 11.0 * L / 210.0 + Iz / (10.0 * A * L), 0.0, 0.0, 0.0,
+									  L2 / 105.0 + 2.0 * Iz / (15.0 * A),
+									  0.0, 13.0 * L / 420.0 - Iz / (10.0 * A * L), 0.0, 0.0, 0.0,
+									  -L2 / 140.0 - Iz / (30.0 * A);
 
 		untransformedMass.col(6) <<
-			1.0 / 6.0, 0.0, 0.0, 0.0, 0.0, 0.0,
-			1.0 / 3.0, 0.0, 0.0, 0.0, 0.0, 0.0;
+								 1.0 / 6.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+									 1.0 / 3.0, 0.0, 0.0, 0.0, 0.0, 0.0;
 
 		untransformedMass.col(7) <<
-			0.0,  9.0 / 70.0 - 6.0 * Iz / (5.0 * A * L2), 0.0, 0.0, 0.0,  13.0 * L / 420.0 - Iz / (10.0 * A * L),
-			0.0, 13.0 / 35.0 + 6.0 * Iz / (5.0 * A * L2), 0.0, 0.0, 0.0, -11.0 * L / 210.0 - Iz / (10.0 * A * L);
+								 0.0,  9.0 / 70.0 - 6.0 * Iz / (5.0 * A * L2), 0.0, 0.0, 0.0,
+									   13.0 * L / 420.0 - Iz / (10.0 * A * L),
+									   0.0, 13.0 / 35.0 + 6.0 * Iz / (5.0 * A * L2), 0.0, 0.0, 0.0,
+									   -11.0 * L / 210.0 - Iz / (10.0 * A * L);
 
 		untransformedMass.col(8) <<
-			0.0, 0.0,  9.0 / 70.0 - 6.0 * Iy / (5.0 * A * L2), 0.0, -13.0 * L / 420.0 + Iy / (10.0 * A * L), 0.0,
-			0.0, 0.0, 13.0 / 35.0 + 6.0 * Iy / (5.0 * A * L2), 0.0,  11.0 * L / 210.0 + Iy / (10.0 * A * L), 0.0;
+								 0.0, 0.0,  9.0 / 70.0 - 6.0 * Iy / (5.0 * A * L2), 0.0,
+									  -13.0 * L / 420.0 + Iy / (10.0 * A * L), 0.0,
+									  0.0, 0.0, 13.0 / 35.0 + 6.0 * Iy / (5.0 * A * L2), 0.0,
+									  11.0 * L / 210.0 + Iy / (10.0 * A * L), 0.0;
 
 		untransformedMass.col(9) <<
-			0.0, 0.0, 0.0, I / (6.0 * A), 0.0, 0.0,
-			0.0, 0.0, 0.0, I / (3.0 * A), 0.0, 0.0;
+								 0.0, 0.0, 0.0, I / (6.0 * A), 0.0, 0.0,
+									  0.0, 0.0, 0.0, I / (3.0 * A), 0.0, 0.0;
 
 		untransformedMass.col(10) <<
-			0.0, 0.0, 13.0 * L / 420.0 - Iy / (10.0 * A * L), 0.0, -L2 / 140.0 - Iy / (30.0 * A), 0.0,
-			0.0, 0.0, 11.0 * L / 210.0 + Iy / (10.0 * A * L), 0.0,  L2 / 105.0 + 2 * Iy / (15.0 * A), 0.0;
+								  0.0, 0.0, 13.0 * L / 420.0 - Iy / (10.0 * A * L), 0.0,
+									   -L2 / 140.0 - Iy / (30.0 * A), 0.0,
+									   0.0, 0.0, 11.0 * L / 210.0 + Iy / (10.0 * A * L), 0.0,
+									   L2 / 105.0 + 2 * Iy / (15.0 * A), 0.0;
 
 		untransformedMass.col(11) <<
-			0.0, -13.0 * L / 420.0 + Iz / (10.0 * A * L), 0.0, 0.0, 0.0, -L2 / 140.0 - Iz / (30.0 * A),
-			0.0, -11.0 * L / 210.0 - Iz / (10.0 * A * L), 0.0, 0.0, 0.0,  L2 / 105.0 + 2 * Iz / (15.0 * A);
+								  0.0, -13.0 * L / 420.0 + Iz / (10.0 * A * L), 0.0, 0.0, 0.0,
+									   -L2 / 140.0 - Iz / (30.0 * A),
+									   0.0, -11.0 * L / 210.0 - Iz / (10.0 * A * L), 0.0, 0.0, 0.0,
+									   L2 / 105.0 + 2 * Iz / (15.0 * A);
 
 		untransformedMass *= m_rho * m_expectedVolume;
 
@@ -314,52 +355,68 @@ public:
 		Eigen::Matrix<double, 12, 12> untransformedStiffness;
 
 		untransformedStiffness.col(0) <<
-			 m_E * A / L, 0.0, 0.0, 0.0, 0.0, 0.0,
-			-m_E * A / L, 0.0, 0.0, 0.0, 0.0, 0.0;
+									  m_E* A / L, 0.0, 0.0, 0.0, 0.0, 0.0,
+										   -m_E* A / L, 0.0, 0.0, 0.0, 0.0, 0.0;
 
 		untransformedStiffness.col(1) <<
-			0.0,  12.0 * m_E * Iz / L3 / (1 + phi_y), 0.0, 0.0, 0.0, 6.0 * m_E * Iz / L2 / (1 + phi_y),
-			0.0, -12.0 * m_E * Iz / L3 / (1 + phi_y), 0.0, 0.0, 0.0, 6.0 * m_E * Iz / L2 / (1 + phi_y);
+									  0.0,  12.0 * m_E* Iz / L3 / (1 + phi_y), 0.0, 0.0, 0.0,
+											6.0 * m_E* Iz / L2 / (1 + phi_y),
+											0.0, -12.0 * m_E* Iz / L3 / (1 + phi_y), 0.0, 0.0, 0.0,
+											6.0 * m_E* Iz / L2 / (1 + phi_y);
 
 		untransformedStiffness.col(2) <<
-			0.0, 0.0,  12.0 * m_E * Iy / L3 / (1 + phi_z), 0.0, -6.0 * m_E * Iy / L2 / (1 + phi_z), 0.0,
-			0.0, 0.0, -12.0 * m_E * Iy / L3 / (1 + phi_z), 0.0, -6.0 * m_E * Iy / L2 / (1 + phi_z), 0.0;
+									  0.0, 0.0,  12.0 * m_E* Iy / L3 / (1 + phi_z), 0.0,
+										   -6.0 * m_E* Iy / L2 / (1 + phi_z), 0.0,
+										   0.0, 0.0, -12.0 * m_E* Iy / L3 / (1 + phi_z), 0.0,
+										   -6.0 * m_E* Iy / L2 / (1 + phi_z), 0.0;
 
 		untransformedStiffness.col(3) <<
-			0.0, 0.0, 0.0, G * I / L, 0.0, 0.0,
-			0.0, 0.0, 0.0, -G * I / L, 0.0, 0.0;
+									  0.0, 0.0, 0.0, G* I / L, 0.0, 0.0,
+										   0.0, 0.0, 0.0, -G* I / L, 0.0, 0.0;
 
 		untransformedStiffness.col(4) <<
-			0.0, 0.0, -6.0 * m_E * Iy / L2 / (1 + phi_z), 0.0, (4.0 + phi_z) * m_E * Iy / L / (1 + phi_z), 0.0,
-			0.0, 0.0,  6.0 * m_E * Iy / L2 / (1 + phi_z), 0.0, (2.0 - phi_z) * m_E * Iy / L / (1 + phi_z), 0.0;
+									  0.0, 0.0, -6.0 * m_E* Iy / L2 / (1 + phi_z), 0.0,
+										   (4.0 + phi_z) * m_E* Iy / L / (1 + phi_z), 0.0,
+										   0.0, 0.0,  6.0 * m_E* Iy / L2 / (1 + phi_z), 0.0,
+										   (2.0 - phi_z) * m_E* Iy / L / (1 + phi_z), 0.0;
 
 		untransformedStiffness.col(5) <<
-			0.0,  6.0 * m_E * Iz / L2 / (1 + phi_y), 0.0, 0.0, 0.0, (4.0 + phi_y) * m_E * Iz / L / (1 + phi_y),
-			0.0, -6.0 * m_E * Iz / L2 / (1 + phi_y), 0.0, 0.0, 0.0, (2.0 - phi_y) * m_E * Iz / L / (1 + phi_y);
+									  0.0,  6.0 * m_E* Iz / L2 / (1 + phi_y), 0.0, 0.0, 0.0,
+											(4.0 + phi_y) * m_E* Iz / L / (1 + phi_y),
+											0.0, -6.0 * m_E* Iz / L2 / (1 + phi_y), 0.0, 0.0, 0.0,
+											(2.0 - phi_y) * m_E* Iz / L / (1 + phi_y);
 
 		untransformedStiffness.col(6) <<
-			-m_E * A / L, 0.0, 0.0, 0.0, 0.0, 0.0,
-			 m_E * A / L, 0.0, 0.0, 0.0, 0.0, 0.0;
+									  -m_E* A / L, 0.0, 0.0, 0.0, 0.0, 0.0,
+									  m_E* A / L, 0.0, 0.0, 0.0, 0.0, 0.0;
 
 		untransformedStiffness.col(7) <<
-			0.0, -12.0 * m_E * Iz / L3 / (1 + phi_y), 0.0, 0.0, 0.0, -6.0 * m_E * Iz / L2 / (1 + phi_y),
-			0.0,  12.0 * m_E * Iz / L3 / (1 + phi_y), 0.0, 0.0, 0.0, -6.0 * m_E * Iz / L2 / (1 + phi_y);
+									  0.0, -12.0 * m_E* Iz / L3 / (1 + phi_y), 0.0, 0.0, 0.0,
+										   -6.0 * m_E* Iz / L2 / (1 + phi_y),
+										   0.0,  12.0 * m_E* Iz / L3 / (1 + phi_y), 0.0, 0.0, 0.0,
+										   -6.0 * m_E* Iz / L2 / (1 + phi_y);
 
 		untransformedStiffness.col(8) <<
-			0.0, 0.0, -12.0 * m_E * Iy / L3 / (1 + phi_z), 0.0, 6.0 * m_E * Iy / L2 / (1 + phi_z), 0.0,
-			0.0, 0.0,  12.0 * m_E * Iy / L3 / (1 + phi_z), 0.0, 6.0 * m_E * Iy / L2 / (1 + phi_z), 0.0;
+									  0.0, 0.0, -12.0 * m_E* Iy / L3 / (1 + phi_z), 0.0,
+										   6.0 * m_E* Iy / L2 / (1 + phi_z), 0.0,
+										   0.0, 0.0,  12.0 * m_E* Iy / L3 / (1 + phi_z), 0.0,
+										   6.0 * m_E* Iy / L2 / (1 + phi_z), 0.0;
 
 		untransformedStiffness.col(9) <<
-			0.0, 0.0, 0.0, -G * I / L, 0.0, 0.0,
-			0.0, 0.0, 0.0,  G * I / L, 0.0, 0.0;
+									  0.0, 0.0, 0.0, -G* I / L, 0.0, 0.0,
+										   0.0, 0.0, 0.0,  G* I / L, 0.0, 0.0;
 
 		untransformedStiffness.col(10) <<
-			0.0, 0.0, -6.0 * m_E * Iy / L2 / (1 + phi_z), 0.0, (2.0 - phi_z) * m_E * Iy / L / (1 + phi_z), 0.0,
-			0.0, 0.0,  6.0 * m_E * Iy / L2 / (1 + phi_z), 0.0, (4.0 + phi_z) * m_E * Iy / L / (1 + phi_z), 0.0;
+									   0.0, 0.0, -6.0 * m_E* Iy / L2 / (1 + phi_z), 0.0,
+											(2.0 - phi_z) * m_E* Iy / L / (1 + phi_z), 0.0,
+											0.0, 0.0,  6.0 * m_E* Iy / L2 / (1 + phi_z), 0.0,
+											(4.0 + phi_z) * m_E* Iy / L / (1 + phi_z), 0.0;
 
 		untransformedStiffness.col(11) <<
-			0.0,  6.0 * m_E * Iz / L2 / (1 + phi_y), 0.0, 0.0, 0.0, (2.0 - phi_y) * m_E * Iz / L / (1 + phi_y),
-			0.0, -6.0 * m_E * Iz / L2 / (1 + phi_y), 0.0, 0.0, 0.0, (4.0 + phi_y) * m_E * Iz / L / (1 + phi_y);
+									   0.0,  6.0 * m_E* Iz / L2 / (1 + phi_y), 0.0, 0.0, 0.0,
+											 (2.0 - phi_y) * m_E* Iz / L / (1 + phi_y),
+											 0.0, -6.0 * m_E* Iz / L2 / (1 + phi_y), 0.0, 0.0, 0.0,
+											 (4.0 + phi_y) * m_E* Iz / L / (1 + phi_y);
 
 		stiffness.setZero();
 		placeIntoAssembly(untransformedStiffness, stiffness);
@@ -371,7 +428,7 @@ public:
 
 		// Transform into correct coordinates and correct place in matrix
 		std::shared_ptr<MockFem1DElement> beam = getBeam();
-		const Eigen::Matrix<double, 12, 12>& r = beam->getInitialRotation();
+		const Eigen::Matrix<double, 12, 12>& r = beam->getInitialRotation12x12();
 
 		SurgSim::Math::addSubMatrix(r * in * r.transpose(), nodeIdsVectorForm, 6, &out);
 	}
@@ -391,7 +448,7 @@ public:
 TEST_F(Fem1DElementBeamTests, ConstructorTest)
 {
 	ASSERT_NO_THROW(
-		{ MockFem1DElement beam(m_nodeIds); });
+	{ MockFem1DElement beam(m_nodeIds); });
 }
 
 TEST_F(Fem1DElementBeamTests, NodeIdsTest)
@@ -480,19 +537,21 @@ TEST_F(Fem1DElementBeamTests, InitialRotationTest)
 	mask.block<3, 3>(3, 3).setZero();
 	mask.block<3, 3>(6, 6).setZero();
 	mask.block<3, 3>(9, 9).setZero();
-	EXPECT_TRUE(beam->getInitialRotation().cwiseProduct(mask).isZero());
+	EXPECT_TRUE(beam->getInitialRotation12x12().cwiseProduct(mask).isZero());
 
 	// Only the 1st direction of the frame can be compared as the 2 other ones are randomly
 	// chosen (can be any 2 vectors forming an Orthonormal frame)
 	Vector3d expected_i = m_orientation.matrix().col(0);
-	Vector3d i_0 = beam->getInitialRotation().block<3, 3>(0, 0).col(0);
-	Vector3d i_1 = beam->getInitialRotation().block<3, 3>(3, 3).col(0);
-	Vector3d i_2 = beam->getInitialRotation().block<3, 3>(6, 6).col(0);
-	Vector3d i_3 = beam->getInitialRotation().block<3, 3>(9, 9).col(0);
+	Vector3d i_0 = beam->getInitialRotation12x12().block<3, 3>(0, 0).col(0);
+	Vector3d i_1 = beam->getInitialRotation12x12().block<3, 3>(3, 3).col(0);
+	Vector3d i_2 = beam->getInitialRotation12x12().block<3, 3>(6, 6).col(0);
+	Vector3d i_3 = beam->getInitialRotation12x12().block<3, 3>(9, 9).col(0);
 	EXPECT_TRUE(i_0.isApprox(expected_i));
 	EXPECT_TRUE(i_1.isApprox(expected_i));
 	EXPECT_TRUE(i_2.isApprox(expected_i));
 	EXPECT_TRUE(i_3.isApprox(expected_i));
+
+	EXPECT_TRUE(beam->getInitialRotation().col(0).isApprox(expected_i));
 }
 
 TEST_F(Fem1DElementBeamTests, CoordinateTests)
@@ -526,13 +585,13 @@ TEST_F(Fem1DElementBeamTests, CoordinateTests)
 	naturalCoordinateB << 0.0, 1.0;
 	naturalCoordinateMiddle << 1.0 / 2.0, 1.0 / 2.0;
 	EXPECT_THROW(ptA = element.computeCartesianCoordinate(m_restState, invalidNaturalCoordinateNegativeValue), \
-		SurgSim::Framework::AssertionFailure);
+				 SurgSim::Framework::AssertionFailure);
 	EXPECT_THROW(ptA = element.computeCartesianCoordinate(m_restState, invalidNaturalCoordinateSize1), \
-		SurgSim::Framework::AssertionFailure);
+				 SurgSim::Framework::AssertionFailure);
 	EXPECT_THROW(ptA = element.computeCartesianCoordinate(m_restState, invalidNaturalCoordinateSize3), \
-		SurgSim::Framework::AssertionFailure);
+				 SurgSim::Framework::AssertionFailure);
 	EXPECT_THROW(ptA = element.computeCartesianCoordinate(m_restState, invalidNaturalCoordinateSumNot1), \
-		SurgSim::Framework::AssertionFailure);
+				 SurgSim::Framework::AssertionFailure);
 	EXPECT_NO_THROW(ptA = element.computeCartesianCoordinate(m_restState, naturalCoordinateA));
 	EXPECT_NO_THROW(ptB = element.computeCartesianCoordinate(m_restState, naturalCoordinateB));
 	EXPECT_NO_THROW(ptMiddle = element.computeCartesianCoordinate(m_restState, naturalCoordinateMiddle));
@@ -554,10 +613,10 @@ TEST_F(Fem1DElementBeamTests, ForceAndMatricesTest)
 
 	Vector vectorOnes = Vector::Ones(6 * m_numberNodes);
 
-	Vector forceVector = Vector::Zero(6 * m_numberNodes);
-	Matrix massMatrix = Matrix::Zero(6 * m_numberNodes, 6 * m_numberNodes);
-	Matrix dampingMatrix = Matrix::Zero(6 * m_numberNodes, 6 * m_numberNodes);
-	Matrix stiffnessMatrix = Matrix::Zero(6 * m_numberNodes, 6 * m_numberNodes);
+	forceVector.setZero();
+	SurgSim::Math::clearMatrix(&massMatrix);
+	SurgSim::Math::clearMatrix(&dampingMatrix);
+	SurgSim::Math::clearMatrix(&stiffnessMatrix);
 
 	Matrix expectedMass(6 * m_numberNodes, 6 * m_numberNodes);
 	Matrix expectedMass2(6 * m_numberNodes, 6 * m_numberNodes);
@@ -568,26 +627,31 @@ TEST_F(Fem1DElementBeamTests, ForceAndMatricesTest)
 	getExpectedMassMatrix2(expectedMass2);
 	getExpectedStiffnessMatrix(expectedStiffness);
 
+	// Update the internal f, M, D, K variables.
+	beam->updateFMDK(m_restState, SurgSim::Math::ODEEQUATIONUPDATE_FMDK);
+
 	// No force should be produced when in rest state (x = x0) => F = K.(x-x0) = 0
-	beam->addForce(m_restState, &forceVector);
+	beam->addForce(&forceVector);
 	EXPECT_TRUE(forceVector.isZero());
 
-	beam->addMass(m_restState, &massMatrix);
-	EXPECT_TRUE(massMatrix.isApprox(expectedMass));
-	EXPECT_TRUE(massMatrix.isApprox(expectedMass2, 1e-6));
+	beam->addMass(&massMatrix);
+	EXPECT_TRUE(massMatrix.isApprox(expectedMass)) << "Expected Mass:" << std::endl << expectedMass << std::endl <<
+			"Mass Matrix:" << std::endl << massMatrix << std::endl;
+	EXPECT_TRUE(massMatrix.isApprox(expectedMass2, 1e-6)) << "Expected Mass 2:" << std::endl << expectedMass <<
+			std::endl << "Mass Matrix:" << std::endl << massMatrix << std::endl;
 
-	beam->addDamping(m_restState, &dampingMatrix);
+	beam->addDamping(&dampingMatrix);
 	EXPECT_TRUE(dampingMatrix.isApprox(expectedDamping));
 
-	beam->addStiffness(m_restState, &stiffnessMatrix);
+	beam->addStiffness(&stiffnessMatrix);
 	EXPECT_TRUE(stiffnessMatrix.isApprox(expectedStiffness));
 
 	forceVector.setZero();
-	massMatrix.setZero();
-	dampingMatrix.setZero();
-	stiffnessMatrix.setZero();
+	SurgSim::Math::clearMatrix(&massMatrix);
+	SurgSim::Math::clearMatrix(&dampingMatrix);
+	SurgSim::Math::clearMatrix(&stiffnessMatrix);
 
-	beam->addFMDK(m_restState, &forceVector, &massMatrix, &dampingMatrix, &stiffnessMatrix);
+	beam->addFMDK(&forceVector, &massMatrix, &dampingMatrix, &stiffnessMatrix);
 	EXPECT_TRUE(forceVector.isZero());
 	EXPECT_TRUE(massMatrix.isApprox(expectedMass));
 	EXPECT_TRUE(massMatrix.isApprox(expectedMass2, 1e-6));
@@ -596,7 +660,7 @@ TEST_F(Fem1DElementBeamTests, ForceAndMatricesTest)
 
 	// Test addMatVec API with Mass component only
 	forceVector.setZero();
-	beam->addMatVec(m_restState, 1.0, 0.0, 0.0, vectorOnes, &forceVector);
+	beam->addMatVec(1.0, 0.0, 0.0, vectorOnes, &forceVector);
 	for (int rowId = 0; rowId < 6 * m_numberNodes; rowId++)
 	{
 		EXPECT_NEAR(expectedMass.row(rowId).sum(), forceVector[rowId], epsilon);
@@ -604,21 +668,21 @@ TEST_F(Fem1DElementBeamTests, ForceAndMatricesTest)
 	}
 	// Test addMatVec API with Damping component only
 	forceVector.setZero();
-	beam->addMatVec(m_restState, 0.0, 1.0, 0.0, vectorOnes, &forceVector);
+	beam->addMatVec(0.0, 1.0, 0.0, vectorOnes, &forceVector);
 	for (int rowId = 0; rowId < 6 * m_numberNodes; rowId++)
 	{
 		EXPECT_NEAR(expectedDamping.row(rowId).sum(), forceVector[rowId], epsilon);
 	}
 	// Test addMatVec API with Stiffness component only
 	forceVector.setZero();
-	beam->addMatVec(m_restState, 0.0, 0.0, 1.0, vectorOnes, &forceVector);
+	beam->addMatVec(0.0, 0.0, 1.0, vectorOnes, &forceVector);
 	for (int rowId = 0; rowId < 6 * m_numberNodes; rowId++)
 	{
 		EXPECT_NEAR(expectedStiffness.row(rowId).sum(), forceVector[rowId], epsilon);
 	}
 	// Test addMatVec API with mix Mass/Damping/Stiffness components
 	forceVector.setZero();
-	beam->addMatVec(m_restState, 1.0, 2.0, 3.0, vectorOnes, &forceVector);
+	beam->addMatVec(1.0, 2.0, 3.0, vectorOnes, &forceVector);
 	for (int rowId = 0; rowId < 6 * m_numberNodes; rowId++)
 	{
 		double expectedCoef = 1.0 * expectedMass.row(rowId).sum()
