@@ -1,5 +1,5 @@
 // This file is a part of the OpenSurgSim project.
-// Copyright 2013, SimQuest Solutions Inc.
+// Copyright 2013-2015, SimQuest Solutions Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -75,7 +75,7 @@ private:
 	SurgSim::Math::Vector m_F, m_U;
 
 protected:
-	virtual void SetUp() override
+	void SetUp() override
 	{
 		m_fem = std::make_shared<Fem2DRepresentation>("Fem2D");
 	}
@@ -123,8 +123,7 @@ public:
 		// Apply load at extremity
 		if (m_F.size() != static_cast<Vector::Index>(m_fem->getInitialState()->getNumDof()))
 		{
-			m_F.resize(m_fem->getInitialState()->getNumDof());
-			m_F.setZero();
+			m_F.setZero(m_fem->getInitialState()->getNumDof());
 		}
 		m_F.segment(m_fem->getNumDofPerNode() * nodeId, 3) = f;
 	}
@@ -168,10 +167,11 @@ public:
 	void solve()
 	{
 		m_fem->initialize(std::make_shared<SurgSim::Framework::Runtime>());
-		Matrix K = m_fem->computeK(*(m_fem->getCurrentState()));
+		m_fem->updateFMDK(*(m_fem->getCurrentState()), Math::ODEEQUATIONUPDATE_K);
+		Matrix K = m_fem->getK();
 		m_fem->getCurrentState()->applyBoundaryConditionsToMatrix(&K);
 		m_fem->getCurrentState()->applyBoundaryConditionsToVector(&m_F);
-		m_U = K.inverse() * m_F;
+		m_U = K.partialPivLu().solve(m_F);
 	}
 
 	double getUx(size_t nodeId) const
