@@ -1,5 +1,5 @@
 // This file is a part of the OpenSurgSim project.
-// Copyright 2013, SimQuest Solutions Inc.
+// Copyright 2013-2016, SimQuest Solutions Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -20,11 +20,14 @@
 
 #include "SurgSim/Framework/Assert.h"
 #include "SurgSim/Math/Matrix.h"
+#include "SurgSim/Math/OdeEquation.h"
 #include "SurgSim/Math/OdeState.h"
 #include "SurgSim/Math/Vector.h"
 #include "SurgSim/Physics/Fem3DElementCube.h"
 
+using SurgSim::Math::clearMatrix;
 using SurgSim::Math::Matrix;
+using SurgSim::Math::SparseMatrix;
 using SurgSim::Math::Vector;
 using SurgSim::Math::Vector3d;
 using SurgSim::Physics::Fem3DElementCube;
@@ -32,13 +35,13 @@ using SurgSim::Physics::Fem3DElementCube;
 namespace
 {
 /// Epsilon used in this unit test, resulting from a trial and error test.
-const double epsilon = 2.6e-9;
+const double epsilon = 3.6e-9;
 };
 
 class MockFem3DElementCube : public Fem3DElementCube
 {
 public:
-	MockFem3DElementCube(std::array<size_t, 8> nodeIds) : Fem3DElementCube(nodeIds)
+	explicit MockFem3DElementCube(std::array<size_t, 8> nodeIds) : Fem3DElementCube(nodeIds)
 	{
 	}
 
@@ -101,8 +104,7 @@ public:
 		using SurgSim::Math::getSubMatrix;
 		using SurgSim::Math::addSubMatrix;
 
-		Eigen::Matrix<double, 24, 24> K;
-		K.setZero();
+		Eigen::Matrix<double, 24, 24> K = Eigen::Matrix<double, 24, 24>::Zero();
 		{
 			// Expected stiffness matrix given in
 			// "Physically-Based Simulation of Objects Represented by Surface Meshes"
@@ -122,37 +124,37 @@ public:
 
 			getSubMatrix(K, 1, 1, 3, 3).setConstant(n);
 			getSubMatrix(K, 1, 1, 3, 3).diagonal().setConstant(d);
-			getSubMatrix(K, 1, 1, 3, 3)(1,2) = e;
-			getSubMatrix(K, 1, 1, 3, 3)(2,1) = e;
+			getSubMatrix(K, 1, 1, 3, 3)(1, 2) = e;
+			getSubMatrix(K, 1, 1, 3, 3)(2, 1) = e;
 
 			getSubMatrix(K, 2, 2, 3, 3).setConstant(n);
 			getSubMatrix(K, 2, 2, 3, 3).diagonal().setConstant(d);
-			getSubMatrix(K, 2, 2, 3, 3)(0,1) = e;
-			getSubMatrix(K, 2, 2, 3, 3)(1,0) = e;
+			getSubMatrix(K, 2, 2, 3, 3)(0, 1) = e;
+			getSubMatrix(K, 2, 2, 3, 3)(1, 0) = e;
 
 			getSubMatrix(K, 3, 3, 3, 3).setConstant(n);
 			getSubMatrix(K, 3, 3, 3, 3).diagonal().setConstant(d);
-			getSubMatrix(K, 3, 3, 3, 3)(0,2) = e;
-			getSubMatrix(K, 3, 3, 3, 3)(2,0) = e;
+			getSubMatrix(K, 3, 3, 3, 3)(0, 2) = e;
+			getSubMatrix(K, 3, 3, 3, 3)(2, 0) = e;
 
 			getSubMatrix(K, 4, 4, 3, 3).setConstant(n);
 			getSubMatrix(K, 4, 4, 3, 3).diagonal().setConstant(d);
 			//getSubMatrix(K, 4, 4, 3, 3)(0,0) = e; // BUG IN THE PAPER !!!
-			getSubMatrix(K, 4, 4, 3, 3)(0,1) = e;
-			getSubMatrix(K, 4, 4, 3, 3)(1,0) = e;
+			getSubMatrix(K, 4, 4, 3, 3)(0, 1) = e;
+			getSubMatrix(K, 4, 4, 3, 3)(1, 0) = e;
 
 			getSubMatrix(K, 5, 5, 3, 3).setConstant(n);
 			getSubMatrix(K, 5, 5, 3, 3).diagonal().setConstant(d);
-			getSubMatrix(K, 5, 5, 3, 3)(0,2) = e;
-			getSubMatrix(K, 5, 5, 3, 3)(2,0) = e;
+			getSubMatrix(K, 5, 5, 3, 3)(0, 2) = e;
+			getSubMatrix(K, 5, 5, 3, 3)(2, 0) = e;
 
 			getSubMatrix(K, 6, 6, 3, 3).setConstant(e);
 			getSubMatrix(K, 6, 6, 3, 3).diagonal().setConstant(d);
 
 			getSubMatrix(K, 7, 7, 3, 3).setConstant(n);
 			getSubMatrix(K, 7, 7, 3, 3).diagonal().setConstant(d);
-			getSubMatrix(K, 7, 7, 3, 3)(1,2) = e;
-			getSubMatrix(K, 7, 7, 3, 3)(2,1) = e;
+			getSubMatrix(K, 7, 7, 3, 3)(1, 2) = e;
+			getSubMatrix(K, 7, 7, 3, 3)(2, 1) = e;
 
 			// Edges
 			{
@@ -373,8 +375,7 @@ public:
 	{
 		using SurgSim::Math::addSubMatrix;
 
-		Eigen::Matrix<double, 24, 24> M;
-		M.setZero();
+		Eigen::Matrix<double, 24, 24> M = Eigen::Matrix<double, 24, 24>::Zero();
 
 		// "Physically-Based Simulation of Objects Represented by Surface Meshes"
 		// Muller, Techner, Gross, CGI 2004
@@ -391,14 +392,14 @@ public:
 		M.diagonal().setConstant(a);
 
 		M.block(0, 3, 21, 21).diagonal().setConstant(b);
-		M.block(3*3, 3*4, 3, 3).diagonal().setConstant(c); // block (3, 4)
+		M.block(3 * 3, 3 * 4, 3, 3).diagonal().setConstant(c); // block (3, 4)
 
 		M.block(0, 6, 18, 18).diagonal().setConstant(c);
-		M.block(3*2, 3*4, 6, 6).diagonal().setConstant(d); // block (2, 4) and block (3, 5)
+		M.block(3 * 2, 3 * 4, 6, 6).diagonal().setConstant(d); // block (2, 4) and block (3, 5)
 
 		M.block(0, 9, 15, 15).diagonal().setConstant(c);
-		M.block(3*0, 3*3, 3, 3).diagonal().setConstant(b); // block (0, 3)
-		M.block(3*4, 3*7, 3, 3).diagonal().setConstant(b); // block (4, 7)
+		M.block(3 * 0, 3 * 3, 3, 3).diagonal().setConstant(b); // block (0, 3)
+		M.block(3 * 4, 3 * 7, 3, 3).diagonal().setConstant(b); // block (4, 7)
 
 		M.block(0, 12, 12, 12).diagonal().setConstant(b);
 
@@ -411,7 +412,7 @@ public:
 		// Symmetry
 		for (size_t row = 0; row < 24; ++row)
 		{
-			for (size_t col = row+1; col < 24; ++col)
+			for (size_t col = row + 1; col < 24; ++col)
 			{
 				M(col, row) = M(row, col);
 			}
@@ -421,11 +422,10 @@ public:
 		addSubMatrix(M, nodeIdsVectorForm, 3 , &m_expectedMassMatrix);
 	}
 
-	virtual void SetUp() override
+	void SetUp() override
 	{
 		using SurgSim::Math::getSubVector;
 		using SurgSim::Math::getSubMatrix;
-		using SurgSim::Math::addSubMatrix;
 
 		m_restState.setNumDof(3, 8);
 		Vector& x0 = m_restState.getPositions();
@@ -438,14 +438,14 @@ public:
 		//     |  0        |  *1     *->x
 		//     |           | /      /
 		//    4*-----------*5       z
-		getSubVector(x0, 0, 3) = Vector3d(-0.5,-0.5,-0.5);
-		getSubVector(x0, 1, 3) = Vector3d( 0.5,-0.5,-0.5);
-		getSubVector(x0, 2, 3) = Vector3d(-0.5, 0.5,-0.5);
-		getSubVector(x0, 3, 3) = Vector3d( 0.5, 0.5,-0.5);
-		getSubVector(x0, 4, 3) = Vector3d(-0.5,-0.5, 0.5);
-		getSubVector(x0, 5, 3) = Vector3d( 0.5,-0.5, 0.5);
+		getSubVector(x0, 0, 3) = Vector3d(-0.5, -0.5, -0.5);
+		getSubVector(x0, 1, 3) = Vector3d(0.5, -0.5, -0.5);
+		getSubVector(x0, 2, 3) = Vector3d(-0.5, 0.5, -0.5);
+		getSubVector(x0, 3, 3) = Vector3d(0.5, 0.5, -0.5);
+		getSubVector(x0, 4, 3) = Vector3d(-0.5, -0.5, 0.5);
+		getSubVector(x0, 5, 3) = Vector3d(0.5, -0.5, 0.5);
 		getSubVector(x0, 6, 3) = Vector3d(-0.5, 0.5, 0.5);
-		getSubVector(x0, 7, 3) = Vector3d( 0.5, 0.5, 0.5);
+		getSubVector(x0, 7, 3) = Vector3d(0.5, 0.5, 0.5);
 
 		// Ordering following the description in
 		// "Physically-Based Simulation of Objects Represented by Surface Meshes"
@@ -469,26 +469,21 @@ public:
 		m_E = 1e6;
 		m_nu = 0.45;
 
-		m_expectedMassMatrix.resize(3*8, 3*8);
-		m_expectedMassMatrix.setZero();
-		m_expectedDampingMatrix.resize(3*8, 3*8);
-		m_expectedDampingMatrix.setZero();
-		m_expectedStiffnessMatrix.resize(3*8, 3*8);
-		m_expectedStiffnessMatrix.setZero();
-		m_vectorOnes.resize(3*8);
-		m_vectorOnes.setConstant(1.0);
+		m_expectedMassMatrix.setZero(3 * 8, 3 * 8);
+		m_expectedDampingMatrix.setZero(3 * 8, 3 * 8);
+		m_expectedStiffnessMatrix.setZero(3 * 8, 3 * 8);
+		m_vectorOnes.setOnes(3 * 8);
 
 		computeExpectedMassMatrix(nodeIdsVectorForm);
-		m_expectedDampingMatrix.setZero();
 		computeExpectedStiffnessMatrix(nodeIdsVectorForm);
 	}
 
 	// This method tests all node permutations for both face definition (2 groups of 4 indices)
 	// keeping their ordering intact (CW or CCW)
 	void testNodeOrderingAllPermutations(const SurgSim::Math::OdeState& m_restState,
-		size_t id0, size_t id1, size_t id2, size_t id3,
-		size_t id4, size_t id5, size_t id6, size_t id7,
-		bool expectThrow)
+										 size_t id0, size_t id1, size_t id2, size_t id3,
+										 size_t id4, size_t id5, size_t id6, size_t id7,
+										 bool expectThrow)
 	{
 		std::array<size_t, 4> face1 = {{id0, id1, id2, id3}};
 		std::array<size_t, 4> face2 = {{id4, id5, id6, id7}};
@@ -527,7 +522,8 @@ TEST_F(Fem3DElementCubeTests, ConstructorTest)
 	ASSERT_NO_THROW({MockFem3DElementCube cube(m_nodeIds);});
 	ASSERT_NO_THROW({MockFem3DElementCube* cube = new MockFem3DElementCube(m_nodeIds); delete cube;});
 	ASSERT_NO_THROW({std::shared_ptr<MockFem3DElementCube> cube =
-		std::make_shared<MockFem3DElementCube>(m_nodeIds);});
+						 std::make_shared<MockFem3DElementCube>(m_nodeIds);
+					});
 }
 
 TEST_F(Fem3DElementCubeTests, InitializeTest)
@@ -578,7 +574,7 @@ TEST_F(Fem3DElementCubeTests, VolumeTest)
 
 		std::array<size_t, 8> nodeIds = {{0, 1, 3, 2, 4, 6, 7, 5}};
 		auto cube = getCubeElement(nodeIds);
-		ASSERT_THROW(cube->getVolume(m_restState), SurgSim::Framework::AssertionFailure);
+		ASSERT_THROW(cube->initialize(m_restState), SurgSim::Framework::AssertionFailure);
 	}
 
 	{
@@ -587,7 +583,7 @@ TEST_F(Fem3DElementCubeTests, VolumeTest)
 		// Copy the 1st 4 points over the 4 following points, so the cube degenerate to a square
 		m_restState.getPositions().segment<3 * 4>(12) = m_restState.getPositions().segment<3 * 4>(0);
 		auto cube = getCubeElement(m_nodeIds);
-		ASSERT_THROW(cube->getVolume(m_restState), SurgSim::Framework::AssertionFailure);
+		ASSERT_THROW(cube->initialize(m_restState), SurgSim::Framework::AssertionFailure);
 	}
 }
 
@@ -659,7 +655,8 @@ TEST_F(Fem3DElementCubeTests, ShapeFunctionsTest)
 	cube->initialize(m_restState);
 
 	EXPECT_TRUE(cube->getInitialPosition().isApprox(m_expectedX0)) <<
-		"x0 = " << cube->getInitialPosition().transpose() << std::endl << "x0 expected = " << m_expectedX0.transpose();
+			"x0 = " << cube->getInitialPosition().transpose() << std::endl <<
+			"x0 expected = " << m_expectedX0.transpose();
 
 	// We should have by construction:
 	// { N0(p0) = 1    N1(p0)=N2(p0)=N3(p0)=0
@@ -689,66 +686,90 @@ TEST_F(Fem3DElementCubeTests, ShapeFunctionsTest)
 	EXPECT_NEAR(Ni_p0[0], 1.0, 1e-12);
 	for (size_t i = 0; i < 8; ++i)
 	{
-		if (i == 0) continue;
+		if (i == 0)
+		{
+			continue;
+		}
 		EXPECT_NEAR(Ni_p0[i], 0.0, 1e-12);
 	}
 
 	EXPECT_NEAR(Ni_p1[1], 1.0, 1e-12);
 	for (size_t i = 0; i < 8; ++i)
 	{
-		if (i == 1) continue;
+		if (i == 1)
+		{
+			continue;
+		}
 		EXPECT_NEAR(Ni_p1[i], 0.0, 1e-12);
 	}
 
 	EXPECT_NEAR(Ni_p2[2], 1.0, 1e-12);
 	for (size_t i = 0; i < 8; ++i)
 	{
-		if (i == 2) continue;
+		if (i == 2)
+		{
+			continue;
+		}
 		EXPECT_NEAR(Ni_p2[i], 0.0, 1e-12);
 	}
 
 	EXPECT_NEAR(Ni_p3[3], 1.0, 1e-12);
 	for (size_t i = 0; i < 8; ++i)
 	{
-		if (i == 3) continue;
+		if (i == 3)
+		{
+			continue;
+		}
 		EXPECT_NEAR(Ni_p3[i], 0.0, 1e-12);
 	}
 
 	EXPECT_NEAR(Ni_p4[4], 1.0, 1e-12);
 	for (size_t i = 0; i < 8; ++i)
 	{
-		if (i == 4) continue;
+		if (i == 4)
+		{
+			continue;
+		}
 		EXPECT_NEAR(Ni_p4[i], 0.0, 1e-12);
 	}
 
 	EXPECT_NEAR(Ni_p5[5], 1.0, 1e-12);
 	for (size_t i = 0; i < 8; ++i)
 	{
-		if (i == 5) continue;
+		if (i == 5)
+		{
+			continue;
+		}
 		EXPECT_NEAR(Ni_p5[i], 0.0, 1e-12);
 	}
 
 	EXPECT_NEAR(Ni_p6[6], 1.0, 1e-12);
 	for (size_t i = 0; i < 8; ++i)
 	{
-		if (i == 6) continue;
+		if (i == 6)
+		{
+			continue;
+		}
 		EXPECT_NEAR(Ni_p6[i], 0.0, 1e-12);
 	}
 
 	EXPECT_NEAR(Ni_p7[7], 1.0, 1e-12);
 	for (size_t i = 0; i < 8; ++i)
 	{
-		if (i == 7) continue;
+		if (i == 7)
+		{
+			continue;
+		}
 		EXPECT_NEAR(Ni_p7[i], 0.0, 1e-12);
 	}
 
 	// We should have the relation sum(Ni(x,y,z) = 1) for all points in the volume
 	// We verify that relation by sampling the tetrahedron volume
-	for (double epsilon = -1.0; epsilon <= 1.0; epsilon+=0.1)
+	for (double epsilon = -1.0; epsilon <= 1.0; epsilon += 0.1)
 	{
-		for (double eta = -1.0; eta <= 1.0; eta+=0.1)
+		for (double eta = -1.0; eta <= 1.0; eta += 0.1)
 		{
-			for (double mu = -1.0; mu <= 1.0; mu+=0.1)
+			for (double mu = -1.0; mu <= 1.0; mu += 0.1)
 			{
 				double Ni_p[8];
 				double sum = 0.0;
@@ -758,11 +779,16 @@ TEST_F(Fem3DElementCubeTests, ShapeFunctionsTest)
 					sum += Ni_p[i];
 				}
 				EXPECT_NEAR(sum, 1.0, 1e-10) <<
-					" for epsilon = " << epsilon << ", eta = " << eta << ", mu = " << mu << std::endl <<
-					" N0(epsilon,eta,mu) = " << Ni_p[0] << " N1(epsilon,eta,mu) = " << Ni_p[1] <<
-					" N2(epsilon,eta,mu) = " << Ni_p[2] << " N3(epsilon,eta,mu) = " << Ni_p[3] <<
-					" N4(epsilon,eta,mu) = " << Ni_p[4] << " N5(epsilon,eta,mu) = " << Ni_p[5] <<
-					" N6(epsilon,eta,mu) = " << Ni_p[6] << " N7(epsilon,eta,mu) = " << Ni_p[7];
+											 " for epsilon = " << epsilon << ", eta = " <<
+											 eta << ", mu = " << mu << std::endl <<
+											 " N0(epsilon,eta,mu) = " << Ni_p[0] <<
+											 " N1(epsilon,eta,mu) = " << Ni_p[1] <<
+											 " N2(epsilon,eta,mu) = " << Ni_p[2] <<
+											 " N3(epsilon,eta,mu) = " << Ni_p[3] <<
+											 " N4(epsilon,eta,mu) = " << Ni_p[4] <<
+											 " N5(epsilon,eta,mu) = " << Ni_p[5] <<
+											 " N6(epsilon,eta,mu) = " << Ni_p[6] <<
+											 " N7(epsilon,eta,mu) = " << Ni_p[7];
 			}
 		}
 	}
@@ -811,7 +837,7 @@ TEST_F(Fem3DElementCubeTests, CoordinateTests)
 		nodePositions << 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125;
 		EXPECT_TRUE(cube->isValidCoordinate(nodePositions));
 		EXPECT_TRUE(SurgSim::Math::Vector3d(0.0, 0.0, 0.0).isApprox(
-					cube->computeCartesianCoordinate(m_restState, nodePositions), epsilon));
+						cube->computeCartesianCoordinate(m_restState, nodePositions), epsilon));
 	}
 
 	{
@@ -819,7 +845,7 @@ TEST_F(Fem3DElementCubeTests, CoordinateTests)
 		nodePositions << 0.01, 0.07, 0.11, 0.05, 0.0, 0.23, 0.13, 0.4;
 		EXPECT_TRUE(cube->isValidCoordinate(nodePositions));
 		EXPECT_TRUE(SurgSim::Math::Vector3d(0.04, 0.19, 0.26).isApprox(
-					cube->computeCartesianCoordinate(m_restState, nodePositions), epsilon));
+						cube->computeCartesianCoordinate(m_restState, nodePositions), epsilon));
 		// 0.01 * (-0.5,-0.5,-0.5) => (-0.005, -0.005, -0.005)
 		// 0.07 * ( 0.5,-0.5,-0.5) => ( 0.035, -0.035, -0.035)
 		// 0.11 * ( 0.5, 0.5,-0.5) => ( 0.055,  0.055, -0.055)
@@ -843,39 +869,51 @@ TEST_F(Fem3DElementCubeTests, ForceAndMatricesTest)
 	auto cube = getCubeElement(m_nodeIds);
 	cube->initialize(m_restState);
 
-	SurgSim::Math::Vector forceVector(3*8);
-	SurgSim::Math::Matrix massMatrix(3*8, 3*8);
-	SurgSim::Math::Matrix dampingMatrix(3*8, 3*8);
-	SurgSim::Math::Matrix stiffnessMatrix(3*8, 3*8);
-
-	forceVector.setZero();
+	Vector forceVector = Vector::Zero(3 * 8);
+	SparseMatrix massMatrix(3 * 8, 3 * 8);
+	SparseMatrix dampingMatrix(3 * 8, 3 * 8);
+	SparseMatrix stiffnessMatrix(3 * 8, 3 * 8);
+	Matrix zeroMatrix = Matrix::Zero(cube->getNumDofPerNode() * cube->getNumNodes(),
+									 cube->getNumDofPerNode() * cube->getNumNodes());
 	massMatrix.setZero();
+	cube->assembleMatrixBlocks(zeroMatrix, cube->getNodeIds(),
+							   static_cast<SparseMatrix::Index>(cube->getNumDofPerNode()), &massMatrix, true);
+	massMatrix.makeCompressed();
 	dampingMatrix.setZero();
+	cube->assembleMatrixBlocks(zeroMatrix, cube->getNodeIds(),
+							   static_cast<SparseMatrix::Index>(cube->getNumDofPerNode()), &dampingMatrix, true);
+	dampingMatrix.makeCompressed();
 	stiffnessMatrix.setZero();
+	cube->assembleMatrixBlocks(zeroMatrix, cube->getNodeIds(),
+							   static_cast<SparseMatrix::Index>(cube->getNumDofPerNode()), &stiffnessMatrix, true);
+	stiffnessMatrix.makeCompressed();
+
+	// Update the internal f, M, D, K variables.
+	cube->updateFMDK(m_restState, SurgSim::Math::ODEEQUATIONUPDATE_FMDK);
 
 	// No force should be produced when in rest state (x = x0) => F = K.(x-x0) = 0
-	cube->addForce(m_restState, &forceVector);
+	cube->addForce(&forceVector);
 	EXPECT_TRUE(forceVector.isZero());
 
-	cube->addMass(m_restState, &massMatrix);
+	cube->addMass(&massMatrix);
 	EXPECT_TRUE(massMatrix.isApprox(m_expectedMassMatrix)) <<
-		"Expected mass matrix :" << std::endl << m_expectedMassMatrix  << std::endl << std::endl <<
-		"Mass matrix :"  << std::endl << massMatrix << std::endl << std::endl <<
-		"Error on the mass matrix is "  << std::endl << m_expectedMassMatrix - massMatrix << std::endl;
+			"Expected mass matrix :" << std::endl << m_expectedMassMatrix  << std::endl << std::endl <<
+			"Mass matrix :"  << std::endl << massMatrix << std::endl;
 
-	cube->addDamping(m_restState, &dampingMatrix);
+	cube->addDamping(&dampingMatrix);
 	EXPECT_TRUE(dampingMatrix.isApprox(m_expectedDampingMatrix));
 
-	cube->addStiffness(m_restState, &stiffnessMatrix);
-	EXPECT_TRUE(stiffnessMatrix.isApprox(m_expectedStiffnessMatrix)) <<
-		"Error on the stiffness matrix is " << std::endl << m_expectedStiffnessMatrix - stiffnessMatrix << std::endl;
+	cube->addStiffness(&stiffnessMatrix);
+	EXPECT_TRUE(stiffnessMatrix.isApprox(m_expectedStiffnessMatrix))  <<
+			"Expected stiffness matrix :" << std::endl << m_expectedStiffnessMatrix  << std::endl << std::endl <<
+			"Stiffness matrix :"  << std::endl << stiffnessMatrix << std::endl;
 
 	forceVector.setZero();
-	massMatrix.setZero();
-	dampingMatrix.setZero();
-	stiffnessMatrix.setZero();
+	clearMatrix(&massMatrix);
+	clearMatrix(&dampingMatrix);
+	clearMatrix(&stiffnessMatrix);
 
-	cube->addFMDK(m_restState, &forceVector, &massMatrix, &dampingMatrix, &stiffnessMatrix);
+	cube->addFMDK(&forceVector, &massMatrix, &dampingMatrix, &stiffnessMatrix);
 	EXPECT_TRUE(forceVector.isZero());
 	EXPECT_TRUE(massMatrix.isApprox(m_expectedMassMatrix));
 	EXPECT_TRUE(dampingMatrix.isApprox(m_expectedDampingMatrix));
@@ -883,33 +921,33 @@ TEST_F(Fem3DElementCubeTests, ForceAndMatricesTest)
 
 	// Test addMatVec API with Mass component only
 	forceVector.setZero();
-	cube->addMatVec(m_restState, 1.0, 0.0, 0.0, m_vectorOnes, &forceVector);
+	cube->addMatVec(1.0, 0.0, 0.0, m_vectorOnes, &forceVector);
 	for (int rowId = 0; rowId < 3 * 8; rowId++)
 	{
 		EXPECT_NEAR(m_expectedMassMatrix.row(rowId).sum(), forceVector[rowId], epsilon);
 	}
 	// Test addMatVec API with Damping component only
 	forceVector.setZero();
-	cube->addMatVec(m_restState, 0.0, 1.0, 0.0, m_vectorOnes, &forceVector);
+	cube->addMatVec(0.0, 1.0, 0.0, m_vectorOnes, &forceVector);
 	for (int rowId = 0; rowId < 3 * 8; rowId++)
 	{
 		EXPECT_NEAR(m_expectedDampingMatrix.row(rowId).sum(), forceVector[rowId], epsilon);
 	}
 	// Test addMatVec API with Stiffness component only
 	forceVector.setZero();
-	cube->addMatVec(m_restState, 0.0, 0.0, 1.0, m_vectorOnes, &forceVector);
+	cube->addMatVec(0.0, 0.0, 1.0, m_vectorOnes, &forceVector);
 	for (int rowId = 0; rowId < 3 * 8; rowId++)
 	{
 		EXPECT_NEAR(m_expectedStiffnessMatrix.row(rowId).sum(), forceVector[rowId], epsilon);
 	}
 	// Test addMatVec API with mix Mass/Damping/Stiffness components
 	forceVector.setZero();
-	cube->addMatVec(m_restState, 1.0, 2.0, 3.0, m_vectorOnes, &forceVector);
+	cube->addMatVec(1.0, 2.0, 3.0, m_vectorOnes, &forceVector);
 	for (int rowId = 0; rowId < 3 * 8; rowId++)
 	{
 		double expectedCoef = 1.0 * m_expectedMassMatrix.row(rowId).sum() +
-			2.0 * m_expectedDampingMatrix.row(rowId).sum() +
-			3.0 * m_expectedStiffnessMatrix.row(rowId).sum();
+							  2.0 * m_expectedDampingMatrix.row(rowId).sum() +
+							  3.0 * m_expectedStiffnessMatrix.row(rowId).sum();
 		EXPECT_NEAR(expectedCoef, forceVector[rowId], epsilon);
 	}
 }
