@@ -21,14 +21,15 @@
 #include "SurgSim/Math/Quaternion.h"
 #include "SurgSim/Math/RigidTransform.h"
 #include "SurgSim/Math/Vector.h"
+#include "SurgSim/Physics/MassSpringLocalization.h"
 #include "SurgSim/Physics/MassSpringRepresentation.h"
-#include "SurgSim/Physics/MassSpringRepresentationLocalization.h"
+#include "SurgSim/Physics/UnitTests/DeformableTestsUtility.h"
 #include "SurgSim/Physics/UnitTests/MockObjects.h"
 
 using SurgSim::Math::Vector;
 using SurgSim::Math::Matrix;
+using SurgSim::Physics::MassSpringLocalization;
 using SurgSim::Physics::MassSpringRepresentation;
-using SurgSim::Physics::MassSpringRepresentationLocalization;
 using SurgSim::Physics::MockLocalization;
 using SurgSim::Physics::MockMassSpring;
 using SurgSim::Physics::MockSpring;
@@ -56,9 +57,9 @@ protected:
 
 	std::shared_ptr<MockMassSpring> m_massSpring;
 	std::shared_ptr<SurgSim::Math::OdeState> m_initialState;
-	std::shared_ptr<SurgSim::Physics::MassSpringRepresentationLocalization> m_localization;
+	std::shared_ptr<SurgSim::Physics::MassSpringLocalization> m_localization;
 
-	virtual void SetUp() override
+	void SetUp() override
 	{
 		m_massSpring = std::make_shared<MockMassSpring>();
 
@@ -124,7 +125,7 @@ protected:
 
 	void createLocalization()
 	{
-		m_localization = std::make_shared<SurgSim::Physics::MassSpringRepresentationLocalization>();
+		m_localization = std::make_shared<SurgSim::Physics::MassSpringLocalization>();
 		m_localization->setRepresentation(m_massSpring);
 		m_localization->setLocalNode(0);
 	}
@@ -137,7 +138,8 @@ TEST_F(MassSpringRepresentationTests, Constructor)
 	ASSERT_NO_THROW({MassSpringRepresentation* m = new MassSpringRepresentation("MassSpring"); delete m;});
 
 	ASSERT_NO_THROW({std::shared_ptr<MassSpringRepresentation> m =
-		std::make_shared<MassSpringRepresentation>("MassSpring");});
+						 std::make_shared<MassSpringRepresentation>("MassSpring");
+					});
 }
 
 TEST_F(MassSpringRepresentationTests, SetGetMethods)
@@ -199,9 +201,6 @@ TEST_F(MassSpringRepresentationTests, SetGetMethods)
 	EXPECT_DOUBLE_EQ(5.5, massSpring->getRayleighDampingMass());
 	massSpring->setRayleighDampingStiffness(5.4);
 	EXPECT_DOUBLE_EQ(5.4, massSpring->getRayleighDampingStiffness());
-
-	// set/get Type
-	EXPECT_EQ(SurgSim::Physics::REPRESENTATION_TYPE_MASSSPRING, massSpring->getType());
 }
 
 TEST_F(MassSpringRepresentationTests, BeforeUpdateTest)
@@ -265,10 +264,8 @@ TEST_F(MassSpringRepresentationTests, TransformInitialStateTest)
 		expectedV.segment<3>(numDofPerNode * nodeId) = initialPose.linear() * v.segment<3>(numDofPerNode * nodeId);
 	}
 
-	// Initialize the component
+	// Initialize the component => apply the pose to the initial state
 	ASSERT_TRUE(m_massSpring->initialize(std::make_shared<SurgSim::Framework::Runtime>()));
-	// Wake-up the component => apply the pose to the initial state
-	ASSERT_TRUE(m_massSpring->wakeUp());
 
 	EXPECT_TRUE(m_massSpring->getInitialState()->getPositions().isApprox(expectedX));
 	EXPECT_TRUE(m_massSpring->getInitialState()->getVelocities().isApprox(expectedV));
@@ -277,11 +274,11 @@ TEST_F(MassSpringRepresentationTests, TransformInitialStateTest)
 TEST_F(MassSpringRepresentationTests, ExternalForceAPITest)
 {
 	using SurgSim::Math::Vector;
-	using SurgSim::Math::Matrix;
+	using SurgSim::Math::SparseMatrix;
 
 	std::shared_ptr<MockMassSpring> massSpring = std::make_shared<MockMassSpring>();
-	std::shared_ptr<MassSpringRepresentationLocalization> localization =
-		std::make_shared<MassSpringRepresentationLocalization>();
+	std::shared_ptr<MassSpringLocalization> localization =
+		std::make_shared<MassSpringLocalization>();
 	std::shared_ptr<MockLocalization> wrongLocalizationType =
 		std::make_shared<MockLocalization>();
 
@@ -297,6 +294,9 @@ TEST_F(MassSpringRepresentationTests, ExternalForceAPITest)
 	massSpring->addSpring(std::make_shared<SurgSim::Physics::LinearSpring>(0, 1));
 
 	// Vector initialized (properly sized and zeroed)
+	SparseMatrix zeroMatrix(static_cast<SparseMatrix::Index>(massSpring->getNumDof()),
+							static_cast<SparseMatrix::Index>(massSpring->getNumDof()));
+	zeroMatrix.setZero();
 	EXPECT_NE(0, massSpring->getExternalForce().size());
 	EXPECT_NE(0, massSpring->getExternalStiffness().size());
 	EXPECT_NE(0, massSpring->getExternalDamping().size());
@@ -306,8 +306,8 @@ TEST_F(MassSpringRepresentationTests, ExternalForceAPITest)
 	EXPECT_EQ(massSpring->getNumDof(), massSpring->getExternalDamping().cols());
 	EXPECT_EQ(massSpring->getNumDof(), massSpring->getExternalDamping().rows());
 	EXPECT_TRUE(massSpring->getExternalForce().isZero());
-	EXPECT_TRUE(massSpring->getExternalStiffness().isZero());
-	EXPECT_TRUE(massSpring->getExternalDamping().isZero());
+	EXPECT_TRUE(massSpring->getExternalStiffness().isApprox(zeroMatrix));
+	EXPECT_TRUE(massSpring->getExternalDamping().isApprox(zeroMatrix));
 
 	localization->setRepresentation(massSpring);
 	localization->setLocalNode(0);
@@ -324,14 +324,14 @@ TEST_F(MassSpringRepresentationTests, ExternalForceAPITest)
 	expectedD.block(0, 0, massSpring->getNumDofPerNode(), massSpring->getNumDofPerNode()) = D;
 
 	ASSERT_THROW(massSpring->addExternalGeneralizedForce(nullptr, F, K, D),
-		SurgSim::Framework::AssertionFailure);
+				 SurgSim::Framework::AssertionFailure);
 	ASSERT_THROW(massSpring->addExternalGeneralizedForce(wrongLocalizationType, F, K, D),
-		SurgSim::Framework::AssertionFailure);
+				 SurgSim::Framework::AssertionFailure);
 
 	massSpring->addExternalGeneralizedForce(localization, F, K, D);
 	EXPECT_FALSE(massSpring->getExternalForce().isZero());
-	EXPECT_FALSE(massSpring->getExternalStiffness().isZero());
-	EXPECT_FALSE(massSpring->getExternalDamping().isZero());
+	EXPECT_FALSE(massSpring->getExternalStiffness().isApprox(zeroMatrix));
+	EXPECT_FALSE(massSpring->getExternalDamping().isApprox(zeroMatrix));
 	EXPECT_TRUE(massSpring->getExternalForce().isApprox(expectedF));
 	EXPECT_TRUE(massSpring->getExternalStiffness().isApprox(expectedK));
 	EXPECT_TRUE(massSpring->getExternalDamping().isApprox(expectedD));
@@ -345,9 +345,7 @@ TEST_F(MassSpringRepresentationTests, ExternalForceAPITest)
 TEST_F(MassSpringRepresentationTests, ComputesWithNoGravityAndNoDampingTest)
 {
 	using SurgSim::Math::Vector3d;
-
-	Vector *F;
-	Matrix *M, *D, *K;
+	using SurgSim::Math::SparseMatrix;
 
 	addMasses();
 	addSprings();
@@ -362,16 +360,8 @@ TEST_F(MassSpringRepresentationTests, ComputesWithNoGravityAndNoDampingTest)
 	{
 		SCOPED_TRACE("Without external force");
 
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeF(*m_initialState).isApprox(m_expectedSpringsForce)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeM(*m_initialState).isApprox(m_expectedMass)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeD(*m_initialState).isApprox(m_expectedDamping)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeK(*m_initialState).isApprox(m_expectedStiffness)));
-		// Test combo method computeFMDK
-		EXPECT_NO_THROW(m_massSpring->computeFMDK(*m_initialState, &F, &M, &D, &K));
-		EXPECT_TRUE((*F).isApprox(m_expectedSpringsForce));
-		EXPECT_TRUE((*M).isApprox(m_expectedMass));
-		EXPECT_TRUE((*D).isApprox(m_expectedDamping));
-		EXPECT_TRUE((*K).isApprox(m_expectedStiffness));
+		testOdeEquationUpdate(m_massSpring, *m_initialState, m_expectedSpringsForce, m_expectedMass,
+			m_expectedDamping, m_expectedStiffness);
 	}
 
 	size_t numDofPerNode = m_massSpring->getNumDofPerNode();
@@ -390,24 +380,16 @@ TEST_F(MassSpringRepresentationTests, ComputesWithNoGravityAndNoDampingTest)
 		externalK.block(0, 0, numDofPerNode, numDofPerNode) = externalLocalK;
 		externalD.block(0, 0, numDofPerNode, numDofPerNode) = externalLocalD;
 		m_massSpring->addExternalGeneralizedForce(m_localization, externalLocalForce, externalLocalK, externalLocalD);
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeF(*m_initialState).isApprox(
-			m_expectedSpringsForce + externalForce)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeK(*m_initialState).isApprox(
-			m_expectedStiffness + externalK)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeD(*m_initialState).isApprox(m_expectedDamping + externalD)));
-		EXPECT_NO_THROW(m_massSpring->computeFMDK(*m_initialState, &F, &M, &D, &K));
-		EXPECT_TRUE((*F).isApprox(m_expectedSpringsForce + externalForce));
-		EXPECT_TRUE((*K).isApprox(m_expectedStiffness + externalK));
-		EXPECT_TRUE((*D).isApprox(m_expectedDamping + externalD));
+
+		testOdeEquationUpdate(m_massSpring, *m_initialState, m_expectedSpringsForce + externalForce, m_expectedMass,
+			m_expectedDamping + externalD, m_expectedStiffness + externalK);
 	}
 }
 
 TEST_F(MassSpringRepresentationTests, ComputesWithNoGravityAndDampingTest)
 {
 	using SurgSim::Math::Vector3d;
-
-	Vector *F;
-	Matrix *M, *D, *K;
+	using SurgSim::Math::SparseMatrix;
 
 	addMasses();
 	addSprings();
@@ -426,17 +408,8 @@ TEST_F(MassSpringRepresentationTests, ComputesWithNoGravityAndDampingTest)
 	{
 		SCOPED_TRACE("Without external force");
 
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeF(*m_initialState).isApprox(expectedForce)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeM(*m_initialState).isApprox(m_expectedMass)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeD(*m_initialState).isApprox(
-			m_expectedDamping + m_expectedRayleighDamping)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeK(*m_initialState).isApprox(m_expectedStiffness)));
-		// Test combo method computeFMDK
-		EXPECT_NO_THROW(m_massSpring->computeFMDK(*m_initialState, &F, &M, &D, &K));
-		EXPECT_TRUE((*F).isApprox(expectedForce));
-		EXPECT_TRUE((*M).isApprox(m_expectedMass));
-		EXPECT_TRUE((*D).isApprox(m_expectedDamping + m_expectedRayleighDamping));
-		EXPECT_TRUE((*K).isApprox(m_expectedStiffness));
+		testOdeEquationUpdate(m_massSpring, *m_initialState, expectedForce, m_expectedMass,
+			m_expectedDamping + m_expectedRayleighDamping, m_expectedStiffness);
 	}
 
 	size_t numDofPerNode = m_massSpring->getNumDofPerNode();
@@ -455,24 +428,16 @@ TEST_F(MassSpringRepresentationTests, ComputesWithNoGravityAndDampingTest)
 		externalK.block(0, 0, numDofPerNode, numDofPerNode) = externalLocalK;
 		externalD.block(0, 0, numDofPerNode, numDofPerNode) = externalLocalD;
 		m_massSpring->addExternalGeneralizedForce(m_localization, externalLocalForce, externalLocalK, externalLocalD);
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeF(*m_initialState).isApprox(expectedForce + externalForce)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeK(*m_initialState).isApprox(
-			m_expectedStiffness + externalK)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeD(*m_initialState).isApprox(
-			m_expectedDamping + m_expectedRayleighDamping + externalD)));
-		EXPECT_NO_THROW(m_massSpring->computeFMDK(*m_initialState, &F, &M, &D, &K));
-		EXPECT_TRUE((*F).isApprox(expectedForce + externalForce));
-		EXPECT_TRUE((*K).isApprox(m_expectedStiffness + externalK));
-		EXPECT_TRUE((*D).isApprox(m_expectedDamping + m_expectedRayleighDamping + externalD));
+
+		testOdeEquationUpdate(m_massSpring, *m_initialState, expectedForce + externalForce, m_expectedMass,
+			m_expectedDamping + m_expectedRayleighDamping + externalD, m_expectedStiffness + externalK);
 	}
 }
 
 TEST_F(MassSpringRepresentationTests, ComputesWithGravityAndNoDampingTest)
 {
 	using SurgSim::Math::Vector3d;
-
-	Vector *F;
-	Matrix *M, *D, *K;
+	using SurgSim::Math::SparseMatrix;
 
 	addMasses();
 	addSprings();
@@ -489,16 +454,8 @@ TEST_F(MassSpringRepresentationTests, ComputesWithGravityAndNoDampingTest)
 	{
 		SCOPED_TRACE("Without external force");
 
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeF(*m_initialState).isApprox(expectedForce)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeM(*m_initialState).isApprox(m_expectedMass)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeD(*m_initialState).isApprox(m_expectedDamping)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeK(*m_initialState).isApprox(m_expectedStiffness)));
-		// Test combo method computeFMDK
-		EXPECT_NO_THROW(m_massSpring->computeFMDK(*m_initialState, &F, &M, &D, &K));
-		EXPECT_TRUE((*F).isApprox(expectedForce));
-		EXPECT_TRUE((*M).isApprox(m_expectedMass));
-		EXPECT_TRUE((*D).isApprox(m_expectedDamping));
-		EXPECT_TRUE((*K).isApprox(m_expectedStiffness));
+		testOdeEquationUpdate(m_massSpring, *m_initialState, expectedForce, m_expectedMass,
+			m_expectedDamping, m_expectedStiffness);
 	}
 
 	size_t numDofPerNode = m_massSpring->getNumDofPerNode();
@@ -517,23 +474,16 @@ TEST_F(MassSpringRepresentationTests, ComputesWithGravityAndNoDampingTest)
 		externalK.block(0, 0, numDofPerNode, numDofPerNode) = externalLocalK;
 		externalD.block(0, 0, numDofPerNode, numDofPerNode) = externalLocalD;
 		m_massSpring->addExternalGeneralizedForce(m_localization, externalLocalForce, externalLocalK, externalLocalD);
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeF(*m_initialState).isApprox(expectedForce + externalForce)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeK(*m_initialState).isApprox(
-			m_expectedStiffness + externalK)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeD(*m_initialState).isApprox(m_expectedDamping + externalD)));
-		EXPECT_NO_THROW(m_massSpring->computeFMDK(*m_initialState, &F, &M, &D, &K));
-		EXPECT_TRUE((*F).isApprox(expectedForce + externalForce));
-		EXPECT_TRUE((*K).isApprox(m_expectedStiffness + externalK));
-		EXPECT_TRUE((*D).isApprox(m_expectedDamping + externalD));
+
+		testOdeEquationUpdate(m_massSpring, *m_initialState, expectedForce + externalForce, m_expectedMass,
+			m_expectedDamping + externalD, m_expectedStiffness + externalK);
 	}
 }
 
 TEST_F(MassSpringRepresentationTests, ComputesWithGravityAndDampingTest)
 {
 	using SurgSim::Math::Vector3d;
-
-	Vector *F;
-	Matrix *M, *D, *K;
+	using SurgSim::Math::SparseMatrix;
 
 	addMasses();
 	addSprings();
@@ -548,22 +498,13 @@ TEST_F(MassSpringRepresentationTests, ComputesWithGravityAndDampingTest)
 	m_massSpring->wakeUp();
 
 	SurgSim::Math::Vector expectedForce = m_expectedSpringsForce + m_expectedRayleighDampingForce +
-		m_expectedGravityForce;
+										  m_expectedGravityForce;
 
 	{
 		SCOPED_TRACE("Without external force");
 
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeF(*m_initialState).isApprox(expectedForce)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeM(*m_initialState).isApprox(m_expectedMass)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeD(*m_initialState).isApprox(
-			m_expectedDamping + m_expectedRayleighDamping)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeK(*m_initialState).isApprox(m_expectedStiffness)));
-		// Test combo method computeFMDK
-		EXPECT_NO_THROW(m_massSpring->computeFMDK(*m_initialState, &F, &M, &D, &K));
-		EXPECT_TRUE((*F).isApprox(expectedForce));
-		EXPECT_TRUE((*M).isApprox(m_expectedMass));
-		EXPECT_TRUE((*D).isApprox(m_expectedDamping + m_expectedRayleighDamping));
-		EXPECT_TRUE((*K).isApprox(m_expectedStiffness));
+		testOdeEquationUpdate(m_massSpring, *m_initialState, expectedForce, m_expectedMass,
+			m_expectedDamping + m_expectedRayleighDamping, m_expectedStiffness);
 	}
 
 	size_t numDofPerNode = m_massSpring->getNumDofPerNode();
@@ -582,14 +523,8 @@ TEST_F(MassSpringRepresentationTests, ComputesWithGravityAndDampingTest)
 		externalK.block(0, 0, numDofPerNode, numDofPerNode) = externalLocalK;
 		externalD.block(0, 0, numDofPerNode, numDofPerNode) = externalLocalD;
 		m_massSpring->addExternalGeneralizedForce(m_localization, externalLocalForce, externalLocalK, externalLocalD);
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeF(*m_initialState).isApprox(expectedForce + externalForce)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeK(*m_initialState).isApprox(
-			m_expectedStiffness + externalK)));
-		EXPECT_NO_THROW(EXPECT_TRUE(m_massSpring->computeD(*m_initialState).isApprox(
-			m_expectedDamping + m_expectedRayleighDamping + externalD)));
-		EXPECT_NO_THROW(m_massSpring->computeFMDK(*m_initialState, &F, &M, &D, &K));
-		EXPECT_TRUE((*F).isApprox(expectedForce + externalForce));
-		EXPECT_TRUE((*K).isApprox(m_expectedStiffness + externalK));
-		EXPECT_TRUE((*D).isApprox(m_expectedDamping + m_expectedRayleighDamping + externalD));
+
+		testOdeEquationUpdate(m_massSpring, *m_initialState, expectedForce + externalForce, m_expectedMass,
+			m_expectedDamping + m_expectedRayleighDamping + externalD, m_expectedStiffness + externalK);
 	}
 }
