@@ -14,6 +14,7 @@
 // limitations under the License.
 
 #include "SurgSim/Physics/UnitTests/MockObjects.h"
+#include "SurgSim/Math/Shape.h"
 #include "SurgSim/Physics/FemPlyReaderDelegate.h"
 
 namespace SurgSim
@@ -25,10 +26,10 @@ SURGSIM_REGISTER(SurgSim::Framework::Component, SurgSim::Physics::MockDeformable
 				 MockDeformableRepresentation);
 
 MockRepresentation::MockRepresentation(const std::string& name) :
-		Representation(name),
-		m_preUpdateCount(0),
-		m_updateCount(0),
-		m_postUpdateCount(0)
+	Representation(name),
+	m_preUpdateCount(0),
+	m_updateCount(0),
+	m_postUpdateCount(0)
 {
 }
 
@@ -36,11 +37,6 @@ MockRepresentation::~MockRepresentation()
 {
 }
 
-RepresentationType MockRepresentation::getType() const
-{
-	return REPRESENTATION_TYPE_FIXED;
-}
-
 void MockRepresentation::beforeUpdate(double dt)
 {
 	m_preUpdateCount++;
@@ -71,85 +67,101 @@ int MockRepresentation::getPostUpdateCount() const
 	return m_postUpdateCount;
 }
 
+std::shared_ptr<Localization> MockRepresentation::createLocalization(
+	const SurgSim::DataStructures::Location& location)
+{
+	return std::make_shared<MockLocalization>();
+}
 
 MockRigidRepresentation::MockRigidRepresentation() : RigidRepresentation("MockRigidRepresentation")
 {
 }
 
-RigidRepresentationState& MockRigidRepresentation::getInitialState()
+RigidState& MockRigidRepresentation::getInitialState()
 {
 	return m_initialState;
 }
 
-RigidRepresentationState& MockRigidRepresentation::getCurrentState()
+RigidState& MockRigidRepresentation::getCurrentState()
 {
 	return m_currentState;
 }
 
-RigidRepresentationState& MockRigidRepresentation::getPreviousState()
+RigidState& MockRigidRepresentation::getPreviousState()
 {
 	return m_previousState;
 }
 
+MockFixedRepresentation::MockFixedRepresentation() : FixedRepresentation("MockFixedRepresentation")
+{
+}
+
+RigidState& MockFixedRepresentation::getInitialState()
+{
+	return m_initialState;
+}
+
+RigidState& MockFixedRepresentation::getCurrentState()
+{
+	return m_currentState;
+}
+
+RigidState& MockFixedRepresentation::getPreviousState()
+{
+	return m_previousState;
+}
 
 MockDeformableRepresentation::MockDeformableRepresentation(const std::string& name) :
 	SurgSim::Physics::DeformableRepresentation(name)
 {
 	this->m_numDofPerNode = 3;
-	m_F = Vector::LinSpaced(3, 1.0, 3.0);
-	m_M = Matrix::Identity(3, 3);
-	m_D = Matrix::Identity(3, 3);
-	m_K = Matrix::Identity(3, 3);
-}
+	m_f = Vector::LinSpaced(3, 1.0, 3.0);
+	m_M.resize(3, 3);
+	m_M.setIdentity();
 
-RepresentationType MockDeformableRepresentation::getType() const
-{
-	return SurgSim::Physics::REPRESENTATION_TYPE_INVALID;
+	m_D.resize(3, 3);
+	m_D.setIdentity();
+
+	m_K.resize(3, 3);
+	m_K.setIdentity();
 }
 
 void MockDeformableRepresentation::addExternalGeneralizedForce(std::shared_ptr<Localization> localization,
-										 SurgSim::Math::Vector& generalizedForce,
-										 const SurgSim::Math::Matrix& K,
-										 const SurgSim::Math::Matrix& D)
+		const Math::Vector& generalizedForce,
+		const Math::Matrix& K,
+		const Math::Matrix& D)
 {
-	std::shared_ptr<MockDeformableRepresentationLocalization> loc =
-		std::dynamic_pointer_cast<MockDeformableRepresentationLocalization>(localization);
+	std::shared_ptr<MockDeformableLocalization> loc =
+		std::dynamic_pointer_cast<MockDeformableLocalization>(localization);
 
 	m_externalGeneralizedForce.segment<3>(3 * loc->getLocalNode()) += generalizedForce;
-	m_externalGeneralizedStiffness.block<3, 3>(3 * loc->getLocalNode(), 3 * loc->getLocalNode()) += K;
-	m_externalGeneralizedDamping.block<3, 3>(3 * loc->getLocalNode(), 3 * loc->getLocalNode()) += D;
+	Math::addSubMatrix(K, static_cast<SparseMatrix::Index>(loc->getLocalNode()),
+					   static_cast<SparseMatrix::Index>(loc->getLocalNode()),
+					   &m_externalGeneralizedStiffness, true);
+	Math::addSubMatrix(D, static_cast<SparseMatrix::Index>(loc->getLocalNode()),
+					   static_cast<SparseMatrix::Index>(loc->getLocalNode()),
+					   &m_externalGeneralizedDamping, true);
+	m_hasExternalGeneralizedForce = true;
 }
 
-Vector& MockDeformableRepresentation::computeF(const OdeState& state)
+void MockDeformableRepresentation::computeF(const OdeState& state)
 {
-	return m_F;
 }
 
-const Matrix& MockDeformableRepresentation::computeM(const OdeState& state)
+void MockDeformableRepresentation::computeM(const OdeState& state)
 {
-	return m_M;
 }
 
-const Matrix& MockDeformableRepresentation::computeD(const OdeState& state)
+void MockDeformableRepresentation::computeD(const OdeState& state)
 {
-	return m_D;
 }
 
-const Matrix& MockDeformableRepresentation::computeK(const OdeState& state)
+void MockDeformableRepresentation::computeK(const OdeState& state)
 {
-	return m_K;
 }
 
-void MockDeformableRepresentation::computeFMDK(const OdeState& state,
-											   Vector** f,
-											   Matrix** M,
-											   Matrix** D,
-											   Matrix** K)
+void MockDeformableRepresentation::computeFMDK(const OdeState& state)
 {
-	*f = &m_F;
-	*M = &m_M;
-	*D = &m_D;
-	*K = &m_K;
 }
 
 void MockDeformableRepresentation::transformState(std::shared_ptr<OdeState> state, const RigidTransform3d& transform)
@@ -175,8 +187,14 @@ void MockDeformableRepresentation::transformState(std::shared_ptr<OdeState> stat
 MockSpring::MockSpring() : SurgSim::Physics::Spring()
 {
 	m_F = Vector::LinSpaced(6, 1.0, 6.0);
-	m_D = Matrix::Identity(6, 6) * 2.0;
-	m_K = Matrix::Identity(6, 6) * 3.0;
+
+	m_D.resize(6, 6);
+	m_D.setIdentity();
+	m_D *= 2.0;
+
+	m_K.resize(6, 6);
+	m_K.setIdentity();
+	m_K *= 3.0;
 }
 
 void MockSpring::addNode(size_t nodeId)
@@ -189,17 +207,48 @@ void MockSpring::addForce(const OdeState& state, Vector* F, double scale)
 	SurgSim::Math::addSubVector(scale * m_F, m_nodeIds, 3, F);
 }
 
-void MockSpring::addDamping(const OdeState& state, Matrix* D, double scale)
+void MockSpring::addDamping(const OdeState& state, SparseMatrix* D, double scale)
 {
-	SurgSim::Math::addSubMatrix(scale * m_D, m_nodeIds, 3, D);
+	Matrix scaledDense(m_D.rows(), m_D.cols());
+	scaledDense = scale * m_D;
+
+	int index1 = 0;
+	for (auto nodeId1 : m_nodeIds)
+	{
+		int index2 = 0;
+		for (auto nodeId2 : m_nodeIds)
+		{
+			Math::addSubMatrix(scaledDense.block(3 * index1, 3 * index2, 3, 3),
+							   static_cast<SparseMatrix::Index>(nodeId1),
+							   static_cast<SparseMatrix::Index>(nodeId2), D, false);
+			++index2;
+		}
+		++index1;
+	}
 }
 
-void MockSpring::addStiffness(const OdeState& state, Matrix* K, double scale)
+void MockSpring::addStiffness(const SurgSim::Math::OdeState& state, SurgSim::Math::SparseMatrix* K,
+							  double scale)
 {
-	SurgSim::Math::addSubMatrix(scale * m_K, m_nodeIds, 3, K);
+	Matrix scaledDense(m_K.rows(), m_K.cols());
+	scaledDense = scale * m_K;
+
+	int index1 = 0;
+	for (auto nodeId1 : m_nodeIds)
+	{
+		int index2 = 0;
+		for (auto nodeId2 : m_nodeIds)
+		{
+			Math::addSubMatrix(scaledDense.block(3 * index1, 3 * index2, 3, 3),
+							   static_cast<SparseMatrix::Index>(nodeId1),
+							   static_cast<SparseMatrix::Index>(nodeId2), K, false);
+			++index2;
+		}
+		++index1;
+	}
 }
 
-void MockSpring::addFDK(const OdeState& state, Vector* f, Matrix* D, Matrix* K)
+void MockSpring::addFDK(const OdeState& state, Vector* f, SparseMatrix* D, SparseMatrix* K)
 {
 	addForce(state, f);
 	addDamping(state, D);
@@ -221,7 +270,7 @@ MockMassSpring::MockMassSpring(const std::string& name,
 							   double rayleighDampingMass, double rayleighDampingStiffness,
 							   double springStiffness, double springDamping,
 							   SurgSim::Math::IntegrationScheme integrationScheme) :
-							   SurgSim::Physics::MassSpringRepresentation(name)
+	SurgSim::Physics::MassSpringRepresentation(name)
 {
 	using SurgSim::Math::getSubVector;
 	using SurgSim::Math::setSubVector;
@@ -236,7 +285,7 @@ MockMassSpring::MockMassSpring(const std::string& name,
 	state->setNumDof(3, numNodes);
 	for (size_t i = 0; i < numNodes; i++)
 	{
-		Vector3d p(static_cast<double>(i)/static_cast<double>(numNodes), 0, 0);
+		Vector3d p(static_cast<double>(i) / static_cast<double>(numNodes), 0, 0);
 		setSubVector(p, i, 3, &state->getPositions());
 		addMass(std::make_shared<Mass>(totalMass / numNodes));
 	}
@@ -246,12 +295,12 @@ MockMassSpring::MockMassSpring(const std::string& name,
 	}
 	for (size_t i = 0; i < numNodes - 1; i++)
 	{
-		std::shared_ptr<LinearSpring> spring = std::make_shared<LinearSpring>(i, i+1);
+		std::shared_ptr<LinearSpring> spring = std::make_shared<LinearSpring>(i, i + 1);
 		spring->setDamping(springDamping);
 		spring->setStiffness(springStiffness);
 		const Vector3d& xi = getSubVector(state->getPositions(), i, 3);
-		const Vector3d& xj = getSubVector(state->getPositions(), i+1, 3);
-		spring->setRestLength( (xj - xi).norm() );
+		const Vector3d& xj = getSubVector(state->getPositions(), i + 1, 3);
+		spring->setRestLength((xj - xi).norm());
 		addSpring(spring);
 	}
 	setInitialState(state);
@@ -269,10 +318,23 @@ const Vector3d& MockMassSpring::getGravityVector() const
 	return getGravity();
 }
 
-
-MockFemElement::MockFemElement() : FemElement(), m_isInitialized(false)
+void MockMassSpring::clearFMDK()
 {
-	setNumDofPerNode(3);
+	m_f.setZero();
+	SurgSim::Math::clearMatrix(&m_M);
+	SurgSim::Math::clearMatrix(&m_D);
+	SurgSim::Math::clearMatrix(&m_K);
+}
+
+MockFemElement::MockFemElement() : FemElement()
+{
+	initializeMembers();
+}
+
+MockFemElement::MockFemElement(std::shared_ptr<FemElementStructs::FemElementParameter> elementData) : FemElement()
+{
+	initializeMembers();
+	m_nodeIds.assign(elementData->nodeIds.begin(), elementData->nodeIds.end());
 }
 
 void MockFemElement::addNode(size_t nodeId)
@@ -285,61 +347,48 @@ double MockFemElement::getVolume(const OdeState& state) const
 	return 1;
 }
 
-void MockFemElement::addForce(const OdeState& state, Vector* F,    double scale)
+Vector MockFemElement::computeCartesianCoordinate(const OdeState& state, const Vector& barycentricCoordinate) const
 {
-	SurgSim::Math::addSubVector(scale * m_F, m_nodeIds, 3, F);
+	SURGSIM_ASSERT(getNumNodes() == static_cast<size_t>(barycentricCoordinate.size()));
+
+	Vector result = Vector::Zero(3);
+	for (size_t node = 0; node < getNumNodes(); node++)
+	{
+		result += state.getPosition(getNodeId(node)) * barycentricCoordinate[node];
+	}
+
+	return result;
 }
 
-void MockFemElement::addMass(const OdeState& state, Matrix* M, double scale)
-{
-	SurgSim::Math::addSubMatrix(scale * m_M, m_nodeIds, 3, M);
-}
-
-void MockFemElement::addDamping(const OdeState& state, Matrix* D, double scale)
-{
-	SurgSim::Math::addSubMatrix(scale * m_D, m_nodeIds, 3, D);
-}
-
-void MockFemElement::addStiffness(const OdeState& state, Matrix* K, double scale)
-{
-	SurgSim::Math::addSubMatrix(scale * m_K, m_nodeIds, 3, K);
-}
-
-void MockFemElement::addFMDK(const OdeState& state, Vector* f, Matrix* M, Matrix* D, Matrix* K)
-{
-	addForce(state, f);
-	addMass(state, M);
-	addDamping(state, D);
-	addStiffness(state, K);
-}
-
-void MockFemElement::addMatVec(const OdeState& state, double alphaM, double alphaD, double alphaK,
-							   const Vector& x, Vector* F)
-{
-	Vector xLocal(3 * m_nodeIds.size()), fLocal;
-	SurgSim::Math::getSubVector(x, m_nodeIds, 3, &xLocal);
-	fLocal = (alphaM * m_M + alphaD * m_D + alphaK * m_K) * xLocal;
-	SurgSim::Math::addSubVector(fLocal, m_nodeIds, 3, F);
-}
-
-Vector MockFemElement::computeCartesianCoordinate(const OdeState& state, const Vector &barycentricCoordinate) const
+Vector MockFemElement::computeNaturalCoordinate(const OdeState& state, const Vector& globalCoordinate) const
 {
 	return SurgSim::Math::Vector3d::Zero();
 }
 
-Vector MockFemElement::computeNaturalCoordinate(const OdeState& state, const Vector &globalCoordinate) const
+void MockFemElement::initializeMembers()
+{
+	m_isInitialized = false;
+	m_useDamping = true;
+	setNumDofPerNode(3);
+}
+
+void MockFemElement::doUpdateFMDK(const Math::OdeState& state, int options)
 {
-	return SurgSim::Math::Vector3d::Zero();
 }
 
 void MockFemElement::initialize(const OdeState& state)
 {
 	FemElement::initialize(state);
 	const size_t numDof = 3 * m_nodeIds.size();
-	m_F = Vector::LinSpaced(numDof, 1.0, static_cast<double>(numDof));
-	m_M = Matrix::Identity(numDof, numDof) * 1.0;
-	m_D = Matrix::Identity(numDof, numDof) * 2.0;
-	m_K = Matrix::Identity(numDof, numDof) * 3.0;
+	m_f = Vector::LinSpaced(numDof, 1.0, static_cast<double>(numDof));
+	m_M.setIdentity();
+
+	m_D.setIdentity();
+	m_D *= 2.0;
+
+	m_K.setIdentity();
+	m_K *= 3.0;
+
 	m_isInitialized = true;
 }
 
@@ -348,13 +397,8 @@ bool MockFemElement::isInitialized() const
 	return m_isInitialized;
 }
 
-bool InvalidMockFemElement::update(const SurgSim::Math::OdeState& state)
-{
-	return false;
-}
-
-
-MockFemRepresentation::MockFemRepresentation(const std::string& name) : FemRepresentation(name)
+MockFemRepresentation::MockFemRepresentation(const std::string& name)
+	: FemRepresentation(name), m_setInitialStateCalled(false)
 {
 	this->m_numDofPerNode = 3;
 }
@@ -363,20 +407,33 @@ MockFemRepresentation::~MockFemRepresentation()
 {
 }
 
-void MockFemRepresentation::addExternalGeneralizedForce(std::shared_ptr<Localization> localization,
-														SurgSim::Math::Vector& generalizedForce,
-														const SurgSim::Math::Matrix& K,
-														const SurgSim::Math::Matrix& D)
+void MockFemRepresentation::setInitialState(std::shared_ptr<SurgSim::Math::OdeState> initialState)
 {
-	std::shared_ptr<MockDeformableRepresentationLocalization> loc =
-		std::dynamic_pointer_cast<MockDeformableRepresentationLocalization>(localization);
+	FemRepresentation::setInitialState(initialState);
+	m_setInitialStateCalled = true;
+}
+
+void MockFemRepresentation::loadFem(const std::string &filename)
+{
+}
+
+void MockFemRepresentation::addExternalGeneralizedForce(std::shared_ptr<Localization> localization,
+		const SurgSim::Math::Vector& generalizedForce,
+		const SurgSim::Math::Matrix& K,
+		const SurgSim::Math::Matrix& D)
+{
+	std::shared_ptr<MockDeformableLocalization> loc =
+		std::dynamic_pointer_cast<MockDeformableLocalization>(localization);
 
 	size_t numDofPerNode = getNumDofPerNode();
 	m_externalGeneralizedForce.segment(numDofPerNode * loc->getLocalNode(), numDofPerNode) += generalizedForce;
-	m_externalGeneralizedStiffness.block(numDofPerNode * loc->getLocalNode(), numDofPerNode * loc->getLocalNode(),
-		numDofPerNode, numDofPerNode) += K;
-	m_externalGeneralizedDamping.block(numDofPerNode * loc->getLocalNode(), numDofPerNode * loc->getLocalNode(),
-		numDofPerNode, numDofPerNode) += D;
+	SurgSim::Math::addSubMatrix(K, static_cast<SparseMatrix::Index>(loc->getLocalNode()),
+								static_cast<SparseMatrix::Index>(loc->getLocalNode()),
+								&m_externalGeneralizedStiffness, true);
+	SurgSim::Math::addSubMatrix(D, static_cast<SparseMatrix::Index>(loc->getLocalNode()),
+								static_cast<SparseMatrix::Index>(loc->getLocalNode()),
+								&m_externalGeneralizedDamping, true);
+	m_hasExternalGeneralizedForce = true;
 }
 
 std::shared_ptr<FemPlyReaderDelegate> MockFemRepresentation::getDelegate()
@@ -384,59 +441,76 @@ std::shared_ptr<FemPlyReaderDelegate> MockFemRepresentation::getDelegate()
 	return nullptr;
 }
 
-RepresentationType MockFemRepresentation::getType() const
-{
-	return REPRESENTATION_TYPE_INVALID;
-}
-
-std::shared_ptr<OdeSolver> MockFemRepresentation::getOdeSolver() const
-{
-	return this->m_odeSolver;
-}
-
 const std::vector<double>& MockFemRepresentation::getMassPerNode() const
 {
 	return m_massPerNode;
 }
 
+void MockFemRepresentation::clearFMDK()
+{
+	m_f.setZero();
+	SurgSim::Math::clearMatrix(&m_M);
+	SurgSim::Math::clearMatrix(&m_D);
+	SurgSim::Math::clearMatrix(&m_K);
+}
+
 void MockFemRepresentation::transformState(std::shared_ptr<OdeState> state, const RigidTransform3d& transform)
 {
 }
 
+bool MockFemRepresentation::hasSetInitialStateBeenCalled()
+{
+	return m_setInitialStateCalled;
+}
+
+SurgSim::Math::Matrix
+MockFemRepresentationValidComplianceWarping::getNodeTransformation(const SurgSim::Math::OdeState& state, size_t nodeId)
+{
+	return SurgSim::Math::Matrix::Identity(getNumDofPerNode(), getNumDofPerNode());
+}
+
 MockFem1DRepresentation::MockFem1DRepresentation(const std::string& name) : SurgSim::Physics::Fem1DRepresentation(name)
 {
 }
 
-const std::shared_ptr<OdeSolver> MockFem1DRepresentation::getOdeSolver() const
+bool MockFem1DRepresentation::doInitialize()
 {
-	return this->m_odeSolver;
+	return Fem1DRepresentation::doInitialize();
 }
 
+double MockFem1DRepresentation::getMassPerNode(size_t nodeId)
+{
+	return m_massPerNode[nodeId];
+}
 
-MockFixedConstraintBilateral3D::MockFixedConstraintBilateral3D() : ConstraintImplementation()
+MockFem2DRepresentation::MockFem2DRepresentation(const std::string& name) : SurgSim::Physics::Fem2DRepresentation(name)
 {
 }
 
-MockFixedConstraintBilateral3D::~MockFixedConstraintBilateral3D()
+double MockFem2DRepresentation::getMassPerNode(size_t nodeId)
+{
+	return m_massPerNode[nodeId];
+}
+
+MockFixedConstraintFixedPoint::MockFixedConstraintFixedPoint() : ConstraintImplementation()
 {
 }
 
-SurgSim::Math::MlcpConstraintType MockFixedConstraintBilateral3D::getMlcpConstraintType() const
+MockFixedConstraintFixedPoint::~MockFixedConstraintFixedPoint()
 {
-	return SurgSim::Math::MLCP_BILATERAL_3D_CONSTRAINT;
 }
 
-RepresentationType MockFixedConstraintBilateral3D::getRepresentationType() const
+SurgSim::Physics::ConstraintType MockFixedConstraintFixedPoint::getConstraintType() const
 {
-	return REPRESENTATION_TYPE_FIXED;
+	return SurgSim::Physics::FIXED_3DPOINT;
 }
 
-size_t MockFixedConstraintBilateral3D::doGetNumDof() const
+size_t MockFixedConstraintFixedPoint::doGetNumDof() const
 {
 	return 3;
 }
 
-void MockFixedConstraintBilateral3D::doBuild(double dt,
+void MockFixedConstraintFixedPoint::doBuild(double dt,
 											 const ConstraintData& data,
 											 const std::shared_ptr<Localization>& localization,
 											 MlcpPhysicsProblem* mlcp,
@@ -446,31 +520,25 @@ void MockFixedConstraintBilateral3D::doBuild(double dt,
 {
 }
 
-
-MockRigidConstraintBilateral3D::MockRigidConstraintBilateral3D() : ConstraintImplementation()
+MockRigidConstraintFixedPoint::MockRigidConstraintFixedPoint() : ConstraintImplementation()
 {
 }
 
-MockRigidConstraintBilateral3D::~MockRigidConstraintBilateral3D()
+MockRigidConstraintFixedPoint::~MockRigidConstraintFixedPoint()
 {
 }
 
-SurgSim::Math::MlcpConstraintType MockRigidConstraintBilateral3D::getMlcpConstraintType() const
+SurgSim::Physics::ConstraintType MockRigidConstraintFixedPoint::getConstraintType() const
 {
-	return SurgSim::Math::MLCP_BILATERAL_3D_CONSTRAINT;
+	return SurgSim::Physics::FIXED_3DPOINT;
 }
 
-RepresentationType MockRigidConstraintBilateral3D::getRepresentationType() const
-{
-	return REPRESENTATION_TYPE_RIGID;
-}
-
-size_t MockRigidConstraintBilateral3D::doGetNumDof() const
+size_t MockRigidConstraintFixedPoint::doGetNumDof() const
 {
 	return 3;
 }
 
-void MockRigidConstraintBilateral3D::doBuild(double dt,
+void MockRigidConstraintFixedPoint::doBuild(double dt,
 											 const ConstraintData& data,
 											 const std::shared_ptr<Localization>& localization,
 											 MlcpPhysicsProblem* mlcp,
@@ -489,20 +557,19 @@ MockLocalization::MockLocalization(std::shared_ptr<Representation> representatio
 {
 }
 
-Vector3d MockLocalization::doCalculatePosition(double time)
+Vector3d MockLocalization::doCalculatePosition(double time) const
 {
 	return SurgSim::Math::Vector3d::Zero();
 }
 
-
-SurgSim::Math::MlcpConstraintType MockConstraintImplementation::getMlcpConstraintType() const
+Vector3d MockLocalization::doCalculateVelocity(double time) const
 {
-	return SurgSim::Math::MLCP_BILATERAL_3D_CONSTRAINT;
+	return SurgSim::Math::Vector3d::Zero();
 }
 
-RepresentationType MockConstraintImplementation::getRepresentationType() const
+SurgSim::Physics::ConstraintType MockConstraintImplementation::getConstraintType() const
 {
-	return SurgSim::Physics::REPRESENTATION_TYPE_FIXED;
+	return SurgSim::Physics::FIXED_3DPOINT;
 }
 
 size_t MockConstraintImplementation::doGetNumDof() const
@@ -511,12 +578,12 @@ size_t MockConstraintImplementation::doGetNumDof() const
 }
 
 void MockConstraintImplementation::doBuild(double dt,
-										   const ConstraintData& data,
-										   const std::shared_ptr<Localization>& localization,
-										   MlcpPhysicsProblem* mlcp,
-										   size_t indexOfRepresentation,
-										   size_t indexOfConstraint,
-										   ConstraintSideSign sign)
+		const ConstraintData& data,
+		const std::shared_ptr<Localization>& localization,
+		MlcpPhysicsProblem* mlcp,
+		size_t indexOfRepresentation,
+		size_t indexOfConstraint,
+		ConstraintSideSign sign)
 {
 }
 
@@ -571,7 +638,7 @@ void MockVirtualToolCoupler::setOptionalAngularDamping(const SurgSim::DataStruct
 }
 
 void MockVirtualToolCoupler::setOptionalAttachmentPoint(
-		const SurgSim::DataStructures::OptionalValue<SurgSim::Math::Vector3d>& val)
+	const SurgSim::DataStructures::OptionalValue<SurgSim::Math::Vector3d>& val)
 {
 	VirtualToolCoupler::setOptionalAttachmentPoint(val);
 }
@@ -581,5 +648,39 @@ const SurgSim::DataStructures::DataGroup& MockVirtualToolCoupler::getOutputData(
 	return m_outputData;
 }
 
+MockCollisionRepresentation::MockCollisionRepresentation(const std::string& name)
+	: SurgSim::Collision::Representation(name), m_numberOfTimesUpdateCalled(0)
+{}
+
+int MockCollisionRepresentation::getShapeType() const
+{
+	return -1;
+}
+
+const std::shared_ptr<SurgSim::Math::Shape> MockCollisionRepresentation::getShape() const
+{
+	return nullptr;
+}
+
+void MockCollisionRepresentation::update(const double& dt)
+{
+	++m_numberOfTimesUpdateCalled;
+}
+
+/// \return The number of times update method has been invoked.
+int MockCollisionRepresentation::getNumberOfTimesUpdateCalled() const
+{
+	return m_numberOfTimesUpdateCalled;
+}
+
+MockComputation::MockComputation(bool doCopyState) : Computation(doCopyState)
+{
+}
+
+std::shared_ptr<PhysicsManagerState> MockComputation::doUpdate(const double& dt,
+		const std::shared_ptr<PhysicsManagerState>& state)
+{
+	return state;
+}
 }; // Physics
 }; // SurgSim
