@@ -16,6 +16,7 @@
 #ifndef SURGSIM_PHYSICS_UPDATECOLLISIONREPRESENTATIONS_H
 #define SURGSIM_PHYSICS_UPDATECOLLISIONREPRESENTATIONS_H
 
+#include "SurgSim/Framework/Macros.h"
 #include "SurgSim/Physics/Computation.h"
 
 namespace SurgSim
@@ -31,11 +32,13 @@ public:
 	/// \param doCopyState whether to copy the PhysicsManagerState on update
 	explicit UpdateCollisionRepresentations(bool doCopyState);
 
+	SURGSIM_CLASSNAME(SurgSim::Physics::UpdateCollisionRepresentations);
+
 	/// Destructor
 	virtual ~UpdateCollisionRepresentations();
 
-	virtual std::shared_ptr<PhysicsManagerState> doUpdate(
-		const double& dt, const std::shared_ptr<PhysicsManagerState>& state) override;
+	std::shared_ptr<PhysicsManagerState> doUpdate(const double& dt, const std::shared_ptr<PhysicsManagerState>& state)
+		override;
 
 };
 
