@@ -15,10 +15,11 @@
 
 #include <Eigen/Eigenvalues>
 
+#include "SurgSim/Collision/CollisionPair.h"
+#include "SurgSim/Collision/Representation.h"
 #include "SurgSim/DataStructures/DataGroupBuilder.h"
 #include "SurgSim/DataStructures/DataStructuresConvert.h"
 #include "SurgSim/Framework/FrameworkConvert.h"
-#include "SurgSim/Framework/LogMacros.h"
 #include "SurgSim/Input/InputComponent.h"
 #include "SurgSim/Input/OutputComponent.h"
 #include "SurgSim/Math/MathConvert.h"
@@ -27,7 +28,6 @@
 #include "SurgSim/Math/RigidTransform.h"
 #include "SurgSim/Math/Vector.h"
 #include "SurgSim/Physics/RigidRepresentation.h"
-#include "SurgSim/Physics/RigidRepresentationLocalization.h"
 #include "SurgSim/Physics/VirtualToolCoupler.h"
 
 using SurgSim::Math::makeSkewSymmetricMatrix;
@@ -37,6 +37,7 @@ using SurgSim::Math::Matrix33d;
 using SurgSim::Math::Matrix66d;
 using SurgSim::Math::RigidTransform3d;
 using SurgSim::Math::Quaterniond;
+using SurgSim::Framework::checkAndConvert;
 
 namespace SurgSim
 {
@@ -53,8 +54,11 @@ VirtualToolCoupler::VirtualToolCoupler(const std::string& name) :
 	m_angularStiffness(std::numeric_limits<double>::quiet_NaN()),
 	m_angularDamping(std::numeric_limits<double>::quiet_NaN()),
 	m_localAttachmentPoint(Vector3d(std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN(),
-		std::numeric_limits<double>::quiet_NaN())),
+									std::numeric_limits<double>::quiet_NaN())),
 	m_calculateInertialTorques(false),
+	m_logger(SurgSim::Framework::Logger::getLogger("Physics/VirtualToolCoupler")),
+	m_hapticOutputOnlyWhenColliding(false),
+	m_previousInputPose(Math::RigidTransform3d::Identity()),
 	m_poseIndex(-1),
 	m_linearVelocityIndex(-1),
 	m_angularVelocityIndex(-1),
@@ -67,25 +71,27 @@ VirtualToolCoupler::VirtualToolCoupler(const std::string& name) :
 	m_damperJacobianIndex(-1)
 {
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, SurgSim::DataStructures::OptionalValue<double>,
-		LinearStiffness, getOptionalLinearStiffness, setOptionalLinearStiffness);
+									  LinearStiffness, getOptionalLinearStiffness, setOptionalLinearStiffness);
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, SurgSim::DataStructures::OptionalValue<double>,
-		LinearDamping, getOptionalLinearDamping, setOptionalLinearDamping);
+									  LinearDamping, getOptionalLinearDamping, setOptionalLinearDamping);
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, SurgSim::DataStructures::OptionalValue<double>,
-		AngularStiffness, getOptionalAngularStiffness, setOptionalAngularStiffness);
+									  AngularStiffness, getOptionalAngularStiffness, setOptionalAngularStiffness);
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, SurgSim::DataStructures::OptionalValue<double>,
-		AngularDamping, getOptionalAngularDamping, setOptionalAngularDamping);
+									  AngularDamping, getOptionalAngularDamping, setOptionalAngularDamping);
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler,
-			SurgSim::DataStructures::OptionalValue<SurgSim::Math::Vector3d>, AttachmentPoint,
-			getOptionalAttachmentPoint, setOptionalAttachmentPoint);
+									  SurgSim::DataStructures::OptionalValue<SurgSim::Math::Vector3d>, AttachmentPoint,
+									  getOptionalAttachmentPoint, setOptionalAttachmentPoint);
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, bool, CalculateInertialTorques,
-		getCalculateInertialTorques, setCalculateInertialTorques);
+									  getCalculateInertialTorques, setCalculateInertialTorques);
+	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, bool, HapticOutputOnlyWhenColliding,
+									  isHapticOutputOnlyWhenColliding, setHapticOutputOnlyWhenColliding);
 
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, std::shared_ptr<SurgSim::Framework::Component>,
-		Input, getInput, setInput);
+									  Input, getInput, setInput);
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, std::shared_ptr<SurgSim::Framework::Component>,
-		Output, getOutput, setOutput);
+									  Output, getOutput, setOutput);
 	SURGSIM_ADD_SERIALIZABLE_PROPERTY(VirtualToolCoupler, std::shared_ptr<SurgSim::Framework::Component>,
-		Representation, getRepresentation, setRepresentation);
+									  Representation, getRepresentation, setRepresentation);
 }
 
 VirtualToolCoupler::~VirtualToolCoupler()
@@ -99,7 +105,7 @@ const std::shared_ptr<SurgSim::Framework::Component> VirtualToolCoupler::getInpu
 
 void VirtualToolCoupler::setInput(const std::shared_ptr<SurgSim::Framework::Component> input)
 {
-	m_input = std::dynamic_pointer_cast<SurgSim::Input::InputComponent>(input);
+	m_input = checkAndConvert<SurgSim::Input::InputComponent>(input, "SurgSim::Input::InputComponent");
 }
 
 const std::shared_ptr<SurgSim::Framework::Component> VirtualToolCoupler::getOutput()
@@ -109,7 +115,7 @@ const std::shared_ptr<SurgSim::Framework::Component> VirtualToolCoupler::getOutp
 
 void VirtualToolCoupler::setOutput(const std::shared_ptr<SurgSim::Framework::Component> output)
 {
-	m_output = std::dynamic_pointer_cast<SurgSim::Input::OutputComponent>(output);
+	m_output = checkAndConvert<SurgSim::Input::OutputComponent>(output, "SurgSim::Input::OutputComponent");
 }
 
 const std::shared_ptr<SurgSim::Framework::Component> VirtualToolCoupler::getRepresentation()
@@ -134,23 +140,49 @@ void VirtualToolCoupler::setPoseName(const std::string& poseName)
 
 void VirtualToolCoupler::update(double dt)
 {
-	SurgSim::DataStructures::DataGroup inputData;
+	DataStructures::DataGroup inputData;
 	m_input->getData(&inputData);
-	RigidTransform3d inputPose;
 	inputData.poses().cacheIndex(m_poseName, &m_poseIndex);
+	inputData.vectors().cacheIndex(DataStructures::Names::LINEAR_VELOCITY, &m_linearVelocityIndex);
+	if (m_linearVelocityIndex < 0)
+	{
+		SURGSIM_LOG_ONCE(m_logger, WARNING) << getFullName() << " is receiving an input DataGroup that does not " <<
+			"contain a linear velocity vector named " << DataStructures::Names::LINEAR_VELOCITY <<
+			", so it will be estimated.";
+	}
+	inputData.vectors().cacheIndex(DataStructures::Names::ANGULAR_VELOCITY, &m_angularVelocityIndex);
+	if (m_angularVelocityIndex < 0)
+	{
+		SURGSIM_LOG_ONCE(m_logger, WARNING) << getFullName() << " is receiving an input DataGroup that does not " <<
+			"contain an angular velocity vector named " << DataStructures::Names::ANGULAR_VELOCITY <<
+			") so it will be estimated.";
+	}
+
+	RigidTransform3d inputPose;
 	if (inputData.poses().get(m_poseIndex, &inputPose))
 	{
-		// TODO(ryanbeasley): If the RigidRepresentation is not colliding, we should turn off the VTC forces and set the
-		// RigidRepresentation's state to the input state.
-		Vector3d inputLinearVelocity, inputAngularVelocity;
-		inputLinearVelocity.setZero();
-		inputData.vectors().cacheIndex(SurgSim::DataStructures::Names::LINEAR_VELOCITY, &m_linearVelocityIndex);
-		inputData.vectors().get(m_linearVelocityIndex, &inputLinearVelocity);
-		inputAngularVelocity.setZero();
-		inputData.vectors().cacheIndex(SurgSim::DataStructures::Names::ANGULAR_VELOCITY, &m_angularVelocityIndex);
-		inputData.vectors().get(m_angularVelocityIndex, &inputAngularVelocity);
+		Vector3d inputLinearVelocity(Vector3d::Zero());
+		const bool gotInputLinearVelocity = inputData.vectors().get(m_linearVelocityIndex, &inputLinearVelocity);
+		if (!gotInputLinearVelocity)
+		{
+			inputLinearVelocity = (inputPose.translation() - m_previousInputPose.translation()) / dt;
+		}
 
-		RigidRepresentationState objectState(m_rigid->getCurrentState());
+		Vector3d inputAngularVelocity(Vector3d::Zero());
+		const bool gotInputAngularVelocity = inputData.vectors().get(m_angularVelocityIndex, &inputAngularVelocity);
+		if (!gotInputAngularVelocity)
+		{
+			Math::computeRotationVector(inputPose, m_previousInputPose, &inputAngularVelocity);
+			inputAngularVelocity /= dt;
+		}
+		m_previousInputPose = inputPose;
+
+		RigidTransform3d inputAlignment = m_input->getLocalPose();
+		inputPose = inputAlignment * inputPose;
+		inputLinearVelocity = inputAlignment.linear() * inputLinearVelocity;
+		inputAngularVelocity = inputAlignment.rotation() * inputAngularVelocity;
+
+		RigidState objectState(m_rigid->getCurrentState());
 		RigidTransform3d objectPose(objectState.getPose());
 		Vector3d objectPosition = objectPose * m_rigid->getMassCenter();
 		Vector3d attachmentPoint = objectPose * m_localAttachmentPoint;
@@ -166,20 +198,17 @@ void VirtualToolCoupler::update(double dt)
 		Vector3d torque = m_angularStiffness * rotationVector;
 		torque += m_angularDamping * (inputAngularVelocity - objectState.getAngularVelocity());
 
+		Vector6d generalizedForce;
+		generalizedForce << force, torque;
+
 		const Matrix33d identity3x3 = Matrix33d::Identity();
 		const Matrix33d zero3x3 = Matrix33d::Zero();
-		const Matrix33d linearStiffnessMatrix = m_linearStiffness * identity3x3;
-		const Matrix33d linearDampingMatrix = m_linearDamping * identity3x3;
-		const Matrix33d angularStiffnessMatrix = m_angularStiffness * identity3x3;
-		const Matrix33d angularDampingMatrix = m_angularDamping * identity3x3;
-
-		Vector6d generalizedForce;
-		Matrix66d generalizedStiffness, generalizedDamping;
-		generalizedForce << force, torque;
-		generalizedStiffness << linearStiffnessMatrix, zero3x3,
-								zero3x3, angularStiffnessMatrix;
-		generalizedDamping << linearDampingMatrix, zero3x3,
-							  zero3x3, angularDampingMatrix;
+		Matrix66d generalizedStiffness;
+		generalizedStiffness << m_linearStiffness * identity3x3, zero3x3,
+								zero3x3                        , m_angularStiffness * identity3x3;
+		Matrix66d generalizedDamping;
+		generalizedDamping << m_linearDamping * identity3x3, zero3x3,
+							  zero3x3                      , m_angularDamping * identity3x3;
 
 		if (m_calculateInertialTorques)
 		{
@@ -194,16 +223,57 @@ void VirtualToolCoupler::update(double dt)
 
 		if (m_output != nullptr)
 		{
-			m_outputData.vectors().set(m_forceIndex, -generalizedForce.segment<3>(0));
-			m_outputData.vectors().set(m_torqueIndex, -generalizedForce.segment<3>(3));
-			m_outputData.vectors().set(m_inputLinearVelocityIndex, inputLinearVelocity);
-			m_outputData.vectors().set(m_inputAngularVelocityIndex, inputAngularVelocity);
+			RigidTransform3d outputAlignment = m_output->getLocalPose().inverse();
+			Matrix33d outputAlignmentUnScaled = outputAlignment.rotation();
 
-			m_outputData.poses().set(m_inputPoseIndex, inputPose);
+			bool output = true;
+			if (m_hapticOutputOnlyWhenColliding)
+			{
+				auto collision = m_rigid->getCollisionRepresentation();
+				output = (collision == nullptr) || (collision->getCollisions().unsafeGet().size() > 0);
+			}
 
-			m_outputData.matrices().set(m_springJacobianIndex, -generalizedStiffness);
-			m_outputData.matrices().set(m_damperJacobianIndex, -generalizedDamping);
+			if (output)
+			{
+				m_outputData.vectors().set(m_forceIndex, outputAlignmentUnScaled * (-force));
+				m_outputData.vectors().set(m_torqueIndex, outputAlignmentUnScaled * (-torque));
+				if (gotInputLinearVelocity && gotInputAngularVelocity)
+				{
+					m_outputData.matrices().set(m_springJacobianIndex, -generalizedStiffness);
+					m_outputData.matrices().set(m_damperJacobianIndex, -generalizedDamping);
+				}
+				else
+				{
+					m_outputData.matrices().reset(m_springJacobianIndex);
+					m_outputData.matrices().reset(m_damperJacobianIndex);
+				}
+			}
+			else
+			{
+				m_outputData.vectors().set(m_forceIndex, Vector3d::Zero());
+				m_outputData.vectors().set(m_torqueIndex, Vector3d::Zero());
+				m_outputData.matrices().reset(m_springJacobianIndex);
+				m_outputData.matrices().reset(m_damperJacobianIndex);
+			}
 
+			if (gotInputLinearVelocity)
+			{
+				m_outputData.vectors().set(m_inputLinearVelocityIndex, outputAlignment.linear() * inputLinearVelocity);
+			}
+			else
+			{
+				m_outputData.vectors().reset(m_inputLinearVelocityIndex);
+			}
+
+			if (gotInputAngularVelocity)
+			{
+				m_outputData.vectors().set(m_inputAngularVelocityIndex, outputAlignmentUnScaled * inputAngularVelocity);
+			}
+			else
+			{
+				m_outputData.vectors().reset(m_inputAngularVelocityIndex);
+			}
+			m_outputData.poses().set(m_inputPoseIndex, outputAlignment * inputPose);
 			m_output->setData(m_outputData);
 		}
 	}
@@ -242,14 +312,13 @@ bool VirtualToolCoupler::doWakeUp()
 {
 	if (m_input == nullptr)
 	{
-		SURGSIM_LOG_SEVERE(SurgSim::Framework::Logger::getDefaultLogger()) << "VirtualToolCoupler named " <<
-			getName() << " does not have an Input Component.";
+		SURGSIM_LOG_SEVERE(m_logger) <<
+			"VirtualToolCoupler named " << getName() << " does not have an Input Component.";
 		return false;
 	}
 	if (m_rigid == nullptr)
 	{
-		SURGSIM_LOG_SEVERE(SurgSim::Framework::Logger::getDefaultLogger()) << "VirtualToolCoupler named " <<
-			getName() << " does not have a Representation.";
+		SURGSIM_LOG_SEVERE(m_logger) << "VirtualToolCoupler named " << getName() << " does not have a Representation.";
 		return false;
 	}
 
@@ -324,7 +393,10 @@ bool VirtualToolCoupler::doWakeUp()
 		}
 	}
 
-
+	SURGSIM_LOG_INFO(m_logger) <<
+		"VirtualToolCoupler named '" << getName() << "' has linear stiffness " << m_linearStiffness <<
+		", linear damping " << m_linearDamping << ", angular stiffness " << m_angularStiffness <<
+		", and angular damping " << m_angularDamping << ". Its Physics Representation has mass " << m_rigid->getMass();
 	return true;
 }
 
@@ -344,7 +416,7 @@ void VirtualToolCoupler::overrideLinearStiffness(double linearStiffness)
 double VirtualToolCoupler::getLinearStiffness()
 {
 	SURGSIM_ASSERT(isAwake() || m_optionalLinearStiffness.hasValue())
-		<< "Vtc parameter has not been initialized";
+			<< "Vtc parameter has not been initialized";
 
 	return m_linearStiffness;
 }
@@ -360,7 +432,7 @@ void VirtualToolCoupler::overrideLinearDamping(double linearDamping)
 double VirtualToolCoupler::getLinearDamping()
 {
 	SURGSIM_ASSERT(isAwake() || m_optionalLinearDamping.hasValue())
-		<< "Vtc parameter has not been initialized";
+			<< "Vtc parameter has not been initialized";
 
 	return m_linearDamping;
 }
@@ -376,7 +448,7 @@ void VirtualToolCoupler::overrideAngularStiffness(double angularStiffness)
 double VirtualToolCoupler::getAngularStiffness()
 {
 	SURGSIM_ASSERT(isAwake() || m_optionalAngularStiffness.hasValue())
-		<< "Vtc parameter has not been initialized";
+			<< "Vtc parameter has not been initialized";
 
 	return m_angularStiffness;
 }
@@ -392,7 +464,7 @@ void VirtualToolCoupler::overrideAngularDamping(double angularDamping)
 double VirtualToolCoupler::getAngularDamping()
 {
 	SURGSIM_ASSERT(isAwake() || m_optionalAngularDamping.hasValue())
-		<< "Vtc parameter has not been initialized";
+			<< "Vtc parameter has not been initialized";
 
 	return m_angularDamping;
 }
@@ -408,7 +480,7 @@ void VirtualToolCoupler::overrideAttachmentPoint(const SurgSim::Math::Vector3d&
 const SurgSim::Math::Vector3d& VirtualToolCoupler::getAttachmentPoint()
 {
 	SURGSIM_ASSERT(isAwake() || m_optionalAttachmentPoint.hasValue())
-		<< "Vtc parameter has not been initialized";
+			<< "Vtc parameter has not been initialized";
 	return m_localAttachmentPoint;
 }
 
@@ -477,5 +549,27 @@ bool VirtualToolCoupler::getCalculateInertialTorques() const
 	return m_calculateInertialTorques;
 }
 
+void VirtualToolCoupler::doRetire()
+{
+	m_outputData.resetAll();
+	Behavior::doRetire();
+}
+
+void VirtualToolCoupler::setLocalActive(bool val)
+{
+	m_outputData.resetAll();
+	Behavior::setLocalActive(val);
+}
+
+bool VirtualToolCoupler::isHapticOutputOnlyWhenColliding() const
+{
+	return m_hapticOutputOnlyWhenColliding;
+}
+
+void VirtualToolCoupler::setHapticOutputOnlyWhenColliding(bool hapticOutputOnlyWhenColliding)
+{
+	m_hapticOutputOnlyWhenColliding = hapticOutputOnlyWhenColliding;
+}
+
 }; /// Physics
 }; /// SurgSim
